@@ -2,7 +2,6 @@
 #
 # mhn.find.sh -- check if a particular command is available
 #
-# $Id: mhn.find.sh,v 1.1.1.1 1999/04/30 18:08:35 doug Exp $
 
 if test -z "$2"; then
     echo "usage: mhn.find.sh search-path program" 1>&2
@@ -23,18 +22,10 @@ for A in $SEARCHPATH; do
 	continue
     fi
 
-    # if program was found in /usr/local/bin, then
-    # just echo program name, else echo full pathname
+    # if program was found then echo full pathname
     if test -f "$A/$PROGRAM"; then
-	if test "$A" = "/usr/local/bin"; then
-	    PGM="$PROGRAM"
-	else
-	    PGM="$A/$PROGRAM"
-	fi
-
-	echo "$PGM"
+	echo "$A/$PROGRAM"
 	exit 0
     fi
 done
 IFS="$oIFS"
-
