@@ -2,10 +2,6 @@
  *           -- (mhparse.c/mhbuildsbr.c)
  */
 
-#define	NPARTS	50
-#define	NTYPES	20
-#define	NPREFS	20
-
 /*
  * Abstract type for header fields
  */
@@ -38,7 +34,6 @@ typedef unsigned long (*SizeCEFunc) (CT);
 struct hfield {
     char *name;		/* field name */
     char *value;	/* field body */
-    int hf_encoding;	/* internal flag for transfer encoding to use */
     HF next;		/* link to next header field */
 };
 
@@ -144,10 +139,6 @@ struct Content {
     int	c_encoding;		/* internal flag for encoding type   */
     int c_reqencoding;		/* Requested encoding (by mhbuild)   */
 
-    /* Content-MD5 info */
-    int	c_digested;		/* have we seen this header before?  */
-    unsigned char c_digest[16];	/* decoded MD5 checksum              */
-
     /* pointers to content-specific structures */
     void *c_ctparams;		/* content type specific data        */
     struct exbody *c_ctexbody;	/* data for type message/external    */
@@ -251,17 +242,7 @@ struct multipart {
 /* Flags for subtypes of MESSAGE */
 #define	MESSAGE_UNKNOWN	 0x00
 #define	MESSAGE_RFC822	 0x01
-#define	MESSAGE_PARTIAL	 0x02
-#define	MESSAGE_EXTERNAL 0x03
-
-/* Structure for message/partial */
-struct partial {
-    char *pm_partid;
-    int	pm_partno;
-    int	pm_maxno;
-    int	pm_marked;
-    int	pm_stored;
-};
+#define	MESSAGE_EXTERNAL 0x02
 
 /* Structure for message/external */
 struct exbody {
@@ -323,105 +304,22 @@ extern struct str2init str2methods[];
  */
 CT parse_mime (char *);
 
-/*
- * Translate a composition file into a MIME data structure.  Arguments are:
- *
- * infile	- Name of input filename
- * autobuild    - A flag to indicate if the composition file parser is
- *		  being run in automatic mode or not.  In auto mode,
- *		  if a MIME-Version header is encountered it is assumed
- *		  that the composition file is already in MIME format
- *		  and will not be processed further.  Otherwise, an
- *		  error is generated.
- * dist		- A flag to indicate if we are being run by "dist".  In
- *		  that case, add no MIME headers to the message.  Existing
- *		  headers will still be encoded by RFC 2047.
- * directives	- A flag to control whether or not build directives are
- *		  processed by default.
- * encoding	- The default encoding to use when doing RFC 2047 header
- *		  encoding.  Must be one of CE_UNKNOWN, CE_BASE64, or
- *		  CE_QUOTED.
- * maxunencoded	- The maximum line length before the default encoding for
- *		  text parts is quoted-printable.
- * verbose	- If 1, output verbose information during message composition
- *
- * Returns a CT structure describing the resulting MIME message.  If the
- * -auto flag is set and a MIME-Version header is encountered, the return
- * value is NULL.
- */
-CT build_mime (char *infile, int autobuild, int dist, int directives,
-	       int encoding, size_t maxunencoded, int verbose);
-
 int add_header (CT, char *, char *);
+bool remove_header (CT, const char *);
 int get_ctinfo (char *, CT, int);
 int params_external (CT, int);
 int open7Bit (CT, char **);
 void close_encoding (CT);
-char *ct_type_str (int);
-char *ct_subtype_str (int, int);
-int ct_str_type (const char *);
-int ct_str_subtype (int, const char *);
-const struct str2init *get_ct_init (int);
-const char *ce_str (int);
-const struct str2init *get_ce_method (const char *);
+char *ct_type_str (int) CONST;
+char *ct_subtype_str (int, int) CONST;
+int ct_str_type (const char *) PURE;
+int ct_str_subtype (int, const char *) PURE;
+const struct str2init *get_ct_init (int) PURE;
+const char *ce_str (int) CONST;
+const struct str2init *get_ce_method (const char *) PURE;
 char *content_charset (CT);
 void reverse_alternative_parts (CT);
 
-/*
- * Given a content structure, return true if the content has a disposition
- * of "inline".
- *
- * Arguments are:
- *
- * ct		- Content structure to examine
- */
-int is_inline(CT ct);
-
-/*
- * Given a list of messages, display information about them on standard
- * output.
- *
- * Arguments are:
- *
- * cts		- An array of CT elements of messages that need to be
- *		  displayed.  Array is terminated by a NULL.
- * headsw	- If 1, display a column header.
- * sizesw	- If 1, display the size of the part.
- * verbosw	- If 1, display verbose information
- * debugsw	- If 1, turn on debugging for the output.
- * disposw	- If 1, display MIME part disposition information.
- *
- */
-void list_all_messages(CT *cts, int headsw, int sizesw, int verbosw,
-		       int debugsw, int disposw);
-
-/*
- * List the content information of a single MIME part on stdout.
- *
- * Arguments are:
- *
- * ct		- MIME Content structure to display.
- * toplevel	- If set, we're at the top level of a message
- * realsize	- If set, determine the real size of the content
- * verbose	- If set, output verbose information
- * debug	- If set, turn on debugging for the output
- * dispo	- If set, display MIME part disposition information.
- *
- * Returns OK on success, NOTOK otherwise.
- */
-int list_content(CT ct, int toplevel, int realsize, int verbose, int debug,
-		 int dispo);
-
-/*
- * Display content-appropriate information on MIME parts, descending recursively
- * into multipart content if appropriate.  Uses list_content() for displaying
- * generic information.
- *
- * Arguments and return value are the same as list_content().
- */
-int list_switch(CT ct, int toplevel, int realsize, int verbose, int debug,
-		int dispo);
-
 /*
  * Given a linked list of parameters, build an output string for them.  This
  * string is designed to be concatenated on an already-built header.
@@ -521,14 +419,17 @@ char *get_param(PM first, const char *name, char replace, int fetchonly);
  */
 char *get_param_value(PM pm, char replace);
 
-extern int checksw;	/* Add Content-MD5 field */
+extern bool skip_mp_cte_check;
+extern bool suppress_bogus_mp_content_warning;
+extern bool bogus_mp_content;
+extern bool suppress_extraneous_trailing_semicolon_warning;
 
-/*
- * mhstore
- * Put it here because it uses the CT typedef.
- */
-typedef struct mhstoreinfo *mhstoreinfo_t;
-mhstoreinfo_t mhstoreinfo_create(CT *, char *, const char *, int, int);
-int mhstoreinfo_files_not_clobbered(const mhstoreinfo_t);
-void mhstoreinfo_free(mhstoreinfo_t);
-void store_all_messages (mhstoreinfo_t);
+extern bool suppress_multiple_mime_version_warning;
+
+#define	NPREFS	20
+typedef struct {
+    char *type;
+    char *subtype;
+} mime_type_subtype;
+extern mime_type_subtype mime_preference[NPREFS];
+extern int npreferred;
