@@ -2,7 +2,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <unistd.h>
@@ -10,7 +10,7 @@
 #include <ctype.h>
 #ifndef NDEBUG
   /* See etc/gen-ctype-checked.c. */
-# include <sbr/ctype-checked.h>
+#include "sbr/ctype-checked.h"
 #endif
 #include <assert.h>
 
@@ -30,6 +30,9 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 
 #ifdef HAVE_SYS_PARAM_H
 # include <sys/param.h>
@@ -77,10 +80,3 @@
 #ifndef HAVE_GETLINE
 ssize_t getline(char **lineptr, size_t *n, FILE *stream);
 #endif
-
-/*
- * Defaults for programs if they aren't configured in a user's profile
- */
-
-#define DEFAULT_PAGER "more"
-#define DEFAULT_EDITOR "vi"
