@@ -1,21 +1,18 @@
-
-/*
- * atooi.c -- octal version of atoi()
- *
- * $Id: atooi.c,v 1.2 2002/07/02 22:09:14 kenh Exp $
+/* atooi.c -- octal version of atoi()
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "atooi.h"
 
 
 int
 atooi(char *cp)
 {
-    register int i, base;
+    int i, base;
 
     i = 0;
     base = 8;
