@@ -6,8 +6,9 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "concat.h"
+#include "utils.h"
 
 
 /* concat returns a non-NULL malloc'd pointer to the catenation of the
