@@ -1,15 +1,15 @@
-
-/*
- * context_del.c -- delete an entry from the context/profile list
- *
- * $Id: context_del.c,v 1.4 2006/03/08 12:14:16 bress Exp $
+/* context_del.c -- delete an entry from the context/profile list
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "context_del.h"
+#include "error.h"
+#include "utils.h"
+#include "globals.h"
 
 /*
  * Delete a key/value pair from the context/profile list.
@@ -19,21 +19,21 @@
 int
 context_del (char *key)
 {
-    register struct node *np, *pp;
+    struct node *np, *pp;
 
+    key = FENDNULL(key);
     for (np = m_defs, pp = NULL; np; pp = np, np = np->n_next) {
-	if (!mh_strcasecmp (np->n_name, key)) {
+	if (!strcasecmp(FENDNULL(np->n_name), key)) {
 	    if (!np->n_context)
-		admonish (NULL, "bug: context_del(key=\"%s\")", np->n_name);
+		inform("bug: context_del(key=\"%s\"), continuing...", np->n_name);
 	    if (pp)
 		pp->n_next = np->n_next;
 	    else
 		m_defs = np->n_next;
 	    free (np->n_name);
-	    if (np->n_field)
-		free (np->n_field);
-	    free ((char *) np);
-	    ctxflags |= CTXMOD;
+            free(np->n_field);
+	    free(np);
+            context_dirty = true;
 	    return 0;
 	}
     }
