@@ -1,5 +1,4 @@
-/*
- * context_read.c -- find and read profile and context files
+/* context_read.c -- find and read profile and context files
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -22,8 +21,19 @@
  *	    environment variable or by the profile.
  */
 
-#include <h/mh.h>				/* mh internals */
-#include <pwd.h>				/* structure for getpwuid() results */
+#include "h/mh.h"
+#include "read_yes_or_no_if_tty.h"
+#include "concat.h"
+#include "context_read.h"
+#include "context_find.h"
+#include "readconfig.h"
+#include "path.h"
+#include "error.h"
+#include "lock_file.h"
+#include "m_maildir.h"
+#include "makedir.h"
+#include "utils.h"
+#include "globals.h"
 
 void
 context_read (void)
@@ -32,28 +42,17 @@ context_read (void)
     char			*cp;		/* miscellaneous pointer */
     char			*nd;		/* nmh directory pointer */
     struct	stat		st;		/* stat() results */
-    register	struct	passwd	*pw;		/* getpwuid() results */
-    register	FILE		*ib;		/* profile and context file pointer */
+    FILE			*ib;		/* profile and context file pointer */
     int failed_to_lock = 0;
 
     /*
-     *  If this routine _is_ called again (despite the wanings in the
+     *  If this routine _is_ called again (despite the warnings in the
      *  comments above), return immediately.
      */
     if ( m_defs != 0 )
         return;
 
-    /*
-     *	Find user's home directory.  Try the HOME environment variable first,
-     *	the home directory field in the password file if that's not found.
-     */
-
-    if ((mypath = getenv("HOME")) == (char *)0) {
-	if ((pw = getpwuid(getuid())) == (struct passwd *)0 || *pw->pw_dir == '\0')
-	    adios(NULL, "cannot determine your home directory");
-	else
-	    mypath = pw->pw_dir;
-    }
+    set_mypath();
 
     /*
      *	Find and read user's profile.  Check for the existence of an MH environment
@@ -65,17 +64,18 @@ context_read (void)
     if ((cp = getenv("MH")) && *cp != '\0') {
 	defpath = path(cp, TFILE);
 
+        /* defpath is an absolute path; make sure that always MH is, too. */
+	setenv("MH", defpath, 1);
 	if (stat(defpath, &st) != -1 && (st.st_mode & S_IFREG) == 0)
-		adios((char *)0, "`%s' specified by your MH environment variable is not a normal file", cp);
+		die("`%s' specified by your MH environment variable is not a normal file", cp);
 
-	if ((ib = fopen(defpath, "r")) == (FILE *)0)
-	    adios((char *)0, "unable to read the `%s' profile specified by your MH environment variable", defpath);
-    }
-    else {
+	if ((ib = fopen(defpath, "r")) == NULL)
+	    die("unable to read the `%s' profile specified by your MH environment variable", defpath);
+    } else {
 	defpath = concat(mypath, "/", mh_profile, NULL);
 
-	if ((ib = fopen(defpath, "r")) == (FILE *)0)
-	    adios((char *)0, "Doesn't look like nmh is installed.  Run install-mh to do so.");
+	if ((ib = fopen(defpath, "r")) == NULL)
+	    die("Doesn't look like nmh is installed.  Run install-mh to do so.");
 
 	cp = mh_profile;
     }
@@ -88,11 +88,11 @@ context_read (void)
      *	Convert a relative path name to an absolute one rooted in the home directory.
      */
 
-    if ((cp = context_find ("path")) == (char *)0)
-	adios(NULL, "Your %s file does not contain a path entry.", defpath);
+    if ((cp = context_find ("path")) == NULL)
+	die("Your %s file does not contain a path entry.", defpath);
 
     if (*cp == '\0')
-	adios(NULL, "Your `%s' profile file does not contain a valid path entry.", defpath);
+	die("Your `%s' profile file does not contain a valid path entry.", defpath);
 
     if (*cp != '/')
 	(void)snprintf (nd = buf, sizeof(buf), "%s/%s", mypath, cp);
@@ -105,41 +105,39 @@ context_read (void)
 
 	cp = concat ("Your MH-directory \"", nd, "\" doesn't exist; Create it? ", NULL);
 
-	if (!getanswer(cp))
-	    adios (NULL, "unable to access MH-directory \"%s\"", nd);
+	if (!read_yes_or_no_if_tty(cp))
+	    die("unable to access MH-directory \"%s\"", nd);
 
 	free (cp);
 
 	if (!makedir (nd))
-	    adios (NULL, "unable to create %s", nd);
+	    die("unable to create %s", nd);
     }
 
     else if ((st.st_mode & S_IFDIR) == 0)
-	adios ((char *)0, "`%s' is not a directory", nd);
+	die("`%s' is not a directory", nd);
 
     /*
      *	Open and read user's context file.  The name of the context file comes from the
      *	profile unless overridden by the MHCONTEXT environment variable.
      */
 
-    if ((cp = getenv ("MHCONTEXT")) == (char *)0 || *cp == '\0')
+    if ((cp = getenv ("MHCONTEXT")) == NULL || *cp == '\0')
 	cp = context;
 
     /* context is NULL if context_foil() was called to disable use of context
      * We also support users setting explicitly setting MHCONTEXT to /dev/null.
-     * (if this wasn't specialcased then the locking would be liable to fail)
+     * (if this wasn't special-cased then the locking would be liable to fail)
      */
     if (!cp || (strcmp(cp,"/dev/null") == 0)) {
 	ctxpath = NULL;
 	return;
     }
     
-    ctxpath = getcpy (m_maildir (cp));
+    ctxpath = mh_xstrdup(m_maildir(cp));
 
     if ((ib = lkfopendata (ctxpath, "r", &failed_to_lock))) {
-	readconfig ((struct node **) 0, ib, cp, 1);
+	readconfig(NULL, ib, cp, 1);
 	lkfclosedata (ib, ctxpath);
     }
-
-    return;
 }
