@@ -5,8 +5,12 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "getcpy.h"
+#include "context_replace.h"
+#include "error.h"
+#include "utils.h"
+#include "globals.h"
 
 
 void
@@ -14,17 +18,19 @@ context_replace (char *key, char *value)
 {
     struct node *np;
 
+    key = FENDNULL(key);
+
     /*
      * If list is empty, allocate head of profile/context list.
      */
     if (!m_defs) {
 	NEW(np);
         m_defs = np;
-	np->n_name = getcpy (key);
+	np->n_name = mh_xstrdup(key);
 	np->n_field = getcpy (value);
 	np->n_context = 1;
 	np->n_next = NULL;
-	ctxflags |= CTXMOD;
+        context_dirty = true;
 	return;
     }
 
@@ -33,13 +39,13 @@ context_replace (char *key, char *value)
      * this key, and replace its value if found.
      */
     for (np = m_defs;; np = np->n_next) {
-	if (!strcasecmp (FENDNULL(np->n_name), FENDNULL(key))) {
+	if (!strcasecmp(FENDNULL(np->n_name), key)) {
 	    if (strcmp (value, np->n_field)) {
 		if (!np->n_context)
 		    inform("bug: context_replace(key=\"%s\",value=\"%s\"), continuing...", key, value);
-                mh_xfree(np->n_field);
+                free(np->n_field);
 		np->n_field = mh_xstrdup(value);
-		ctxflags |= CTXMOD;
+                context_dirty = true;
 	    }
 	    return;
 	}
@@ -52,9 +58,9 @@ context_replace (char *key, char *value)
      */
     NEW(np->n_next);
     np = np->n_next;
-    np->n_name = getcpy (key);
+    np->n_name = mh_xstrdup(key);
     np->n_field = getcpy (value);
     np->n_context = 1;
     np->n_next = NULL;
-    ctxflags |= CTXMOD;
+    context_dirty = true;
 }
