@@ -11,9 +11,13 @@
  * because there no longer are setuid/setgid programs in nmh.
  */
 
-#include <h/mh.h>
-#include <h/signals.h>
+#include "h/mh.h"
+#include "context_save.h"
+#include "error.h"
+#include <signal.h>
+#include "signals.h"
 #include "lock_file.h"
+#include "globals.h"
 
 void
 context_save (void)
@@ -27,9 +31,9 @@ context_save (void)
     if (!ctxpath)
        return;
 
-    if (!(ctxflags & CTXMOD))
+    if (!context_dirty)
 	return;
-    ctxflags &= ~CTXMOD;
+    context_dirty = false;
 
     /* block a few signals */
     sigemptyset (&set);
@@ -40,11 +44,9 @@ context_save (void)
     sigprocmask (SIG_BLOCK, &set, &oset);
 
     if (!(out = lkfopendata (ctxpath, "w", &failed_to_lock))) {
-	if (failed_to_lock) {
+	if (failed_to_lock)
 	    adios (ctxpath, "failed to lock");
-	} else {
-	    adios (ctxpath, "unable to write");
-	}
+        adios (ctxpath, "unable to write");
     }
     for (np = m_defs; np; np = np->n_next)
 	if (np->n_context)
