@@ -6,7 +6,9 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "cpydgst.h"
+#include "error.h"
 
 /*
  * We want to perform the substitution
