@@ -1,15 +1,19 @@
-/*
- * Routines to encode message headers using RFC 2047-encoding.
+/* encode_rfc2047.c -- encode message headers using RFC 2047 encoding.
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/mhparse.h>
-#include <h/addrsbr.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "encode_rfc2047.h"
+#include "check_charset.h"
+#include "error.h"
+#include "h/mhparse.h"
+#include "addrsbr.h"
+#include "utils.h"
+#include "base64.h"
+#include "unquote.h"
 
 /*
  * List of headers that contain addresses and as a result require special
@@ -49,7 +53,6 @@ static char *address_headers[] = {
 
 #define ENCODELINELIMIT	76
 
-static void unfold_header(char **, int);
 static int field_encode_address(const char *, char **, int, const char *);
 static int field_encode_quoted(const char *, char **, const char *, int,
 			       int, int);
@@ -79,13 +82,13 @@ encode_rfc2047(const char *name, char **value, int encoding,
 	if (isascii((unsigned char) *p)) {
 	    asciicount++;
 	    if (qpspecial((unsigned char) *p))
-	    	qpspecialcount++;
+		qpspecialcount++;
 	} else
 	    eightbitcount++;
     }
 
     if (eightbitcount == 0)
-    	return 0;
+	return 0;
 
     /*
      * Some rules from RFC 2047:
@@ -104,10 +107,10 @@ encode_rfc2047(const char *name, char **value, int encoding,
      */
 
     if (charset == NULL)
-    	charset = write_charset_8bit();
+	charset = write_charset_8bit();
 
     if (strcasecmp(charset, "US-ASCII") == 0) {
-    	advise(NULL, "Cannot use US-ASCII with 8 bit characters in header");
+	inform("Cannot use US-ASCII with 8 bit characters in header");
 	return 1;
     }
 
@@ -117,7 +120,7 @@ encode_rfc2047(const char *name, char **value, int encoding,
      */
 
     for (i = 0; address_headers[i]; i++) {
-    	if (strcasecmp(name, address_headers[i]) == 0)
+	if (strcasecmp(name, address_headers[i]) == 0)
 	    return field_encode_address(name, value, encoding, charset);
     }
 
@@ -139,14 +142,14 @@ encode_rfc2047(const char *name, char **value, int encoding,
     switch (encoding) {
 
     case CE_BASE64:
-    	return field_encode_base64(name, value, charset);
+	return field_encode_base64(name, value, charset);
 
     case CE_QUOTED:
 	return field_encode_quoted(name, value, charset, asciicount,
 				   eightbitcount + qpspecialcount, 0);
 
     default:
-    	advise(NULL, "Internal error: unknown RFC-2047 encoding type");
+	inform("Internal error: unknown RFC-2047 encoding type");
 	return 1;
     }
 }
@@ -159,7 +162,7 @@ static int
 field_encode_quoted(const char *name, char **value, const char *charset,
 		    int ascii, int encoded, int phraserules)
 {
-    int prefixlen = name ? strlen(name) + 2: 0, outlen = 0, column, newline = 1;
+    int prefixlen = name ? strlen(name) + 2: 0, outlen = 0, column;
     int charsetlen = strlen(charset), utf8;
     char *output = NULL, *p, *q = NULL;
 
@@ -174,11 +177,12 @@ field_encode_quoted(const char *name, char **value, const char *charset,
 
     utf8 = strcasecmp(charset, "UTF-8") == 0;
 
+    bool newline = true;
     while (*p != '\0') {
-    	/*
+	/*
 	 * Start a new line, if it's time
 	 */
-    	if (newline) {
+	if (newline) {
 	    /*
 	     * If it's the start of the header, we don't need to pad it
 	     *
@@ -210,12 +214,12 @@ field_encode_quoted(const char *name, char **value, const char *charset,
 		for (i = 0; i < prefixlen; i++)
 		    *q++ = ' ';
 	    } else {
-	    	/*
+		/*
 		 * A bit of a hack here; the header can contain multiple
 		 * spaces (probably at least one) until we get to the
 		 * actual text.  Copy until we get to a non-space.
 		 */
-	    	output = mh_xmalloc(outlen);
+		output = mh_xmalloc(outlen);
 		q = output;
 		while (is_fws(*p))
 		    *q++ = *p++;
@@ -224,7 +228,7 @@ field_encode_quoted(const char *name, char **value, const char *charset,
 	    tokenlen = snprintf(q, outlen - (q - output), "=?%s?Q?", charset);
 	    q += tokenlen;
 	    column = prefixlen + tokenlen;
-	    newline = 0;
+	    newline = false;
 	}
 
 	/*
@@ -241,7 +245,7 @@ field_encode_quoted(const char *name, char **value, const char *charset,
 	    ascii--;
 	} else if (isascii((unsigned char) *p) &&
 		   (phraserules ? qphrasevalid((unsigned char) *p) :
-		   			!qpspecial((unsigned char) *p))) {
+					!qpspecial((unsigned char) *p))) {
 	    *q++ = *p;
 	    ascii--;
 	} else {
@@ -265,7 +269,7 @@ field_encode_quoted(const char *name, char **value, const char *charset,
 	    continue;
 
 	if (column >= ENCODELINELIMIT - 2) {
-	    newline = 1;
+	    newline = true;
 	} else if (utf8) {
 	    /*
 	     * Okay, this is a bit weird, but to explain a bit more ...
@@ -281,16 +285,22 @@ field_encode_quoted(const char *name, char **value, const char *charset,
 	     * allow for the encoded output.
 	     */
 	    if (column + (utf8len(p) * 3) > ENCODELINELIMIT - 2) {
-	    	newline = 1;
+		newline = true;
 	    }
 	}
     }
 
+    if (q == NULL) {
+	/* This should never happen, but just in case.  Found by
+	   clang static analyzer. */
+	inform("null output encoding for %s, continuing...", *value);
+	return 1;
+    }
     *q++ = '?';
     *q++ = '=';
 
     if (prefixlen)
-    	*q++ = '\n';
+	*q++ = '\n';
 
     *q = '\0';
 
@@ -320,7 +330,7 @@ field_encode_base64(const char *name, char **value, const char *charset)
      */
 
     while (*p == ' ' || *p == '\t')
-    	p++;
+	p++;
 
     /*
      * If we had a zero-length prefix, then just encode the whole field
@@ -332,7 +342,7 @@ field_encode_base64(const char *name, char **value, const char *charset)
      */
 
     while (prefixlen && ((base64len(strlen(p)) + 7 + charsetlen +
-    			  prefixlen) > ENCODELINELIMIT)) {
+			  prefixlen) > ENCODELINELIMIT)) {
 
 	/*
 	 * Our very first time, don't pad the line in the front
@@ -375,8 +385,8 @@ field_encode_base64(const char *name, char **value, const char *charset)
 	numencode = strbase64(ENCODELINELIMIT - (q - linestart) - 2);
 
 	if (numencode <= 0) {
-	    advise(NULL, "Internal error: tried to encode %d characters "
-	    	   "in base64", numencode);
+	    inform("Internal error: tried to encode %d characters "
+		   "in base64", numencode);
 	    return 1;
 	}
 
@@ -396,10 +406,10 @@ field_encode_base64(const char *name, char **value, const char *charset)
 	     */
 
 	    while (numencode > 0 && ((*(p + numencode) & 0xc0) == 0x80))
-	    	numencode--;
+		numencode--;
 
 	    if (numencode == 0) {
-	    	advise(NULL, "Internal error: could not find start of "
+		inform("Internal error: could not find start of "
 		       "UTF-8 character when base64 encoding header");
 		return 1;
 	    }
@@ -407,7 +417,7 @@ field_encode_base64(const char *name, char **value, const char *charset)
 
 	if (writeBase64raw((unsigned char *) p, numencode,
 			   (unsigned char *) q) != OK) {
-	    advise(NULL, "Internal error: base64 encoding of header failed");
+	    inform("Internal error: base64 encoding of header failed");
 	    return 1;
 	}
 
@@ -450,18 +460,18 @@ field_encode_base64(const char *name, char **value, const char *charset)
     q = output + curlen;
 
     q += snprintf(q, outlen - (q - output), "%s=?%s?B?",
-    		  prefixlen ? " " : "", charset);
+		  prefixlen ? " " : "", charset);
 
     if (writeBase64raw((unsigned char *) p, strlen(p),
-    		       (unsigned char *) q) != OK) {
-	advise(NULL, "Internal error: base64 encoding of header failed");
+		       (unsigned char *) q) != OK) {
+	inform("Internal error: base64 encoding of header failed");
 	return 1;
     }
 
     strcat(q, "?=");
 
     if (prefixlen)
-    	strcat(q, "\n");
+	strcat(q, "\n");
 
     free(*value);
 
@@ -483,14 +493,14 @@ utf8len(const char *p)
     int len = 1;
 
     if (*p == '\0')
-    	return 0;
+	return 0;
 
     if (isascii((unsigned char) *p) || (((unsigned char) *p) & 0xc0) == 0x80)
-    	return 0;
+	return 0;
 
     p++;
     while ((((unsigned char) *p++) & 0xc0) == 0x80)
-    	len++;
+	len++;
 
     return len;
 }
@@ -502,14 +512,14 @@ utf8len(const char *p)
  * original length here is fine.
  */
 
-static void
-unfold_header(char **value, int len)
+void
+unfold_header(char **value, size_t len)
 {
     char *str = mh_xmalloc(len + 1);
     char *p = str, *q = *value;
 
     while (*q != '\0') {
-    	if (*q == '\n') {
+	if (*q == '\n') {
 	    /*
 	     * When we get a newline, skip to the next non-whitespace
 	     * character and add a space to replace all of the whitespace
@@ -518,9 +528,9 @@ unfold_header(char **value, int len)
 	     * for the header; we put it back in the encoding routine.
 	     */
 	    while (is_fws(*q))
-	    	q++;
+		q++;
 	    if (*q == '\0')
-	    	break;
+		break;
 
 	    *p++ = ' ';
 	} else {
@@ -544,7 +554,11 @@ field_encode_address(const char *name, char **value, int encoding,
 		     const char *charset)
 {
     int prefixlen = strlen(name) + 2, column = prefixlen, groupflag;
-    int asciichars, specialchars, eightbitchars, reformat = 0, errflag = 0;
+    int asciichars;
+    int specialchars;
+    int eightbitchars;
+    bool reformat = false;
+    bool errflag = false;
     size_t len;
     char *mp, *cp = NULL, *output = NULL;
     char *tmpbuf = NULL;
@@ -567,13 +581,13 @@ field_encode_address(const char *name, char **value, int encoding,
     output = add(" ", output);
 
     for (groupflag = 0; (mp = getname(*value)); ) {
-    	if ((mn = getm(mp, NULL, 0, errbuf, sizeof(errbuf))) == NULL) {
-	    advise(NULL, "%s: %s", errbuf, mp);
-	    errflag++;
+	if ((mn = getm(mp, NULL, 0, errbuf, sizeof(errbuf))) == NULL) {
+	    inform("%s: %s", errbuf, mp);
+	    errflag = true;
 	    continue;
 	}
 
-	reformat = 0;
+	reformat = false;
 
 	/*
 	 * We only care if the phrase (m_pers) or any trailing comment
@@ -598,7 +612,7 @@ field_encode_address(const char *name, char **value, int encoding,
 	     */
 
 	    if (encoding == CE_UNKNOWN)
-	    	encoding = pref_encoding(asciichars, specialchars,
+		encoding = pref_encoding(asciichars, specialchars,
 					 eightbitchars);
 
 	    /*
@@ -611,27 +625,27 @@ field_encode_address(const char *name, char **value, int encoding,
 	    switch (encoding) {
 
 	    case CE_BASE64:
-	    	if (field_encode_base64(NULL, &mn->m_pers, charset)) {
-		    errflag++;
+		if (field_encode_base64(NULL, &mn->m_pers, charset)) {
+		    errflag = true;
 		    goto out;
 		}
 		break;
 
 	    case CE_QUOTED:
-	    	if (field_encode_quoted(NULL, &mn->m_pers, charset, asciichars,
+		if (field_encode_quoted(NULL, &mn->m_pers, charset, asciichars,
 					eightbitchars + specialchars, 1)) {
-		    errflag++;
+		    errflag = true;
 		    goto out;
 		}
 		break;
 
 	    default:
-		advise(NULL, "Internal error: unknown RFC-2047 encoding type");
-		errflag++;
+		inform("Internal error: unknown RFC-2047 encoding type");
+		errflag = true;
 		goto out;
 	    }
 
-	    reformat++;
+	    reformat = true;
 	}
 
 	check_note:
@@ -652,9 +666,9 @@ field_encode_address(const char *name, char **value, int encoding,
 	}
 
 	if (mn->m_note[0] != '(' || mn->m_note[len - 1] != ')') {
-	    advise(NULL, "Internal error: Invalid note field \"%s\"",
-	    	   mn->m_note);
-	    errflag++;
+	    inform("Internal error: Invalid note field \"%s\"",
+		   mn->m_note);
+	    errflag = true;
 	    goto out;
 	}
 
@@ -668,33 +682,33 @@ field_encode_address(const char *name, char **value, int encoding,
 	     */
 
 	    if (encoding == CE_UNKNOWN)
-	    	encoding = pref_encoding(asciichars, specialchars,
+		encoding = pref_encoding(asciichars, specialchars,
 					 eightbitchars);
 
 	    switch (encoding) {
 
 	    case CE_BASE64:
-	    	if (field_encode_base64(NULL, &tmpbuf, charset)) {
-		    errflag++;
+		if (field_encode_base64(NULL, &tmpbuf, charset)) {
+		    errflag = true;
 		    goto out;
 		}
 		break;
 
 	    case CE_QUOTED:
-	    	if (field_encode_quoted(NULL, &tmpbuf, charset, asciichars,
+		if (field_encode_quoted(NULL, &tmpbuf, charset, asciichars,
 					eightbitchars + specialchars, 1)) {
-		    errflag++;
+		    errflag = true;
 		    goto out;
 		}
 		break;
 
 	    default:
-		advise(NULL, "Internal error: unknown RFC-2047 encoding type");
-		errflag++;
+		inform("Internal error: unknown RFC-2047 encoding type");
+		errflag = true;
 		goto out;
 	    }
 
-	    reformat++;
+	    reformat = true;
 
 	    /*
 	     * Make sure the size of tmpbuf is correct (it always gets
@@ -731,13 +745,13 @@ do_reformat:
 
 	if (groupflag && ! mn->m_ingrp) {
 	    output = add(";", output);
-	    column += 1;
+	    column++;
 	}
 
 	groupflag = mn->m_ingrp;
 
 	if (mn->m_gname) {
-	    cp = add(mn->m_gname, NULL);
+	    cp = mh_xstrdup(mn->m_gname);
 	}
 
 	if (reformat) {
@@ -756,13 +770,13 @@ do_reformat:
 	if (column != prefixlen) {
 	    if (len + column + 2 > OUTPUTLINELEN) {
 
-	    	if ((size_t) (prefixlen + 3) < tmpbufsize)
+		if ((size_t) (prefixlen + 3) < tmpbufsize)
 		    tmpbuf = mh_xrealloc(tmpbuf, tmpbufsize = prefixlen + 3);
 
 		snprintf(tmpbuf, tmpbufsize, ",\n%*s", column = prefixlen, "");
 		output = add(tmpbuf, output);
 	    } else {
-	    	output = add(", ", output);
+		output = add(", ", output);
 		column += 2;
 	    }
 	}
@@ -792,13 +806,10 @@ do_reformat:
     output = NULL;
 
 out:
+    free(tmpbuf);
+    free(output);
 
-    if (tmpbuf)
-    	free(tmpbuf);
-    if (output)
-    	free(output);
-
-    return errflag > 0;
+    return errflag;
 }
 
 /*
@@ -814,7 +825,7 @@ scanstring(const char *string, int *asciilen, int *eightbitchars,
     *specialchars = 0;
 
     for (; *string != '\0'; string++) {
-    	if ((isascii((unsigned char) *string))) {
+	if ((isascii((unsigned char) *string))) {
 	    (*asciilen)++;
 	    /*
 	     * So, a space is not a valid phrase character, but we're counting
@@ -822,7 +833,7 @@ scanstring(const char *string, int *asciilen, int *eightbitchars,
 	     * encoded as an underscore.
 	     */
 	    if (!qphrasevalid((unsigned char) *string) && *string != ' ')
-	    	(*specialchars)++;
+		(*specialchars)++;
 	} else {
 	    (*eightbitchars)++;
 	}
@@ -861,5 +872,5 @@ pref_encoding(int ascii, int specials, int eightbits)
      */
 
     return base64len(ascii + eightbits) < (ascii - specials +
-    			(specials + eightbits) * 3) ? CE_BASE64 : CE_QUOTED;
+			(specials + eightbits) * 3) ? CE_BASE64 : CE_QUOTED;
 }
