@@ -5,15 +5,18 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "error.h"
 
 #include <sys/types.h>
 #include <sys/uio.h>
-
+#include "done.h"
+#include "globals.h"
 
 /* inform calls advertise() with no what and no tail.
  * Thus the simple "[invo_name: ]fmt\n" results. */
-void inform(char *fmt, ...)
+void
+inform(char *fmt, ...)
 {
     va_list ap;
 
@@ -41,6 +44,7 @@ advise (const char *what, const char *fmt, ...)
  * It calls advertise() with no tail to print fmt, and perhaps what, to
  * stderr, and exits the program with an error status.
  * Thus "[invo_name: ]fmt[[ what]: errno]\n" results.
+ *
  * The route to exit is via the done function pointer and may not be
  * straightforward, e.g. longjmp(3), but it must not return to adios().
  * If it does then it's a bug and adios() will abort(3) as callers do
@@ -58,6 +62,22 @@ adios (const char *what, const char *fmt, ...)
 }
 
 
+/* die is the same as adios(), but without the what as that's commonly
+ * NULL.
+ * Thus "[invo_name: ]fmt\n" results. */
+void NORETURN
+die(const char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap, fmt);
+    advertise(NULL, NULL, fmt, ap);
+    va_end(ap);
+    done(1);
+    abort();
+}
+
+
 /* admonish calls advertise() with a tail indicating the program
  * continues.
  * Thus "[invo_name: ]fmt[[ what]: errno], continuing...\n" results. */
@@ -73,17 +93,23 @@ admonish (char *what, char *fmt, ...)
 
 
 /* advertise prints fmt and ap to stderr after flushing stdout.
- * If invo_name isn't NULL or empty then "invo_name: " precedes fmt.
- * If what isn't NULL or empty      then " what"   is appended.
- * If what isn't NULL               then ": errno" is appended.
- * If tail isn't NULL or empty      then ", tail"  is appended.
- * A "\n" finishes the output to stderr.
+ * The print happens in a single syscall from a buffer.
+ * If invo_name isn't NULL or empty     then "invo_name: " is buffered.
+ *                                      Then fmt           is buffered.
+ * If what isn't NULL or empty          then " what"       is buffered.
+ * If what isn't NULL and errno isn't 0 then ": errno"     is buffered.
+ * If tail isn't NULL or empty          then ", tail"      is buffered.
+ *                                      Then "\n"          is buffered.
+ *
+ * Note, a what of "" doesn't print it or the preceding space, but does
+ * trigger the printing of a non-zero errno; such is the cunning design.
+ *
  * In summary: "[invo_name: ]fmt[[ what]: errno][, tail]\n". */
 void
 advertise (const char *what, char *tail, const char *fmt, va_list ap)
 {
     int	eindex = errno;
-    char buffer[NMH_BUFSIZ], *err;
+    char buffer[NMH_BUFSIZ];
     struct iovec iob[10], *iov;
     size_t niov;
 
@@ -109,9 +135,11 @@ advertise (const char *what, char *tail, const char *fmt, va_list ap)
             ADD_LITERAL(" ");
             ADD_VAR((void *)what);
 	}
-        ADD_LITERAL(": ");
-        err = strerror(eindex);
-        ADD_VAR(err);
+        if (eindex) {
+            ADD_LITERAL(": ");
+            char *err = strerror(eindex);
+            ADD_VAR(err);
+        }
     }
     if (tail && *tail) {
         ADD_LITERAL(", ");
