@@ -1,14 +1,16 @@
-
-/*
- * fmt_new.c -- read format file/string and normalize
+/* fmt_new.c -- read format file/string and normalize
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "fmt_new.h"
+#include "getcpy.h"
+#include "error.h"
+#include "utils.h"
+#include "path.h"
 
 #define QUOTE '\\'
 
@@ -28,22 +30,22 @@ char *
 new_fs (char *form, char *format, char *default_fs)
 {
     struct stat st;
-    register FILE *fp;
+    FILE *fp;
 
-    if (formats)
-	free (formats);
+    free(formats);
 
     if (form) {
-	if ((fp = fopen (etcpath (form), "r")) == NULL)
-	    adios (form, "unable to open format file");
+        char *path = etcpath(form);
+        if ((fp = fopen(path, "r")) == NULL)
+            adios(path, "unable to open format file");
 
 	if (fstat (fileno (fp), &st) == -1)
-	    adios (form, "unable to stat format file");
+            adios(path, "unable to stat format file");
 
 	formats = mh_xmalloc ((size_t) st.st_size + 1);
 
 	if (read (fileno(fp), formats, (int) st.st_size) != st.st_size)
-	    adios (form, "error reading format file");
+            adios(path, "error reading format file");
 
 	formats[st.st_size] = '\0';
 
@@ -58,6 +60,14 @@ new_fs (char *form, char *format, char *default_fs)
 }
 
 
+void
+free_fs(void)
+{
+    free (formats);
+    formats = 0;
+}
+
+
 /*
  * Expand escapes in format strings
  */
@@ -96,7 +106,8 @@ normalize (char *cp)
 		    break;
 
 		case 0: 
-		    cp--;	/* fall */
+		    cp--;
+		    /* FALLTHRU */
 		default: 
 		    *dp++ = *cp;
 		    break;
