@@ -6,7 +6,13 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "m_name.h"
+#include "seq_setcur.h"
+#include "ext_hook.h"
+#include "folder_realloc.h"
+#include "folder_pack.h"
+#include "error.h"
 
 /*
  * Pack the message in a folder.
@@ -33,19 +39,18 @@ folder_pack (struct msgs **mpp, int verbose)
      * for all numbers from 1 to current high message.
      */
     if (mp->lowoff > 1) {
-	if ((mp = folder_realloc (mp, 1, mp->hghmsg)))
-	    *mpp = mp;
-	else {
+	if (!(mp = folder_realloc(mp, 1, mp->hghmsg))) {
 	    inform("unable to allocate folder storage");
 	    return -1;
 	}
+        *mpp = mp;
     }
 
     for (msgnum = mp->lowmsg, hole = 1; msgnum <= mp->hghmsg; msgnum++) {
 	if (does_exist (mp, msgnum)) {
 	    if (msgnum != hole) {
-		strncpy (newmsg, m_name (hole), sizeof(newmsg));
-		strncpy (oldmsg, m_name (msgnum), sizeof(oldmsg));
+		strncpy (newmsg, m_name (hole), sizeof(newmsg) - 1);
+		strncpy (oldmsg, m_name (msgnum), sizeof(oldmsg) - 1);
 		if (verbose)
 		    printf ("message %s becomes %s\n", oldmsg, newmsg);
 
