@@ -5,8 +5,10 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "geteditor.h"
+#include "context_find.h"
+#include "utils.h"
 
 static char *default_editor = NULL;
 
@@ -16,12 +18,12 @@ get_default_editor(void)
     char *str;
 
     if (default_editor)
-    	return default_editor;
+	return default_editor;
 
     if (!(str = context_find("editor")) && !(str = getenv("VISUAL")) &&
-    	!(str = getenv("EDITOR"))) {
-	str = DEFAULT_EDITOR;
+	!(str = getenv("EDITOR"))) {
+	str = "prompter";
     }
 
-    return (default_editor = str);
+    return default_editor = str;
 }
