@@ -5,9 +5,13 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "getfolder.h"
+#include "context_find.h"
+#include "globals.h"
 
 
+/* Does not return NULL. */
 char *
 getfolder(int wantcurrent)
 {
