@@ -1,6 +1,4 @@
-
-/*
- * m_atoi.c -- Parse a string representation of a message number, and
+/* m_atoi.c -- Parse a string representation of a message number, and
  *          -- return the numeric value of the message.  If the string
  *          -- contains any non-digit characters, then return 0.
  *
@@ -9,7 +7,8 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "m_atoi.h"
 
 
 int
