@@ -5,9 +5,12 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
-#include <h/signals.h>
+#include "h/mh.h"
+#include "concat.h"
+#include "r1bindex.h"
+#include "utils.h"
+#include <signal.h>
+#include "signals.h"
 #include "m_maildir.h"
 #include "m_mktemp.h"
 
@@ -51,7 +54,6 @@ m_mktemp (
 {
     static char tmpfil[BUFSIZ];
     int fd = -1;
-    int keep_open = 0;
     mode_t oldmode = umask(077);
 
     if (pfx_in == NULL) {
@@ -69,9 +71,10 @@ m_mktemp (
 
     register_for_removal(tmpfil);
 
+    bool keep_open = false;
     if (fd_ret != NULL) {
         *fd_ret = fd;
-        keep_open = 1;
+        keep_open = true;
     }
     if (fp_ret != NULL) {
         FILE *fp = fdopen(fd, "w+");
@@ -84,7 +87,7 @@ m_mktemp (
             return NULL;
         }
         *fp_ret = fp;
-        keep_open = 1;
+        keep_open = true;
     }
     if (!keep_open) {
         close(fd);
@@ -151,7 +154,6 @@ m_mktemps(
 {
     char *tmpfil;
     int fd = -1;
-    int keep_open = 0;
     mode_t oldmode = umask(077);
 
     if (suffix == NULL) {
@@ -202,9 +204,10 @@ m_mktemps(
 
     register_for_removal(tmpfil);
 
+    bool keep_open = false;
     if (fd_ret != NULL) {
         *fd_ret = fd;
-        keep_open = 1;
+        keep_open = true;
     }
     if (fp_ret != NULL) {
         FILE *fp = fdopen(fd, "w+");
@@ -218,7 +221,7 @@ m_mktemps(
             return NULL;
         }
         *fp_ret = fp;
-        keep_open = 1;
+        keep_open = true;
     }
     if (!keep_open) {
         close(fd);
@@ -256,9 +259,10 @@ static svector_t exit_filelist = NULL;
  * Register a file for removal at program termination.
  */
 static void
-register_for_removal(const char *pathname) {
+register_for_removal(const char *pathname)
+{
     if (exit_filelist == NULL) exit_filelist = svector_create(20);
-    (void) svector_push_back(exit_filelist, add(pathname, NULL));
+    (void) svector_push_back(exit_filelist, mh_xstrdup(pathname));
 }
 
 /*
@@ -280,7 +284,8 @@ register_for_removal(const char *pathname) {
  * quickly created with the same name.
  */
 void
-unregister_for_removal(int remove_files) {
+unregister_for_removal(int remove_files)
+{
     if (exit_filelist) {
         size_t i;
 
@@ -303,7 +308,8 @@ unregister_for_removal(int remove_files) {
  * any case, unlink it.
  */
 int
-m_unlink(const char *pathname) {
+m_unlink(const char *pathname)
+{
     if (exit_filelist) {
         char **slot = svector_find(exit_filelist, pathname);
 
@@ -321,7 +327,8 @@ m_unlink(const char *pathname) {
  * Remove all registered temporary files.
  */
 void
-remove_registered_files_atexit(void) {
+remove_registered_files_atexit(void)
+{
     unregister_for_removal(1);
 }
 
@@ -333,7 +340,8 @@ remove_registered_files_atexit(void) {
  * in case the use was expecting a core dump.
  */
 void
-remove_registered_files(int sig) {
+remove_registered_files(int sig)
+{
     struct sigaction act;
 
     /*
