@@ -6,12 +6,16 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
-#include <h/netsec.h>
-#include <h/oauth.h>
+#include "h/mh.h"
+#include "credentials.h"
+#include "getcpy.h"
+#include "brkstring.h"
+#include "utils.h"
+#include "h/netsec.h"
+#include "oauth.h"
 #include <stdarg.h>
 #include <sys/select.h>
+#include "base64.h"
 
 #ifdef CYRUS_SASL
 #include <sasl/sasl.h>
@@ -29,7 +33,7 @@ static int netsec_get_user(void *context, int id, const char **result,
 static int netsec_get_password(sasl_conn_t *conn, void *context, int id,
 			       sasl_secret_t **psecret);
 
-static int sasl_initialized = 0;
+static bool sasl_initialized;
 
 #define SASL_MAXRECVBUF 65536
 #endif /* CYRUS_SASL */
@@ -38,7 +42,7 @@ static int sasl_initialized = 0;
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 
-static int tls_initialized = 0;
+static bool tls_initialized;
 static SSL_CTX *sslctx = NULL;		/* SSL Context */
 
 #endif /* TLS_SUPPORT */
@@ -56,9 +60,9 @@ struct _netsec_context {
     int ns_writefd;		/* Write descriptor for network connection */
     int ns_noclose;		/* Do not close file descriptors if set */
     int ns_snoop;		/* If true, display network data */
-    int ns_snoop_noend;		/* If true, didn't get a CR/LF on last line */
     netsec_snoop_callback *ns_snoop_cb; /* Snoop output callback */
     void *ns_snoop_context;	/* Context data for snoop function */
+    char *ns_snoop_savebuf;	/* Save buffer for snoop data */
     int ns_timeout;		/* Network read timeout, in seconds */
     char *ns_userid;		/* Userid for authentication */
     char *ns_hostname;		/* Hostname we've connected to */
@@ -73,6 +77,7 @@ struct _netsec_context {
     char *sasl_mech;		/* User-requested mechanism */
     char *sasl_chosen_mech;	/* Mechanism chosen by SASL */
     netsec_sasl_callback sasl_proto_cb; /* SASL callback we use */
+    void *sasl_proto_context;	/* Context to be used by SASL callback */
 #ifdef OAUTH_SUPPORT
     char *oauth_service;	/* OAuth2 service name */
 #endif /* OAUTH_SUPPORT */
@@ -143,9 +148,9 @@ netsec_init(void)
     nsc->ns_writefd = -1;
     nsc->ns_noclose = 0;
     nsc->ns_snoop = 0;
-    nsc->ns_snoop_noend = 0;
     nsc->ns_snoop_cb = NULL;
     nsc->ns_snoop_context = NULL;
+    nsc->ns_snoop_savebuf = NULL;
     nsc->ns_userid = NULL;
     nsc->ns_hostname = NULL;
     nsc->ns_timeout = 60;	/* Our default */
@@ -160,6 +165,7 @@ netsec_init(void)
     nsc->sasl_mech = NULL;
     nsc->sasl_chosen_mech = NULL;
     nsc->sasl_proto_cb = NULL;
+    nsc->sasl_proto_context = NULL;
 #ifdef OAUTH_SUPPORT
     nsc->oauth_service = NULL;
 #endif /* OAUTH_SUPPORT */
@@ -187,19 +193,20 @@ netsec_init(void)
 void
 netsec_shutdown(netsec_context *nsc)
 {
-    mh_xfree(nsc->ns_userid);
-    mh_xfree(nsc->ns_hostname);
-    mh_xfree(nsc->ns_inbuffer);
-    mh_xfree(nsc->ns_outbuffer);
-    mh_xfree(nsc->sasl_mech);
-    mh_xfree(nsc->sasl_chosen_mech);
+    free(nsc->ns_userid);
+    free(nsc->ns_hostname);
+    free(nsc->ns_inbuffer);
+    free(nsc->ns_outbuffer);
+    free(nsc->sasl_mech);
+    free(nsc->sasl_chosen_mech);
+    free(nsc->ns_snoop_savebuf);
 #ifdef OAUTH_SERVICE
-    mh_xfree(nsc->oauth_service);
+    free(nsc->oauth_service);
 #endif /* OAUTH_SERVICE */
 #ifdef CYRUS_SASL
     if (nsc->sasl_conn)
 	sasl_dispose(&nsc->sasl_conn);
-    mh_xfree(nsc->sasl_cbs);
+    free(nsc->sasl_cbs);
     if (nsc->sasl_creds)
 	nmh_credentials_free(nsc->sasl_creds);
     if (nsc->sasl_secret) {
@@ -208,7 +215,7 @@ netsec_shutdown(netsec_context *nsc)
 	}
 	free(nsc->sasl_secret);
     }
-    mh_xfree(nsc->sasl_tmpbuf);
+    free(nsc->sasl_tmpbuf);
 #endif /* CYRUS_SASL */
 #ifdef TLS_SUPPORT
     if (nsc->ssl_io)
@@ -284,7 +291,8 @@ netsec_set_snoop(netsec_context *nsc, int snoop)
  * Set the snoop callback for this connection.
  */
 
-void netsec_set_snoop_callback(netsec_context *nsc,
+void
+netsec_set_snoop_callback(netsec_context *nsc,
 			       netsec_snoop_callback callback, void *context)
 {
     nsc->ns_snoop_cb = callback;
@@ -315,7 +323,7 @@ netsec_b64_snoop_decoder(netsec_context *nsc, const char *string, size_t len,
 	len -= offset;
     }
 
-    if (decodeBase64(string, &decoded, &decodedlen, 1, NULL) == OK) {
+    if (decodeBase64(string, &decoded, &decodedlen, 1) == OK) {
 	/*
 	 * Some mechanisms produce large binary tokens, which aren't really
 	 * readable.  So let's do a simple heuristic.  If the token is greater
@@ -460,8 +468,10 @@ retry:
 		if (nsc->ns_snoop_cb)
 		    nsc->ns_snoop_cb(nsc, sptr, strlen(sptr),
 				     nsc->ns_snoop_context);
-		else
-		    fprintf(stderr, "%s\n", sptr);
+		else {
+                    fputs(sptr, stderr);
+                    putc('\n', stderr);
+                }
 	    }
 	    return sptr;
 	}
@@ -473,7 +483,7 @@ retry:
      */
 
     if (count >= nsc->ns_inbufsize / 2) {
-	netsec_err(errstr, "Unable to find a line terminator after %d bytes",
+	netsec_err(errstr, "Unable to find a line terminator after %zu bytes",
 		   count);
 	return NULL;
     }
@@ -583,7 +593,6 @@ retry:
     if (startoffset > nsc->ns_inbufsize / 2) {
 	memmove(nsc->ns_inbuffer, nsc->ns_inptr, nsc->ns_inbuflen);
 	nsc->ns_inptr = nsc->ns_inbuffer;
-	startoffset = 0;
 	remaining = nsc->ns_inbufsize - nsc->ns_inbuflen;
 	end = nsc->ns_inptr + nsc->ns_inbuflen;
     }
@@ -827,7 +836,7 @@ retry:
 	     * At that point, just give up.
 	     */
 	    netsec_err(errstr, "Internal error: wanted to printf() a total of "
-	    	       "%d bytes, but our buffer size was only %d bytes",
+		       "%d bytes, but our buffer size was only %u bytes",
 		       rc, nsc->ns_outbufsize);
 	    return NOTOK;
 	}
@@ -844,35 +853,6 @@ retry:
         goto retry;
     }
 
-    if (nsc->ns_snoop) {
-	int outlen = rc;
-	if (outlen > 0 && nsc->ns_outptr[outlen - 1] == '\n') {
-	    outlen--;
-	    if (outlen > 0 && nsc->ns_outptr[outlen - 1] == '\r')
-		outlen--;
-	} else {
-	    nsc->ns_snoop_noend = 1;
-	}
-	if (outlen > 0 || nsc->ns_snoop_noend == 0) {
-#ifdef CYRUS_SASL
-	    if (nsc->sasl_seclayer)
-		fprintf(stderr, "(sasl-encrypted) ");
-#endif /* CYRUS_SASL */
-#ifdef TLS_SUPPORT
-	    if (nsc->tls_active)
-		fprintf(stderr, "(tls-encrypted) ");
-#endif /* TLS_SUPPORT */
-	    fprintf(stderr, "=> ");
-	    if (nsc->ns_snoop_cb)
-		nsc->ns_snoop_cb(nsc, (char *) nsc->ns_outptr, outlen,
-				 nsc->ns_snoop_context);
-	    else
-		 fprintf(stderr, "%.*s\n", outlen, nsc->ns_outptr); 
-	} else {
-	    nsc->ns_snoop_noend = 0;
-	}
-    }
-
     nsc->ns_outptr += rc;
     nsc->ns_outbuflen += rc;
 
@@ -898,6 +878,96 @@ netsec_flush(netsec_context *nsc, char **errstr)
     if (netoutlen == 0)
 	return OK;
 
+    /*
+     * If we have snoop turned on, output the data.
+     *
+     * Note here; if we don't have a CR or LF at the end, save the data
+     * in ns_snoop_savebuf for later and print it next time.
+     */
+
+    if (nsc->ns_snoop) {
+	unsigned int snoopoutlen = netoutlen;
+	const char *snoopoutbuf = (const char *) nsc->ns_outbuffer;
+
+	while (snoopoutlen > 0) {
+	    const char *end = strpbrk(snoopoutbuf, "\r\n");
+	    unsigned int outlen;
+
+	    if (! end) {
+		if (nsc->ns_snoop_savebuf) {
+		    nsc->ns_snoop_savebuf = mh_xrealloc(nsc->ns_snoop_savebuf,
+						strlen(nsc->ns_snoop_savebuf) +
+						snoopoutlen + 1);
+		    strncat(nsc->ns_snoop_savebuf, snoopoutbuf, snoopoutlen);
+		} else {
+		    nsc->ns_snoop_savebuf = mh_xmalloc(snoopoutlen + 1);
+		    strncpy(nsc->ns_snoop_savebuf, snoopoutbuf, snoopoutlen);
+		    nsc->ns_snoop_savebuf[snoopoutlen] = '\0';
+		}
+		break;
+	    }
+
+	    outlen = end - snoopoutbuf;
+
+#ifdef CYRUS_SASL
+	    if (nsc->sasl_seclayer)
+		fprintf(stderr, "(sasl-encrypted) ");
+#endif /* CYRUS_SASL */
+#ifdef TLS_SUPPORT
+	    if (nsc->tls_active)
+		fprintf(stderr, "(tls-encrypted) ");
+#endif /* TLS_SUPPORT */
+	    fprintf(stderr, "=> ");
+	    if (nsc->ns_snoop_cb) {
+		const char *ptr;
+		unsigned int cb_len = outlen;
+
+		if (nsc->ns_snoop_savebuf) {
+		    cb_len += strlen(nsc->ns_snoop_savebuf);
+		    nsc->ns_snoop_savebuf = mh_xrealloc(nsc->ns_snoop_savebuf,
+							outlen);
+		    ptr = nsc->ns_snoop_savebuf;
+		} else {
+		    ptr = snoopoutbuf;
+		}
+
+		nsc->ns_snoop_cb(nsc, ptr, cb_len, nsc->ns_snoop_context);
+
+		if (nsc->ns_snoop_savebuf) {
+		    free(nsc->ns_snoop_savebuf);
+		    nsc->ns_snoop_savebuf = NULL;
+		}
+	    } else {
+		if (nsc->ns_snoop_savebuf) {
+		    fprintf(stderr, "%s", nsc->ns_snoop_savebuf);
+		    free(nsc->ns_snoop_savebuf);
+		    nsc->ns_snoop_savebuf = NULL;
+		}
+		fwrite(snoopoutbuf, 1, outlen, stderr);
+		putc('\n', stderr);
+	    }
+
+	    /*
+	     * Alright, hopefully any previous leftover data is done,
+	     * and we have the current line output.  Move things past the
+	     * next CR/LF.
+	     */
+
+	    snoopoutlen -= outlen;
+	    snoopoutbuf += outlen;
+
+	    if (snoopoutlen > 0 && *snoopoutbuf == '\r') {
+		snoopoutlen--;
+		snoopoutbuf++;
+	    }
+
+	    if (snoopoutlen > 0 && *snoopoutbuf == '\n') {
+		snoopoutlen--;
+		snoopoutbuf++;
+	    }
+	}
+    }
+
     /*
      * If SASL security layers are in effect, run the data through
      * sasl_encode() first.
@@ -951,7 +1021,7 @@ netsec_flush(netsec_context *nsc, char **errstr)
 int
 netsec_set_sasl_params(netsec_context *nsc, const char *service,
 		       const char *mechanism, netsec_sasl_callback callback,
-		       char **errstr)
+		       void *context, char **errstr)
 {
 #ifdef CYRUS_SASL
     sasl_callback_t *sasl_cbs;
@@ -969,7 +1039,7 @@ netsec_set_sasl_params(netsec_context *nsc, const char *service,
 		       sasl_errstring(retval, NULL, NULL));
 	    return NOTOK;
 	}
-	sasl_initialized++;
+	sasl_initialized = true;
     }
 
     /*
@@ -1024,15 +1094,12 @@ netsec_set_sasl_params(netsec_context *nsc, const char *service,
      */
 
     if (mechanism) {
-	char *p;
 	nsc->sasl_mech = mh_xstrdup(mechanism);
-
-	for (p = nsc->sasl_mech; *p; p++)
-	    if (isascii((unsigned char) *p)) /* Leave non-ASCII lower alone. */
-		*p = toupper((unsigned char) *p);
+	to_upper(nsc->sasl_mech);
     }
 
     nsc->sasl_proto_cb = callback;
+    nsc->sasl_proto_context = context;
 
     return OK;
 }
@@ -1043,7 +1110,8 @@ netsec_set_sasl_params(netsec_context *nsc, const char *service,
  * library when asked.
  */
 
-int netsec_get_user(void *context, int id, const char **result,
+int
+netsec_get_user(void *context, int id, const char **result,
 		    unsigned int *len)
 {
     netsec_context *nsc = (netsec_context *) context;
@@ -1085,7 +1153,7 @@ netsec_get_password(sasl_conn_t *conn, void *context, int id,
      * us room for a terminating NUL
      */
 
-    *psecret = (sasl_secret_t *) malloc(sizeof(sasl_secret_t) + len);
+    *psecret = malloc(sizeof(sasl_secret_t) + len);
 
     if (! *psecret)
 	return SASL_NOMEM;
@@ -1123,6 +1191,22 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
     int rc;
 #endif /* CYRUS_SASL || OAUTH_SUPPORT */
 
+    /*
+     * Output some SASL information if snoop is turned on
+     */
+
+    if (nsc->ns_snoop) {
+	fprintf(stderr, "SASL mechanisms supported by server: %s\n", mechlist);
+
+	if (nsc->sasl_mech) {
+		fprintf(stderr, "User has requested SASL mechanism: %s\n",
+			nsc->sasl_mech);
+	} else {
+		fprintf(stderr, "No SASL mech selected, will pick "
+			"the best mech supported by SASL library\n");
+	}
+    }
+
     /*
      * If we've been passed a requested mechanism, check our mechanism
      * list from the protocol.  If it's not supported, return an error.
@@ -1161,6 +1245,10 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
 	 * error message.
 	 */
 
+	if (nsc->ns_snoop) {
+		fprintf(stderr, "Using internal XOAUTH2 mechanism\n");
+	}
+
 	if (! nsc->oauth_service) {
 	    netsec_err(errstr, "Internal error: OAuth2 service name not given");
 	    return NOTOK;
@@ -1172,12 +1260,13 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
 			       &xoauth_client_res, &xoauth_client_res_len,
 			       nsc->ns_snoop ? stderr : NULL) != OK) {
 	    netsec_err(errstr, "Internal error: Unable to get OAuth2 "
-	    	       "bearer token");
+		       "bearer token");
 	    return NOTOK;
 	}
 
 	rc = nsc->sasl_proto_cb(NETSEC_SASL_START, xoauth_client_res,
-				xoauth_client_res_len, NULL, 0, errstr);
+				xoauth_client_res_len, NULL, 0,
+				nsc->sasl_proto_context, errstr);
 	free(xoauth_client_res);
 
 	if (rc != OK)
@@ -1190,7 +1279,8 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
 	 * error.
 	 */
 
-	rc = nsc->sasl_proto_cb(NETSEC_SASL_FINISH, NULL, 0, NULL, 0, errstr);
+	rc = nsc->sasl_proto_cb(NETSEC_SASL_FINISH, NULL, 0, NULL, 0,
+				nsc->sasl_proto_context, errstr);
 
 	if (rc != OK) {
 	    /*
@@ -1200,9 +1290,10 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
 	     * NOT get a success message at this point.
 	     */
 	    free(*errstr);
-	    nsc->sasl_proto_cb(NETSEC_SASL_WRITE, NULL, 0, NULL, 0, NULL);
+	    nsc->sasl_proto_cb(NETSEC_SASL_WRITE, NULL, 0, NULL, 0,
+			       nsc->sasl_proto_context, NULL);
 	    rc = nsc->sasl_proto_cb(NETSEC_SASL_FINISH, NULL, 0, NULL, 0,
-				    errstr);
+				    nsc->sasl_proto_context, errstr);
 	    if (rc == 0) {
 		netsec_err(errstr, "Unexpected success after OAuth failure!");
 	    }
@@ -1220,7 +1311,22 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
      * messages.
      */
 
-    memset(&secprops, 0, sizeof(secprops));
+    if (nsc->ns_snoop) {
+	const char *client_mechlist;
+
+	rc = sasl_listmech(nsc->sasl_conn, NULL, NULL, " ", NULL,
+			   &client_mechlist, NULL, NULL);
+
+	if (rc != SASL_OK) {
+		fprintf(stderr, "Unable to get client mechlist: %s\n",
+			sasl_errstring(rc, NULL, NULL));
+	} else {
+		fprintf(stderr, "Client supported SASL mechanisms: %s\n",
+			client_mechlist);
+	}
+    }
+
+    ZERO(&secprops);
     secprops.maxbufsize = SASL_MAXRECVBUF;
 
     /*
@@ -1233,6 +1339,12 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
 #endif /* TLS_SUPPORT */
 		UINT_MAX;
 
+#ifdef TLS_SUPPORT
+    if (nsc->ns_snoop && nsc->tls_active)
+	fprintf(stderr, "SASL security layers disabled due to the use "
+		"of TLS\n");
+#endif /* TLS_SUPPORT */
+
     rc = sasl_setprop(nsc->sasl_conn, SASL_SEC_PROPS, &secprops);
 
     if (rc != SASL_OK) {
@@ -1247,7 +1359,7 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
      */
 
     rc = sasl_client_start(nsc->sasl_conn,
-    			   nsc->sasl_mech ? nsc->sasl_mech : mechlist, NULL,
+			   nsc->sasl_mech ? nsc->sasl_mech : mechlist, NULL,
 			   (const char **) &saslbuf, &saslbuflen,
 			   &chosen_mech);
 
@@ -1259,8 +1371,11 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
 
     nsc->sasl_chosen_mech = getcpy(chosen_mech);
 
+    if (nsc->ns_snoop)
+	fprintf(stderr, "Chosen sasl mechanism: %s\n", chosen_mech);
+
     if (nsc->sasl_proto_cb(NETSEC_SASL_START, saslbuf, saslbuflen, NULL, 0,
-			   errstr) != OK)
+			   nsc->sasl_proto_context, errstr) != OK)
 	return NOTOK;
 
     /*
@@ -1268,32 +1383,36 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
      */
 
     while (rc == SASL_CONTINUE) {
-    	/*
+	/*
 	 * Call our SASL callback, which will handle the details of
 	 * reading data from the network.
 	 */
 
 	if (nsc->sasl_proto_cb(NETSEC_SASL_READ, NULL, 0, &outbuf, &outbuflen,
-			       errstr) != OK) {
-	    nsc->sasl_proto_cb(NETSEC_SASL_CANCEL, NULL, 0, NULL, 0, NULL);
+			       nsc->sasl_proto_context, errstr) != OK) {
+	    nsc->sasl_proto_cb(NETSEC_SASL_CANCEL, NULL, 0, NULL, 0,
+			       nsc->sasl_proto_context, NULL);
 	    return NOTOK;
 	}
 
 	rc = sasl_client_step(nsc->sasl_conn, (char *) outbuf, outbuflen, NULL,
 			      (const char **) &saslbuf, &saslbuflen);
 
-        mh_xfree(outbuf);
+        free(outbuf);
 
 	if (rc != SASL_OK && rc != SASL_CONTINUE) {
 	    netsec_err(errstr, "SASL client negotiation failed: %s",
 		       sasl_errdetail(nsc->sasl_conn));
-	    nsc->sasl_proto_cb(NETSEC_SASL_CANCEL, NULL, 0, NULL, 0, NULL);
+	    nsc->sasl_proto_cb(NETSEC_SASL_CANCEL, NULL, 0, NULL, 0,
+			       nsc->sasl_proto_context, NULL);
 	    return NOTOK;
 	}
 
 	if (nsc->sasl_proto_cb(NETSEC_SASL_WRITE, saslbuf, saslbuflen,
-			       NULL, 0, errstr) != OK) {
-	    nsc->sasl_proto_cb(NETSEC_SASL_CANCEL, NULL, 0, NULL, 0, NULL);
+			       NULL, 0, nsc->sasl_proto_context,
+			       errstr) != OK) {
+	    nsc->sasl_proto_cb(NETSEC_SASL_CANCEL, NULL, 0, NULL, 0,
+			       nsc->sasl_proto_context, NULL);
 	    return NOTOK;
 	}
     }
@@ -1304,7 +1423,7 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
      */
 
     if (nsc->sasl_proto_cb(NETSEC_SASL_FINISH, NULL, 0, NULL, 0,
-			   errstr) != OK) {
+			   nsc->sasl_proto_context, errstr) != OK) {
 	/*
 	 * At this point we can't really send an abort since the SASL dialog
 	 * has completed, so just bubble back up the error message.
@@ -1329,6 +1448,10 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
     nsc->sasl_ssf = *ssf;
 
     if (nsc->sasl_ssf > 0) {
+	if (nsc->ns_snoop)
+	    fprintf(stderr, "SASL security layer negotiated, SASL will "
+		    "perform encryption\n");
+
 	rc = sasl_getprop(nsc->sasl_conn, SASL_MAXOUTBUF,
 			  (const void **) &outbufmax);
 
@@ -1381,6 +1504,15 @@ netsec_negotiate_sasl(netsec_context *nsc, const char *mechlist, char **errstr)
 	}
 
 	nsc->sasl_seclayer = 1;
+    } else if (nsc->ns_snoop) {
+	fprintf(stderr, "SASL Security layer NOT negotiated, SASL will NOT "
+		"perform encryption\n");
+#ifdef TLS_SUPPORT
+	if (nsc->tls_active) {
+	    fprintf(stderr, "Encryption will be handled by TLS\n");
+	} else
+#endif /* TLS_SUPPORT */
+	    fprintf(stderr, "Connection will NOT be encrypted, use caution\n");
     }
 
     return OK;
@@ -1412,6 +1544,21 @@ netsec_get_sasl_mechanism(netsec_context *nsc)
     return nsc->sasl_chosen_mech;
 }
 
+/*
+ * Return the negotiated SASL strength security factor (SSF)
+ */
+
+int
+netsec_get_sasl_ssf(netsec_context *nsc)
+{
+#ifdef CYRUS_SASL
+    return nsc->sasl_ssf;
+#else /* CYRUS_SASL */
+    NMH_UNUSED(nsc);
+    return 0;
+#endif /* CYRUS_SASL */
+}
+
 /*
  * Set an OAuth2 service name, if we support it.
  */
@@ -1469,7 +1616,7 @@ netsec_set_tls(netsec_context *nsc, int tls, int noverify, char **errstr)
 		return NOTOK;
 	    }
 
-	    tls_initialized++;
+	    tls_initialized = true;
 	}
 
 	if (nsc->ns_readfd == -1 || nsc->ns_writefd == -1) {
@@ -1477,6 +1624,11 @@ netsec_set_tls(netsec_context *nsc, int tls, int noverify, char **errstr)
 	    return NOTOK;
 	}
 
+	if (!nsc->ns_hostname) {
+	    netsec_err(errstr, "Internal error: hostname not set");
+	    return NOTOK;
+	}
+
 	/*
 	 * Create the SSL structure which holds the data for a single
 	 * TLS connection.
@@ -1533,6 +1685,12 @@ netsec_set_tls(netsec_context *nsc, int tls, int noverify, char **errstr)
 	SSL_set_bio(ssl, rbio, wbio);
 	SSL_set_connect_state(ssl);
 
+	/*
+	 * Use the hostname to set the Server Name Indicator extension
+	 */
+
+	SSL_set_tlsext_host_name(ssl, nsc->ns_hostname);
+
 	/*
 	 * If noverify is NOT set, then do certificate validation.
 	 * Turning on SSL_VERIFY_PEER will verify the certificate chain
@@ -1549,12 +1707,6 @@ netsec_set_tls(netsec_context *nsc, int tls, int noverify, char **errstr)
 #endif /* HAVE_X509_VERIFY_PARAM_SET1_HOST */
 
 	    SSL_set_verify(ssl, SSL_VERIFY_PEER, NULL);
-	    if (! nsc->ns_hostname) {
-		netsec_err(errstr, "Internal error: hostname not set and "
-			   "certification verification enabled");
-		SSL_free(ssl);
-		return NOTOK;
-	    }
 
 #ifdef HAVE_X509_VERIFY_PARAM_SET1_HOST
 	    param = SSL_get0_param(ssl);
@@ -1573,7 +1725,7 @@ netsec_set_tls(netsec_context *nsc, int tls, int noverify, char **errstr)
 
 	if (! ssl_bio) {
 	    netsec_err(errstr, "Unable to create a SSL BIO: %s",
-	    	       ERR_error_string(ERR_get_error(), NULL));
+		       ERR_error_string(ERR_get_error(), NULL));
 	    SSL_free(ssl);
 	    return NOTOK;
 	}
@@ -1698,7 +1850,7 @@ netsec_err(char **errstr, const char *fmt, ...)
     int rc = 127;
 
     if (! errstr)
-    	return;
+	return;
 
     do {
 	errbufsize = rc + 1;
