@@ -5,8 +5,10 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/signals.h>
+#include "h/mh.h"
+#include "pidwait.h"
+#include <signal.h>
+#include "signals.h"
 
 int
 pidwait (pid_t id, int sigsok)
@@ -31,5 +33,5 @@ pidwait (pid_t id, int sigsok)
 	SIGNAL (SIGQUIT, qstat);
     }
 
-    return (pid == -1 ? -1 : status);
+    return pid == -1 ? -1 : status;
 }
