@@ -5,7 +5,9 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "config/version.h"
+#include "h/mh.h"
+#include "print_version.h"
 
 
 void
