@@ -5,7 +5,10 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "read_yes_or_no_if_tty.h"
+#include "read_switch.h"
+#include "globals.h"
 
 
 int
@@ -16,5 +19,5 @@ read_yes_or_no_if_tty (const char *prompt)
     if (interactive < 0)
         interactive = isatty (fileno (stdin));
 
-    return (interactive ? read_switch (prompt, anoyes) : 1);
+    return interactive ? read_switch(prompt, anoyes) : 1;
 }
