@@ -1,13 +1,15 @@
-
-/*
- * seq_add.c -- add message(s) to a sequence
+/* seq_add.c -- add message(s) to a sequence
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "seq_nameok.h"
+#include "seq_add.h"
+#include "error.h"
+#include "globals.h"
 
 
 /*
@@ -26,7 +28,7 @@ int
 seq_addsel (struct msgs *mp, char *cp, int public, int zero)
 {
     unsigned int i;
-    int msgnum, new_seq = 1;
+    int msgnum;
 
     if (!seq_nameok (cp))
 	return 0;
@@ -41,9 +43,10 @@ seq_addsel (struct msgs *mp, char *cp, int public, int zero)
     /*
      * Get the number for this sequence
      */
+    bool new_seq = true;
     for (i = 0; i < svector_size (mp->msgattrs); i++) {
 	if (!strcmp (svector_at (mp->msgattrs, i), cp)) {
-	    new_seq = 0;
+	    new_seq = false;
 	    break;
 	}
     }
@@ -53,7 +56,7 @@ seq_addsel (struct msgs *mp, char *cp, int public, int zero)
      */
     if (new_seq) {
 	if (!(svector_push_back (mp->msgattrs, strdup (cp)))) {
-	    advise (NULL, "strdup failed");
+	    inform("strdup failed");
 	    return 0;
 	}
     }
@@ -114,7 +117,7 @@ int
 seq_addmsg (struct msgs *mp, char *cp, int msgnum, int public, int zero)
 {
     unsigned int i;
-    int j, new_seq = 1;
+    int j;
 
     if (!seq_nameok (cp))
 	return 0;
@@ -128,9 +131,10 @@ seq_addmsg (struct msgs *mp, char *cp, int msgnum, int public, int zero)
     /*
      * Get the number for this sequence
      */
+    bool new_seq = true;
     for (i = 0; i < svector_size (mp->msgattrs); i++) {
 	if (!strcmp (svector_at (mp->msgattrs, i), cp)) {
-	    new_seq = 0;
+	    new_seq = false;
 	    break;
 	}
     }
@@ -140,7 +144,7 @@ seq_addmsg (struct msgs *mp, char *cp, int msgnum, int public, int zero)
      */
     if (new_seq) {
 	if (!(svector_push_back (mp->msgattrs, strdup (cp)))) {
-	    advise (NULL, "strdup failed");
+	    inform("strdup failed");
 	    return 0;
 	}
     }
