@@ -6,7 +6,8 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "seq_getnum.h"
 
 
 int
