@@ -1,13 +1,13 @@
-
-/*
- * seq_nameok.c -- check if a sequence name is ok
+/* seq_nameok.c -- check if a sequence name is ok
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "seq_nameok.h"
+#include "error.h"
 
 
 int
@@ -16,7 +16,7 @@ seq_nameok (char *s)
     char *pp;
 
     if (s == NULL || *s == '\0') {
-	advise (NULL, "empty sequence name");
+	inform("empty sequence name");
 	return 0;
     }
 
@@ -30,7 +30,7 @@ seq_nameok (char *s)
 	  strcmp (s, "last") &&
 	  strcmp (s, "prev") &&
 	  strcmp (s, "next"))) {
-	advise (NULL, "illegal sequence name: %s", s);
+        inform("sequence name is reserved: %s", s);
 	return 0;
     }
 
@@ -39,7 +39,7 @@ seq_nameok (char *s)
      * an alphabetic character ...
      */
     if (!isalpha ((unsigned char) *s)) {
-	advise (NULL, "illegal sequence name: %s", s);
+        inform("sequence name must start with letter: %s", s);
 	return 0;
     }
 
@@ -48,7 +48,7 @@ seq_nameok (char *s)
      */
     for (pp = s + 1; *pp; pp++)
 	if (!isalnum ((unsigned char) *pp)) {
-	    advise (NULL, "illegal sequence name: %s", s);
+            inform("sequence name must be alphanumeric: %s", s);
 	    return 0;
 	}
 
