@@ -6,9 +6,17 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "m_getfld.h"
+#include "seq_read.h"
+#include "trimcpy.h"
+#include "getcpy.h"
+#include "m_atoi.h"
+#include "brkstring.h"
+#include "error.h"
+#include "utils.h"
 #include "lock_file.h"
+#include "globals.h"
 
 /*
  * static prototypes
@@ -63,7 +71,7 @@ seq_public (struct msgs *mp, int lockflag, int *failed_to_lock)
     char *cp, seqfile[PATH_MAX];
     char name[NAMESZ], field[NMH_BUFSIZ];
     FILE *fp;
-    m_getfld_state_t gstate = 0;
+    m_getfld_state_t gstate;
 
     /*
      * If mh_seq == NULL or if *mh_seq == '\0' (the user has defined
@@ -80,17 +88,18 @@ seq_public (struct msgs *mp, int lockflag, int *failed_to_lock)
 	== NULL)
 	return NOTOK;
 
-    /* Use m_getfld to scan sequence file */
+    /* Use m_getfld2 to scan sequence file */
+    gstate = m_getfld_state_init(fp);
     for (;;) {
 	int fieldsz = sizeof field;
-	switch (state = m_getfld (&gstate, name, field, &fieldsz, fp)) {
+	switch (state = m_getfld2(&gstate, name, field, &fieldsz)) {
 	    case FLD: 
 	    case FLDPLUS:
 		if (state == FLDPLUS) {
 		    cp = mh_xstrdup(field);
 		    while (state == FLDPLUS) {
 			fieldsz = sizeof field;
-			state = m_getfld (&gstate, name, field, &fieldsz, fp);
+			state = m_getfld2(&gstate, name, field, &fieldsz);
 			cp = add (field, cp);
 		    }
 		    seq_init (mp, mh_xstrdup(name), trimcpy (cp));
@@ -101,16 +110,16 @@ seq_public (struct msgs *mp, int lockflag, int *failed_to_lock)
 		continue;
 
 	    case BODY:
-	    	lkfclosedata (fp, seqfile);
-		adios (NULL, "no blank lines are permitted in %s", seqfile);
+		lkfclosedata (fp, seqfile);
+		die("no blank lines are permitted in %s", seqfile);
 		break;
 
 	    case FILEEOF:
 		break;
 
 	    default: 
-	    	lkfclosedata (fp, seqfile);
-		adios (NULL, "%s is poorly formatted", seqfile);
+		lkfclosedata (fp, seqfile);
+		die("%s is poorly formatted", seqfile);
 	}
 	break;	/* break from for loop */
     }
@@ -145,7 +154,7 @@ seq_private (struct msgs *mp)
     plen = strlen (mp->foldpath) + 1;
 
     for (np = m_defs; np; np = np->n_next) {
-	if (ssequal ("atr-", np->n_name)
+	if (has_prefix(np->n_name, "atr-")
 		&& (j = strlen (np->n_name) - plen) > alen
 		&& *(np->n_name + j) == '-'
 		&& strcmp (mp->foldpath, np->n_name + j + 1) == 0) {
