@@ -6,10 +6,17 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/signals.h>
+#include "h/mh.h"
+#include "seq_save.h"
+#include "context_replace.h"
+#include "context_del.h"
+#include "seq_list.h"
+#include "error.h"
+#include <signal.h>
+#include "signals.h"
 #include "lock_file.h"
 #include "m_mktemp.h"
+#include "globals.h"
 
 
 /*
@@ -31,7 +38,7 @@ seq_save (struct msgs *mp)
 
     /* check if sequence information has changed */
     if (!(mp->msgflags & SEQMOD)) {
-    	if (mp->seqhandle) {
+	if (mp->seqhandle) {
 	    lkfclosedata (mp->seqhandle, mp->seqname);
 	    mp->seqhandle = NULL;
 	    free(mp->seqname);
