@@ -5,8 +5,9 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/signals.h>
+#include "h/mh.h"
+#include <signal.h>
+#include "signals.h"
 #include "m_mktemp.h"
 
 
@@ -39,8 +40,8 @@ SIGNAL (int sig, SIGNAL_HANDLER func)
 # endif
     }
     if (sigaction(sig, &act, &oact) < 0)
-	return (SIG_ERR);
-    return (oact.sa_handler);
+	return SIG_ERR;
+    return oact.sa_handler;
 }
 
 
@@ -56,7 +57,7 @@ SIGNAL2 (int sig, SIGNAL_HANDLER func)
     struct sigaction act, oact;
 
     if (sigaction(sig, NULL, &oact) < 0)
-	return (SIG_ERR);
+	return SIG_ERR;
     if (oact.sa_handler != SIG_IGN) {
 	act.sa_handler = func;
 	sigemptyset(&act.sa_mask);
@@ -72,9 +73,9 @@ SIGNAL2 (int sig, SIGNAL_HANDLER func)
 # endif
 	}
 	if (sigaction(sig, &act, &oact) < 0)
-	    return (SIG_ERR);
+	    return SIG_ERR;
     }
-    return (oact.sa_handler);
+    return oact.sa_handler;
 }
 
 
@@ -82,7 +83,8 @@ SIGNAL2 (int sig, SIGNAL_HANDLER func)
  * For use by nmh_init().
  */
 int
-setup_signal_handlers(void) {
+setup_signal_handlers(void)
+{
     /*
      * Catch HUP, INT, QUIT, and TERM so that we can clean up tmp
      * files when the user terminates the process early.  And also a
