@@ -1,24 +1,25 @@
-
-/*
- * snprintb.c -- snprintf a %b string
+/* snprintb.c -- snprintf a %b string
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "snprintb.h"
 
 
 char *
 snprintb (char *buffer, size_t n, unsigned v, char *bits)
 {
-    register int i, j;
-    register char c, *bp;
+    size_t len;
+    int i, j;
+    char c, *bp;
 
     snprintf (buffer, n, bits && *bits == 010 ? "0%o" : "0x%x", v);
-    bp = buffer + strlen(buffer);
-    n -= strlen(buffer);
+    len = strlen(buffer);
+    bp = buffer + len;
+    n -= len;
 
     if (bits && *++bits) {
 	j = 0;
@@ -33,8 +34,7 @@ snprintb (char *buffer, size_t n, unsigned v, char *bits)
 		    *bp++ = c;
 		    n--;
 		}
-	    }
-	    else
+	    } else
 		for (; *bits > 32; bits++)
 		    continue;
 	if (n > 1)
