@@ -6,7 +6,7 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
 #include "unquote.h"
 
 /*
