@@ -5,7 +5,8 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "uprf.h"
 
 
 /* uprf returns true if s starts with prefix, ignoring case.
