@@ -5,8 +5,10 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "vfgets.h"
+#include "error.h"
+#include "utils.h"
 
 #define	QUOTE	'\\'
 
@@ -28,13 +30,13 @@ vfgets (FILE *in, char **bp)
 		*bp = pp;
 		return 0;
 	    }
-	    return (ferror (in) && !feof (in) ? -1 : 1);
+	    return ferror(in) && !feof(in) ? -1 : 1;
 	}
 
 	if ((dp = cp + strlen (cp) - 2) < cp || *dp != QUOTE) {
 wrong_guess:
 	    if (cp > ++dp)
-		adios (NULL, "vfgets() botch -- you lose big");
+		die("vfgets() botch -- you lose big");
 	    if (*dp == '\n') {
 		*bp = pp;
 		return 0;
@@ -44,8 +46,7 @@ wrong_guess:
 	    for (fp = dp - 1, toggle = 0; fp >= cp; fp--) {
 		if (*fp != QUOTE)
 		    break;
-		else
-		    toggle = !toggle;
+                toggle = !toggle;
 	    }
 	    if (toggle)
 		goto wrong_guess;
