@@ -14,6 +14,12 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "server.h"
+
+#ifndef min
+# define min(a,b) ((a) < (b) ? (a) : (b))
+#endif
+
 #define PIDFILE "/tmp/fakesmtp.pid"
 
 #define LINESIZE 1024
@@ -29,9 +35,6 @@ enum {
 	SMTP_XOAUTH_ERR
 };
 
-void putcrlf(int, char *);
-int serve(const char *, const char *);
-
 static int getsmtp(int, char *);
 
 int
@@ -70,7 +73,8 @@ main(int argc, char *argv[])
 		if (rc == -1)
 			break;	/* EOF */
 
-		fprintf(f, "%s\n", line);
+                fputs(line, f);
+                putc('\n', f);
 
 		switch (smtp_state) {
 		case SMTP_DATA:
@@ -154,9 +158,9 @@ getsmtp(int socket, char *data)
 		if (bytesinbuf > 0 && (p = strchr(buffer, '\r')) &&
 							*(p + 1) == '\n') {
 			*p = '\0';
-			strncpy(data, buffer, LINESIZE);
-			data[LINESIZE - 1] = '\0';
 			cc = strlen(buffer);
+			memcpy(data, buffer, min(cc + 1, LINESIZE));
+			data[LINESIZE - 1] = '\0';
 
 			/*
 			 * Shuffle leftover bytes back to the beginning
