@@ -21,9 +21,12 @@ main(int argc, char *argv[])
 	struct passwd *pwd;
 	char buf[BUFSIZ], *p;
 
+        if (argc > 2) {
+		fprintf (stderr, "usage: %s [name]\n", argv[0]);
+		return 1;
+	}
 	if (argc < 2) {
 		pwd = getpwuid(getuid());
-
 		if (! pwd) {
 			fprintf(stderr, "Unable to retrieve user info for "
 				"userid %ld\n", (long) getuid());
@@ -32,12 +35,8 @@ main(int argc, char *argv[])
 
 		strncpy(buf, pwd->pw_gecos, sizeof(buf));
 		buf[sizeof(buf) - 1] = '\0';
-	} else if (argc == 2) {
-		strncpy(buf, argv[1], sizeof(buf));
-	} else if (argc > 2) {
-		fprintf (stderr, "usage: %s [name]\n", argv[0]);
-		return 1;
-	}
+	} else
+		strncpy(buf, argv[1], sizeof(buf) - 1);
 
 	/*
 	 * Perform the same processing that getuserinfo() does.
@@ -53,8 +52,7 @@ main(int argc, char *argv[])
 	 * Quote the entire string if it has a special character in it.
 	 */
 	escape_display_name (buf, sizeof(buf));
-
-	printf("%s\n", buf);
+	puts(buf);
 
 	exit(0);
 }
