@@ -5,11 +5,22 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/addrsbr.h>
-#include <h/aliasbr.h>
-#include <h/mts.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "sbr/getarguments.h"
+#include "sbr/smatch.h"
+#include "sbr/getcpy.h"
+#include "sbr/context_find.h"
+#include "sbr/brkstring.h"
+#include "sbr/ambigsw.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
+#include "sbr/addrsbr.h"
+#include "aliasbr.h"
+#include "sbr/mts.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
+#include "sbr/globals.h"
 
 #define ALI_SWITCHES \
     X("alias aliasfile", 0, ALIASW) \
@@ -31,32 +42,31 @@ DEFINE_SWITCH_ARRAY(ALI, switches);
 
 static int pos = 1;
 
-extern struct aka *akahead;
-
 /*
  * prototypes
  */
-static void print_aka (char *, int, int);
-static void print_usr (char *, int);
+static void print_aka (char *, bool, int);
+static void print_usr (char *, bool);
 
 
 int
 main (int argc, char **argv)
 {
-    int i, vecp = 0, inverted = 0, list = 0;
-    int noalias = 0;
+    int i, vecp = 0;
+    bool inverted, list, noalias;
     char *cp, **ap, **argp, buf[BUFSIZ];
     /* Really only need to allocate for argc-1, but must allocate at least 1,
        so go ahead and allocate for argc char pointers. */
     char **vec = mh_xmalloc (argc * sizeof (char *)), **arguments;
     struct aka *ak;
 
-    if (nmh_init(argv[0], 1)) { return 1; }
+    if (nmh_init(argv[0], true, true)) { return 1; }
 
     mts_init ();
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    inverted = list = noalias = false;
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -64,7 +74,7 @@ main (int argc, char **argv)
 		    ambigsw (cp, switches);
 		    done (1);
 		case UNKWNSW: 
-		    adios (NULL, "-%s unknown", cp);
+		    die("-%s unknown", cp);
 
 		case HELPSW: 
 		    snprintf (buf, sizeof(buf), "%s [switches] aliases ...",
@@ -77,26 +87,26 @@ main (int argc, char **argv)
 
 		case ALIASW: 
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    if ((i = alias (cp)) != AK_OK)
-			adios (NULL, "aliasing error in %s - %s", cp, akerror (i));
+			die("aliasing error in %s - %s", cp, akerror (i));
 		    continue;
 		case NALIASW: 
-		    noalias++;
+		    noalias = true;
 		    continue;
 
 		case LISTSW: 
-		    list++;
+		    list = true;
 		    continue;
 		case NLISTSW: 
-		    list = 0;
+		    list = false;
 		    continue;
 
 		case USERSW: 
-		    inverted++;
+		    inverted = true;
 		    continue;
 		case NUSERSW: 
-		    inverted = 0;
+		    inverted = false;
 		    continue;
 	    }
 	}
@@ -106,7 +116,7 @@ main (int argc, char **argv)
 	} else {
 	    /* Should never happen, but try to protect against code changes
 	       that could allow it. */
-	    adios (NULL, "too many arguments");
+	    die("too many arguments");
 	}
     }
 
@@ -117,8 +127,8 @@ main (int argc, char **argv)
 
 	    for (ap = brkstring(dp = mh_xstrdup(cp), " ", "\n"); ap && *ap; ap++)
 		if ((i = alias (*ap)) != AK_OK)
-		    adios (NULL, "aliasing error in %s - %s", *ap, akerror (i));
-            mh_xfree(dp);
+		    die("aliasing error in %s - %s", *ap, akerror (i));
+            free(dp);
 	}
 	alias (AliasFile);
     }
@@ -128,7 +138,7 @@ main (int argc, char **argv)
      */
     if (inverted) {
 	if (vecp == 0)
-	    adios (NULL, "usage: %s -user addresses ...  (you forgot the addresses)",
+	    die("usage: %s -user addresses ...  (you forgot the addresses)",
 		   invo_name);
 
 	for (i = 0; i < vecp; i++)
@@ -158,7 +168,7 @@ main (int argc, char **argv)
 }
 
 static void
-print_aka (char *p, int list, int margin)
+print_aka (char *p, bool list, int margin)
 {
     char c;
 
@@ -175,10 +185,10 @@ print_aka (char *p, int list, int margin)
 			printf ("\n%*s", margin, "");
 		    else {
 			if (pos >= 68) {
-			    printf (",\n ");
+			    fputs(",\n ", stdout);
 			    pos = 2;
 			} else {
-			    printf (", ");
+			    fputs(", ", stdout);
 			    pos += 2;
 			}
 		    }
@@ -198,16 +208,16 @@ print_aka (char *p, int list, int margin)
 }
 
 static void
-print_usr (char *s, int list)
+print_usr (char *s, bool list)
 {
     char *cp, *pp, *vp;
     struct aka *ak;
     struct mailname *mp, *np;
 
     if ((pp = getname (s)) == NULL)
-	adios (NULL, "no address in \"%s\"", s);
+	die("no address in \"%s\"", s);
     if ((mp = getm (pp, NULL, 0, NULL, 0)) == NULL)
-	adios (NULL, "bad address \"%s\"", s);
+	die("bad address \"%s\"", s);
     while (getname (""))
 	continue;
 
@@ -235,5 +245,5 @@ print_usr (char *s, int list)
 
     print_aka (vp ? vp : s, list, 0);
 
-    mh_xfree(vp);
+    free(vp);
 }
