@@ -5,30 +5,30 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/aliasbr.h>
-#include <h/addrsbr.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "sbr/concat.h"
+#include "sbr/vfgets.h"
+#include "sbr/getcpy.h"
+#include "aliasbr.h"
+#include "sbr/addrsbr.h"
+#include "sbr/utils.h"
+#include "sbr/path.h"
 #include <pwd.h>
 
 static int akvis;
 static char *akerrst;
 
+/* The first also-known-as struct in the linked list if the list is not empty. */
 struct aka *akahead = NULL;
+/* The last also-known-as struct in the linked list if the list is not empty. */
 struct aka *akatail = NULL;
 
 /*
  * prototypes
  */
-int alias (char *);
-int akvisible (void);
-char *akresult (struct aka *);
-char *akvalue (char *);
-char *akerror (int);
-
 static  char *akval (struct aka *, char *);
-static int aleq (char *, char *);
-static char *scanp (char *);
+static bool aleq (char *, char *) PURE;
+static char *scanp (char *) PURE;
 static char *getp (char *);
 static char *seekp (char *, char *, char **);
 static int addfile (struct aka *, char *);
@@ -61,6 +61,12 @@ akvisible (void)
 }
 
 
+struct adr {
+    char *ad_text;		/* text of this address in list        */
+    struct adr *ad_next;	/* next adr in list                    */
+    char ad_local;		/* text is local (check for expansion) */
+};
+
 char *
 akresult (struct aka *ak)
 {
@@ -76,8 +82,7 @@ akresult (struct aka *ak)
 	    cp = concat (cp, ",", pp, NULL);
 	    free (dp);
 	    free (pp);
-	}
-	else
+	} else
 	    cp = pp;
     }
 
@@ -120,7 +125,7 @@ akval (struct aka *ak, char *s)
 		struct mailname *mp = getm (name, NULL, 0, NULL, 0);
 
 		if (mp	&&  mp->m_ingrp) {
-		    char *gname = add (mp->m_gname, NULL);
+		    char *gname = mh_xstrdup(FENDNULL(mp->m_gname));
 
                     /* FIXME: gname must be true;  add() never returns NULL.
 		     * Is some other test required? */
@@ -147,20 +152,23 @@ akval (struct aka *ak, char *s)
 }
 
 
-static int
+/* Does string match aliasent where the latter may contain a ‘*’ which
+ * matches zero or more characters and stops matching,
+ * e.g. "foobar" matches "foo*xyzzy". */
+static bool
 aleq (char *string, char *aliasent)
 {
     char c;
 
     while ((c = *string++)) {
 	if (*aliasent == '*')
-	    return 1;
+	    return true;
         if (tolower((unsigned char)c) != tolower((unsigned char)*aliasent))
-            return 0;
+            return false;
         aliasent++;
     }
 
-    return (*aliasent == 0 || *aliasent == '*');
+    return *aliasent == 0 || *aliasent == '*';
 }
 
 
@@ -184,7 +192,7 @@ alias (char *file)
     while (vfgets (fp, &ap) == OK) {
 	bp = ap;
 	switch (*(pp = scanp (bp))) {
-	    case '<': 		/* recurse a level */
+	    case '<':		/* recurse a level */
 		if (!*(cp = getp (pp + 1))) {
 		    akerrst = "'<' without alias-file";
 		    fclose (fp);
@@ -195,10 +203,10 @@ alias (char *file)
 		    return i;
 		}
 		continue;
-	    case ':': 		/* comment */
-	    case ';': 
+	    case ':':		/* comment */
+	    case ';':
 	    case '#':
-	    case 0: 
+	    case 0:
 		continue;
 	}
 
@@ -212,25 +220,25 @@ alias (char *file)
 	    return AK_LIMIT;
 	}
 	switch (lc) {
-	    case ':': 
-		ak->ak_visible = 0;
+	    case ':':
+		ak->ak_visible = false;
 		break;
 
-	    case ';': 
-		ak->ak_visible = 1;
+	    case ';':
+		ak->ak_visible = true;
 		break;
 
-	    default: 
+	    default:
 		fclose (fp);
 		return AK_ERROR;
 	}
 
 	switch (*(pp = scanp (ap))) {
-	    case 0: 		/* EOL */
+	    case 0:		/* EOL */
 		fclose (fp);
 		return AK_ERROR;
 
-	    case '<': 		/* read values from file */
+	    case '<':		/* read values from file */
 		if (!*(cp = getp (pp + 1))) {
 		    fclose (fp);
 		    return AK_ERROR;
@@ -241,7 +249,7 @@ alias (char *file)
 		}
 		break;
 
-	    default: 		/* list */
+	    default:		/* list */
 		while ((cp = getalias (pp)))
 		    add_aka (ak, cp);
 		break;
@@ -253,25 +261,27 @@ alias (char *file)
 }
 
 
+/* Return a pointer to a static string holding a formatted error message
+ * for the AK_* value in ‘i’ using akerrst. */
 char *
 akerror (int i)
 {
     static char buffer[BUFSIZ];
 
     switch (i) {
-	case AK_NOFILE: 
+	case AK_NOFILE:
 	    snprintf (buffer, sizeof(buffer), "unable to read '%s'", akerrst);
 	    break;
 
-	case AK_ERROR: 
+	case AK_ERROR:
 	    snprintf (buffer, sizeof(buffer), "error in line '%s'", akerrst);
 	    break;
 
-	case AK_LIMIT: 
+	case AK_LIMIT:
 	    snprintf (buffer, sizeof(buffer), "out of memory while on '%s'", akerrst);
 	    break;
 
-	default: 
+	default:
 	    snprintf (buffer, sizeof(buffer), "unknown error (%d)", i);
 	    break;
     }
@@ -280,6 +290,8 @@ akerror (int i)
 }
 
 
+/* Skip zero or more isspace(3) bytes and return a pointer to the next
+ * byte. */
 static char *
 scanp (char *p)
 {
@@ -289,6 +301,9 @@ scanp (char *p)
 }
 
 
+/* Skip zero or more isspace(3) bytes and return a pointer to the next,
+ * possibly NUL, byte having terminated that run of zero or more
+ * non-isspace(3) bytes with a NUL.  "\t\tfoo bar" returns "foo\0bar". */
 static char *
 getp (char *p)
 {
@@ -303,6 +318,11 @@ getp (char *p)
 }
 
 
+/* Skip zero or more isspace(3) bytes and return a pointer to the next,
+ * possibly NUL, byte having terminated that run of zero or more
+ * non-isspace(3), non-colon, non-semicolon bytes with a NUL.
+ * "\t\tfoo bar; xyzzy" returns "foo bar\0 xyzzy" and sets ‘c’ to ';'
+ * and ‘*a’ to " xyzzy". */
 static char *
 seekp (char *p, char *c, char **a)
 {
@@ -323,7 +343,7 @@ static int
 addfile (struct aka *ak, char *file)
 {
     char *cp;
-    char buffer[BUFSIZ];
+    static char buffer[BUFSIZ];
     FILE *fp;
 
     if (!(fp = fopen (etcpath (file), "r"))) {
@@ -348,15 +368,14 @@ getalias (char *addrs)
 
     if (cp == NULL)
 	cp = addrs;
-    else
-	if (*cp == 0)
-	    return (cp = NULL);
+    else if (*cp == 0)
+        return cp = NULL;
 
     /* Remove leading any space from the address. */
     for (pp = cp; isspace ((unsigned char) *pp); pp++)
 	continue;
     if (*pp == 0)
-	return (cp = NULL);
+	return cp = NULL;
     /* Find the end of the address. */
     for (qp = pp; *qp != 0 && *qp != ','; qp++)
 	continue;
@@ -388,13 +407,16 @@ add_aka (struct aka *ak, char *pp)
     ad->ad_text = mh_xstrdup(pp);
     ad->ad_local = strchr(pp, '@') == NULL && strchr(pp, '!') == NULL;
     ad->ad_next = NULL;
-    if (ak->ak_addr)
+    if (ak->ak_addr) {
+	assert (ld != NULL);
 	ld->ad_next = ad;
-    else
+    } else
 	ak->ak_addr = ad;
 }
 
 
+/* Appends a new also-known-as struct onto the end of the akahead linked
+ * list and returns the new struct. */
 static struct aka *
 akalloc (char *id)
 {
@@ -402,7 +424,7 @@ akalloc (char *id)
 
     NEW(p);
     p->ak_name = getcpy (id);
-    p->ak_visible = 0;
+    p->ak_visible = false;
     p->ak_addr = NULL;
     p->ak_next = NULL;
     if (akatail != NULL)
