@@ -5,10 +5,37 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
-#include "../sbr/m_maildir.h"
+#include "h/mh.h"
+#include "whatnowproc.h"
+#include "sbr/m_name.h"
+#include "sbr/m_gmprot.h"
+#include "sbr/getarguments.h"
+#include "sbr/read_switch_multiword.h"
+#include "sbr/seq_setprev.h"
+#include "sbr/seq_setcur.h"
+#include "sbr/seq_save.h"
+#include "sbr/showfile.h"
+#include "sbr/smatch.h"
+#include "sbr/refile.h"
+#include "sbr/cpydata.h"
+#include "sbr/m_draft.h"
+#include "sbr/m_convert.h"
+#include "sbr/getfolder.h"
+#include "sbr/folder_read.h"
+#include "sbr/context_save.h"
+#include "sbr/context_replace.h"
+#include "sbr/context_find.h"
+#include "sbr/ambigsw.h"
+#include "sbr/path.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
+#include "sbr/utils.h"
+#include "sbr/m_maildir.h"
 #include <fcntl.h>
+#include "sbr/done.h"
+#include "forwsbr.h"
+#include "sbr/globals.h"
 
 #define DIST_SWITCHES \
     X("annotate", 0, ANNOSW) \
@@ -70,10 +97,14 @@ static struct swit aqrl[] = {
 int
 main (int argc, char **argv)
 {
-    int anot = 0, inplace = 1, nedit = 0;
-    int nwhat = 0, i, in, isdf = 0, out;
+    bool anot = false;
+    bool inplace = true;
+    bool nedit = false;
+    bool nwhat = false;
+    int i, in, isdf = 0, out;
     int outputlinelen = OUTPUTLINELEN;
-    int dat[5], atfile = 0;
+    int dat[5];
+    bool atfile = false;
     char *cp, *cwd, *maildir, *msgnam, *dfolder = NULL;
     char *dmsg = NULL, *ed = NULL, *file = NULL, *folder = NULL;
     char *form = NULL, *msg = NULL, buf[BUFSIZ], drft[BUFSIZ];
@@ -82,7 +113,7 @@ main (int argc, char **argv)
     struct msgs *mp = NULL;
     struct stat st;
 
-    if (nmh_init(argv[0], 1)) { return 1; }
+    if (nmh_init(argv[0], true, true)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -94,7 +125,7 @@ main (int argc, char **argv)
 		    ambigsw (cp, switches);
 		    done (1);
 		case UNKWNSW: 
-		    adios (NULL, "-%s unknown", cp);
+		    die("-%s unknown", cp);
 
 		case HELPSW: 
 		    snprintf (buf, sizeof(buf), "%s [+folder] [msg] [switches]",
@@ -106,62 +137,62 @@ main (int argc, char **argv)
 		    done (0);
 
 		case ANNOSW: 
-		    anot++;
+		    anot = true;
 		    continue;
 		case NANNOSW: 
-		    anot = 0;
+		    anot = false;
 		    continue;
 
 		case EDITRSW: 
 		    if (!(ed = *argp++) || *ed == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
-		    nedit = 0;
+			die("missing argument to %s", argp[-2]);
+		    nedit = false;
 		    continue;
 		case NEDITSW:
-		    nedit++;
+		    nedit = true;
 		    continue;
 		    
 		case WHATSW: 
 		    if (!(whatnowproc = *argp++) || *whatnowproc == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
-		    nwhat = 0;
+			die("missing argument to %s", argp[-2]);
+		    nwhat = false;
 		    continue;
 		case NWHATSW: 
-		    nwhat++;
+		    nwhat = true;
 		    continue;
 
 		case FILESW: 
 		    if (file)
-			adios (NULL, "only one file at a time!");
+			die("only one file at a time!");
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    file = path (cp, TFILE);
 		    continue;
 		case FORMSW: 
 		    if (!(form = *argp++) || *form == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 
 		case INPLSW: 
-		    inplace++;
+		    inplace = true;
 		    continue;
 		case NINPLSW: 
-		    inplace = 0;
+		    inplace = false;
 		    continue;
 
 		case DFOLDSW: 
 		    if (dfolder)
-			adios (NULL, "only one draft folder at a time!");
+			die("only one draft folder at a time!");
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    dfolder = path (*cp == '+' || *cp == '@' ? cp + 1 : cp,
 			    *cp != '@' ? TFOLDER : TSUBCWF);
 		    continue;
 		case DMSGSW: 
 		    if (dmsg)
-			adios (NULL, "only one draft message at a time!");
+			die("only one draft message at a time!");
 		    if (!(dmsg = *argp++) || *dmsg == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 		case NDFLDSW: 
 		    dfolder = NULL;
@@ -170,50 +201,48 @@ main (int argc, char **argv)
 
 		case FROMSW:
 		    if (!(cp = *argp++) || *cp == '-')
-		    	adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    from = addlist(from, cp);
 		    continue;
 		case TOSW:
 		    if (!(cp = *argp++) || *cp == '-')
-		    	adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    to = addlist(to, cp);
 		    continue;
 		case CCSW:
 		    if (!(cp = *argp++) || *cp == '-')
-		    	adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    cc = addlist(cc, cp);
 		    continue;
 		case FCCSW:
 		    if (!(cp = *argp++) || *cp == '-')
-		    	adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    fcc = addlist(fcc, cp);
 		    continue;
 
 		case WIDTHSW:
 		    if (!(cp = *argp++) || *cp == '-')
-		    	adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    if ((outputlinelen = atoi(cp)) < 10)
-		    	adios (NULL, "impossible width %d", outputlinelen);
+			die("impossible width %d", outputlinelen);
 		    continue;
 
 		case ATFILESW:
-		    atfile++;
+		    atfile = true;
 		    continue;
 		case NOATFILESW:
-		    atfile = 0;
+		    atfile = false;
 		    continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
 	    if (folder)
-		adios (NULL, "only one folder at a time!");
-	    else
-		folder = pluspath (cp);
+		die("only one folder at a time!");
+            folder = pluspath (cp);
 	} else {
 	    if (msg)
-		adios (NULL, "only one message at a time!");
-	    else
-		msg = cp;
+		die("only one message at a time!");
+            msg = cp;
 	}
     }
 
@@ -222,13 +251,13 @@ main (int argc, char **argv)
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
     if (file && (msg || folder))
-	adios (NULL, "can't mix files and folders/msgs");
+	die("can't mix files and folders/msgs");
 
 try_it_again:
-    strncpy (drft, m_draft (dfolder, dmsg, NOUSE, &isdf), sizeof(drft));
+    strncpy (drft, m_draft (dfolder, dmsg, NOUSE, &isdf), sizeof(drft) - 1);
 
     /* Check if draft already exists */
-    if (stat (drft, &st) != NOTOK) {
+    if (stat (drft, &st) != -1) {
 	printf ("Draft \"%s\" exists (%ld bytes).", drft, (long) st.st_size);
 	for (i = LISTDSW; i != YESW;) {
 	    if (!(argp = read_switch_multiword ("\nDisposition? ",
@@ -260,7 +289,7 @@ try_it_again:
 	/*
 	 * Dist a file
 	 */
-	anot = 0;	/* don't want to annotate a file */
+	anot = false;	/* don't want to annotate a file */
     } else {
 	/*
 	 * Dist a message
@@ -271,16 +300,16 @@ try_it_again:
 	    folder = getfolder (1);
 	maildir = m_maildir (folder);
 
-	if (chdir (maildir) == NOTOK)
+	if (chdir (maildir) == -1)
 	    adios (maildir, "unable to change directory to");
 
 	/* read folder and create message structure */
 	if (!(mp = folder_read (folder, 1)))
-	    adios (NULL, "unable to read folder %s", folder);
+	    die("unable to read folder %s", folder);
 
 	/* check for empty folder */
 	if (mp->nummsg == 0)
-	    adios (NULL, "no messages in %s", folder);
+	    die("no messages in %s", folder);
 
 	/* parse the message range/sequence/name and set SELECTED */
 	if (!m_convert (mp, msg))
@@ -288,7 +317,7 @@ try_it_again:
 	seq_setprev (mp);	/* set the previous-sequence */
 
 	if (mp->numsel > 1)
-	    adios (NULL, "only one message at a time!");
+	    die("only one message at a time!");
     }
 
     msgnam = file ? file : mh_xstrdup(m_name (mp->lowsel));
@@ -300,20 +329,17 @@ try_it_again:
     dat[4] = 0;
 
     if (!form)
-    	form = distcomps;
+	form = distcomps;
 
     in = build_form(form, NULL, dat, from, to, cc, fcc, NULL, msgnam);
 
-    if ((out = creat (drft, m_gmprot ())) == NOTOK)
+    if ((out = creat (drft, m_gmprot ())) == -1)
 	adios (drft, "unable to create");
 
     cpydata (in, out, form, drft);
     close (in);
     close (out);
 
-    if ((in = open (msgnam, O_RDONLY)) == NOTOK)
-	adios (msgnam, "unable to open message");
-
     if (!file) {
 	context_replace (pfolder, folder);/* update current folder  */
 	seq_setcur (mp, mp->lowsel);	  /* update current message */
