@@ -7,10 +7,34 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/crawl_folders.h>
-#include <h/utils.h>
-#include "../sbr/m_maildir.h"
+#include "h/mh.h"
+#include "sbr/getarguments.h"
+#include "sbr/concat.h"
+#include "sbr/seq_setprev.h"
+#include "sbr/seq_setcur.h"
+#include "sbr/seq_save.h"
+#include "sbr/smatch.h"
+#include "sbr/getcpy.h"
+#include "sbr/m_convert.h"
+#include "sbr/getfolder.h"
+#include "sbr/folder_read.h"
+#include "sbr/folder_pack.h"
+#include "sbr/folder_free.h"
+#include "sbr/context_save.h"
+#include "sbr/context_replace.h"
+#include "sbr/context_del.h"
+#include "sbr/context_find.h"
+#include "sbr/brkstring.h"
+#include "sbr/ambigsw.h"
+#include "sbr/path.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
+#include "sbr/crawl_folders.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
+#include "sbr/m_maildir.h"
+#include "sbr/globals.h"
 
 #define FOLDER_SWITCHES \
     X("all", 0, ALLSW) \
@@ -46,14 +70,14 @@ DEFINE_SWITCH_ENUM(FOLDER);
 DEFINE_SWITCH_ARRAY(FOLDER, switches);
 #undef X
 
-static int fshort   = 0;	/* output only folder names                 */
+static bool fshort;	        /* output only folder names                 */
 static int fcreat   = 0;	/* should we ask to create new folders?     */
-static int fpack    = 0;	/* are we packing the folder?               */
-static int fverb    = 0;	/* print actions taken while packing folder */
+static bool fpack;		/* are we packing the folder?               */
+static bool fverb;		/* print actions taken while packing folder */
 static int fheader  = 0;	/* should we output a header?               */
-static int frecurse = 0;	/* recurse through subfolders               */
+static bool frecurse;		/* recurse through subfolders               */
 static int ftotal   = 0;	/* should we output the totals?             */
-static int all      = 0;	/* should we output all folders             */
+static bool all;		/* should we output all folders             */
 
 static int total_folders = 0;	/* total number of folders                  */
 
@@ -95,23 +119,25 @@ static void readonly_folders (void);
  * Function for printing error message if folder does not exist with
  * -nocreate.
  */
-static
-void
-nonexistent_folder (int status) {
+static void
+nonexistent_folder (int status)
+{
     NMH_UNUSED (status);
-    adios (NULL, "folder %s does not exist", folder);
+    die("folder %s does not exist", folder);
 }
 
 
 int
 main (int argc, char **argv)
 {
-    int printsw = 0, listsw = 0;
-    int pushsw = 0, popsw = 0;
+    int printsw = -1;
+    bool listsw = false;
+    bool pushsw = false;
+    bool popsw = false;
     char *cp, *dp, *msg = NULL, *argfolder = NULL;
     char **ap, **argp, buf[BUFSIZ], **arguments;
 
-    if (nmh_init(argv[0], 1)) { return 1; }
+    if (nmh_init(argv[0], true, true)) { return 1; }
 
     /*
      * If program was invoked with name ending
@@ -129,7 +155,7 @@ main (int argc, char **argv)
 		    ambigsw (cp, switches);
 		    done (1);
 		case UNKWNSW: 
-		    adios (NULL, "-%s unknown", cp);
+		    die("-%s unknown", cp);
 
 		case HELPSW: 
 		    snprintf (buf, sizeof(buf), "%s [+folder] [msg] [switches]",
@@ -141,11 +167,11 @@ main (int argc, char **argv)
 		    done (0);
 
 		case ALLSW: 
-		    all = 1;
+		    all = true;
 		    continue;
 
 		case NALLSW:
-		    all = 0;
+		    all = false;
 		    continue;
 
 		case CREATSW: 
@@ -156,10 +182,10 @@ main (int argc, char **argv)
 		    continue;
 
 		case FASTSW: 
-		    fshort++;
+		    fshort = true;
 		    continue;
 		case NFASTSW: 
-		    fshort = 0;
+		    fshort = false;
 		    continue;
 
 		case HDRSW: 
@@ -170,24 +196,24 @@ main (int argc, char **argv)
 		    continue;
 
 		case PACKSW: 
-		    fpack++;
+		    fpack = true;
 		    continue;
 		case NPACKSW: 
-		    fpack = 0;
+		    fpack = false;
 		    continue;
 
 		case VERBSW:
-		    fverb++;
+		    fverb = true;
 		    continue;
 		case NVERBSW:
-		    fverb = 0;
+		    fverb = false;
 		    continue;
 
 		case RECURSW: 
-		    frecurse++;
+		    frecurse = true;
 		    continue;
 		case NRECRSW: 
-		    frecurse = 0;
+		    frecurse = false;
 		    continue;
 
 		case TOTALSW: 
@@ -205,34 +231,32 @@ main (int argc, char **argv)
 		    continue;
 
 		case LISTSW: 
-		    listsw = 1;
+		    listsw = true;
 		    continue;
 		case NLISTSW: 
-		    listsw = 0;
+		    listsw = false;
 		    continue;
 
 		case PUSHSW: 
-		    pushsw = 1;
-		    listsw = 1;
-		    popsw  = 0;
+		    pushsw = true;
+		    listsw = true;
+		    popsw  = false;
 		    continue;
 		case POPSW: 
-		    popsw  = 1;
-		    listsw = 1;
-		    pushsw = 0;
+		    popsw  = true;
+		    listsw = true;
+		    pushsw = false;
 		    continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
 	    if (argfolder)
-		adios (NULL, "only one folder at a time!");
-	    else
-		argfolder = pluspath (cp);
+		die("only one folder at a time!");
+            argfolder = pluspath (cp);
 	} else {
 	    if (msg)
-		adios (NULL, "only one (current) message at a time!");
-	    else
-		msg = cp;
+		die("only one (current) message at a time!");
+            msg = cp;
 	}
     }
 
@@ -241,11 +265,13 @@ main (int argc, char **argv)
     nmhdir = concat (m_maildir (""), "/", NULL);
 
     /*
-     * If we aren't working with the folder stack
-     * (-push, -pop, -list) then the default is to print.
+     * If not directed via -print/-noprint, we print folder summary
+     * info unless if we're working with the folder stack (i.e.,
+     * -push, -pop, or -list).
      */
-    if (pushsw == 0 && popsw == 0 && listsw == 0)
-	printsw++;
+    if (printsw == -1) {
+	printsw = !(pushsw || popsw || listsw);
+    }
 
     /* Pushing a folder onto the folder stack */
     if (pushsw) {
@@ -257,9 +283,9 @@ main (int argc, char **argv)
 		ap = brkstring (dp, " ", "\n");
 		argfolder = getcpy(*ap++);
 	    } else {
-		adios (NULL, "no other folder");
+		die("no other folder");
 	    }
-	    for (cp = getcpy (getfolder (1)); *ap; ap++)
+	    for (cp = mh_xstrdup(getfolder(1)); *ap; ap++)
 		cp = add (*ap, add (" ", cp));
 	    free (dp);
 	    context_replace (stack, cp);	/* update folder stack */
@@ -268,20 +294,20 @@ main (int argc, char **argv)
 	    context_replace (stack,
 		    (cp = context_find (stack))
 		    ? concat (getfolder (1), " ", cp, NULL)
-		    : getcpy (getfolder (1)));
+		    : mh_xstrdup(getfolder(1)));
 	}
     }
 
     /* Popping a folder off of the folder stack */
     if (popsw) {
 	if (argfolder)
-	    adios (NULL, "sorry, no folders allowed with -pop");
+	    die("sorry, no folders allowed with -pop");
 	if ((cp = context_find (stack))) {
 	    dp = mh_xstrdup(cp);
 	    ap = brkstring (dp, " ", "\n");
 	    argfolder = getcpy(*ap++);
 	} else {
-	    adios (NULL, "folder stack empty");
+	    die("folder stack empty");
 	}
 	if (*ap) {
 	    /* if there's anything left in the stack */
@@ -296,7 +322,7 @@ main (int argc, char **argv)
     }
     if (pushsw || popsw) {
 	cp = m_maildir(argfolder);
-	if (access (cp, F_OK) == NOTOK)
+	if (access (cp, F_OK) == -1)
 	    adios (cp, "unable to find folder");
 	context_replace (pfolder, argfolder);	/* update current folder   */
 	context_save ();		/* save the context file   */
@@ -326,7 +352,7 @@ main (int argc, char **argv)
      * Scan the folders
      */
     /* change directory to base of nmh directory for crawl_folders */
-    if (chdir (nmhdir) == NOTOK)
+    if (chdir (nmhdir) == -1)
 	adios (nmhdir, "unable to change directory to");
     if (all || ftotal > 0) {
 	/*
@@ -337,10 +363,10 @@ main (int argc, char **argv)
 		inform("no folder given for message %s, continuing...", msg);
 	    readonly_folders (); /* do any readonly folders */
 	    cp = context_find(pfolder);
-	    strncpy (folder, FENDNULL(cp), sizeof(folder));
+	    strncpy (folder, FENDNULL(cp), sizeof(folder) - 1);
 	    crawl_folders (".", get_folder_info_callback, NULL);
 	} else {
-	    strncpy (folder, argfolder, sizeof(folder));
+	    strncpy (folder, argfolder, sizeof(folder) - 1);
 	    if (get_folder_info (argfolder, msg)) {
 		context_replace (pfolder, argfolder);/* update current folder */
 		context_save ();		     /* save the context file */
@@ -353,7 +379,8 @@ main (int argc, char **argv)
 		crawl_folders (folder, get_folder_info_callback, NULL);
 	}
     } else {
-	strncpy (folder, argfolder ? argfolder : getfolder (1), sizeof(folder));
+	strncpy (folder, argfolder ? argfolder : getfolder (1),
+		 sizeof(folder) - 1);
 
 	/*
 	 * Check if folder exists.  If not, then see if
@@ -370,7 +397,8 @@ main (int argc, char **argv)
     /*
      * Print out folder information
      */
-    print_folders();
+    if (printsw)
+	print_folders();
 
     context_save ();	/* save the context file */
     done (0);
@@ -378,7 +406,7 @@ main (int argc, char **argv)
 }
 
 static int
-get_folder_info_body (char *fold, char *msg, boolean *crawl_children)
+get_folder_info_body (char *fold, char *msg, bool *crawl_children)
 {
     int	i, retval = 1;
     struct msgs *mp = NULL;
@@ -408,7 +436,7 @@ get_folder_info_body (char *fold, char *msg, boolean *crawl_children)
 	 */
 	if (!(mp = folder_read (fold, fpack))) {
 	    inform("unable to read folder %s, continuing...", fold);
-	    *crawl_children = FALSE;
+	    *crawl_children = false;
 	    return 0;
 	}
 
@@ -418,7 +446,7 @@ get_folder_info_body (char *fold, char *msg, boolean *crawl_children)
 
 	if (fpack) {
 	    if (folder_pack (&mp, fverb) == -1) {
-		*crawl_children = FALSE; /* to please clang static analyzer */
+		*crawl_children = false; /* to please clang static analyzer */
 		done (1);
 	    }
 	    seq_save (mp);		/* synchronize the sequences */
@@ -442,10 +470,10 @@ get_folder_info_body (char *fold, char *msg, boolean *crawl_children)
     return retval;
 }
 
-static boolean
+static bool
 get_folder_info_callback (char *fold, void *baton)
 {
-    boolean crawl_children;
+    bool crawl_children;
     NMH_UNUSED (baton);
 
     get_folder_info_body (fold, NULL, &crawl_children);
@@ -456,7 +484,7 @@ get_folder_info_callback (char *fold, void *baton)
 static int
 get_folder_info (char *fold, char *msg)
 {
-    boolean crawl_children;
+    bool crawl_children;
     int retval;
 
     retval = get_folder_info_body (fold, msg, &crawl_children);
@@ -475,12 +503,13 @@ get_folder_info (char *fold, char *msg)
 static void
 print_folders (void)
 {
-    int i, len, hasempty = 0, curprinted;
+    int i, len;
+    bool hasempty = false;
+    bool curprinted;
     int maxlen = 0, maxnummsg = 0, maxlowmsg = 0;
     int maxhghmsg = 0, maxcurmsg = 0, total_msgs = 0;
     int nummsgdigits, lowmsgdigits;
     int hghmsgdigits, curmsgdigits;
-    char tmpname[BUFSIZ];
 
     /*
      * compute a few values needed to for
@@ -519,7 +548,7 @@ print_folders (void)
 
 	/* check for empty folders */
 	if (fi[i].nummsg == 0)
-	    hasempty = 1;
+	    hasempty = true;
     }
     nummsgdigits = num_digits (maxnummsg);
     lowmsgdigits = num_digits (maxlowmsg);
@@ -545,25 +574,25 @@ print_folders (void)
      */
     if (all || fshort || ftotal < 1) {
 	for (i = 0; i < total_folders; i++) {
+	    char *name = fi[i].name;
+
 	    if (fshort) {
-		puts(fi[i].name);
+		puts(name);
 		continue;
 	    }
 
-	    /* Add `+' to end of name, if folder is current */
-	    if (strcmp (folder, fi[i].name))
-		snprintf (tmpname, sizeof(tmpname), "%s", fi[i].name);
-	    else
-		snprintf (tmpname, sizeof(tmpname), "%s+", fi[i].name);
+	    if (!strcmp(name, folder)) {
+		int spaces = maxlen - strlen(name);
+		printf("%s+%*s ", name, spaces, "");
+	    } else
+		printf("%-*s  ", maxlen, name);
 
 	    if (fi[i].error) {
-		printf ("%-*s is unreadable\n", maxlen+1, tmpname);
+                puts("is unreadable");
 		continue;
 	    }
 
-	    printf ("%-*s ", maxlen+1, tmpname);
-
-	    curprinted = 0; /* remember if we print cur */
+	    curprinted = false; /* remember if we print cur */
 	    if (fi[i].nummsg == 0) {
 		printf ("has %*s messages%*s",
 			nummsgdigits, "no",
@@ -575,7 +604,7 @@ print_folders (void)
 			lowmsgdigits, fi[i].lowmsg,
 			hghmsgdigits, fi[i].hghmsg);
 		if (fi[i].curmsg >= fi[i].lowmsg && fi[i].curmsg <= fi[i].hghmsg) {
-		    curprinted = 1;
+		    curprinted = true;
 		    printf ("; cur=%*d", curmsgdigits, fi[i].curmsg);
 		}
 	    }
@@ -639,7 +668,7 @@ readonly_folders (void)
     atrlen = strlen (atrcur);
 
     for (np = m_defs; np; np = np->n_next)
-	if (ssequal (atrcur, np->n_name)
-		&& !ssequal (nmhdir, np->n_name + atrlen))
+	if (has_prefix(np->n_name, atrcur)
+		&& !has_prefix(np->n_name + atrlen, nmhdir))
 	    get_folder_info (np->n_name + atrlen, NULL);
 }
