@@ -5,25 +5,35 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "sbr/getarguments.h"
+#include "sbr/smatch.h"
+#include "sbr/m_backup.h"
+#include "sbr/context_find.h"
+#include "sbr/readconfig.h"
+#include "sbr/ambigsw.h"
+#include "sbr/path.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
 #include <fcntl.h>
-#include <h/md5.h>
-#include <h/mts.h>
-#include <h/tws.h>
-#include <h/mime.h>
-#include <h/mhparse.h>
-#include <h/mhcachesbr.h>
-#include <h/utils.h>
-#include "../sbr/m_maildir.h"
-#include "../sbr/m_mktemp.h"
+#include "sbr/mts.h"
+#include "h/tws.h"
+#include "h/mime.h"
+#include "h/mhparse.h"
+#include "mhbuildsbr.h"
+#include "mhlistsbr.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
+#include "sbr/m_maildir.h"
+#include "sbr/m_mktemp.h"
 #include "mhfree.h"
 #include "mhoutsbr.h"
+#include "sbr/globals.h"
 
 #define MHBUILD_SWITCHES \
     X("auto", 0, AUTOSW) \
     X("noauto", 0, NAUTOSW) \
-    X("check", 0, CHECKSW) \
-    X("nocheck", 0, NCHECKSW) \
     X("directives", 0, DIRECTIVES) \
     X("nodirectives", 0, NDIRECTIVES) \
     X("headers", 0, HEADSW) \
@@ -38,8 +48,6 @@
     X("noverbose", 0, NVERBSW) \
     X("disposition", 0, DISPOSW) \
     X("nodisposition", 0, NDISPOSW) \
-    X("rcache policy", 0, RCACHESW) \
-    X("wcache policy", 0, WCACHESW) \
     X("contentid", 0, CONTENTIDSW) \
     X("nocontentid", 0, NCONTENTIDSW) \
     X("headerencoding encoding-algorithm", 0, HEADERENCSW) \
@@ -72,31 +80,29 @@ DEFINE_SWITCH_ENUM(MIMEENCODING);
 DEFINE_SWITCH_ARRAY(MIMEENCODING, encodingswitches);
 #undef X
 
-int debugsw = 0;
-
-int listsw   = 0;
-int rfc934sw = 0;
-int contentidsw = 1;
+bool debugsw;
 
 /*
  * Temporary files
  */
 static char infile[BUFSIZ];
-static int unlink_infile  = 0;
-
 static char outfile[BUFSIZ];
-static int unlink_outfile = 0;
-
-static void unlink_done (int) NORETURN;
 
 
 int
 main (int argc, char **argv)
 {
-    int sizesw = 1, headsw = 1, directives = 1, autobuild = 0, dist = 0;
-    int verbosw = 0, dispo = 0;
+    bool sizesw = true;
+    bool headsw = true;
+    bool contentidsw = true;
+    bool rfc934sw = false;
+    bool directives = true;
+    bool autobuild = false;
+    bool dist = false;
+    bool listsw = false;
+    bool verbosw = false;
+    bool dispo = false;
     size_t maxunencoded = MAXTEXTPERLN;
-    int *icachesw;
     char *cp, buf[BUFSIZ];
     char buffer[BUFSIZ], *compfile = NULL;
     char **argp, **arguments;
@@ -106,9 +112,7 @@ main (int argc, char **argv)
     int header_encoding = CE_UNKNOWN;
     size_t n;
 
-    if (nmh_init(argv[0], 2)) { return 1; }
-
-    done=unlink_done;
+    if (nmh_init(argv[0], true, false)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -116,11 +120,10 @@ main (int argc, char **argv)
     while ((cp = *argp++)) {
 	if (cp[0] == '-' && cp[1] == '\0') {
 	    if (compfile)
-		adios (NULL, "cannot specify both standard input and a file");
-	    else
-		compfile = cp;
-	    listsw = 0;		/* turn off -list if using standard in/out */
-	    verbosw = 0;	/* turn off -verbose listings */
+		die("cannot specify both standard input and a file");
+            compfile = cp;
+	    listsw = false;	/* turn off -list if using standard in/out */
+	    verbosw = false;	/* turn off -verbose listings */
 	    break;
 	}
 	if (*cp == '-') {
@@ -129,7 +132,7 @@ main (int argc, char **argv)
 		ambigsw (cp, switches);
 		done (1);
 	    case UNKWNSW: 
-		adios (NULL, "-%s unknown", cp);
+		die("-%s unknown", cp);
 
 	    case HELPSW: 
 		snprintf (buf, sizeof(buf), "%s [switches] file", invo_name);
@@ -140,98 +143,70 @@ main (int argc, char **argv)
 		done (0);
 
 	    case AUTOSW:
-	    	/* -auto implies -nodirectives */
-		autobuild = 1;
-		directives = 0;
+		/* -auto implies -nodirectives */
+		autobuild = true;
+		directives = false;
 		continue;
 	    case NAUTOSW:
-	    	/*
+		/*
 		 * We're turning directives back on since this is likely here
 		 * to override a profile entry
 		 */
-		autobuild = 0;
-		directives = 1;
-		continue;
-
-	    case RCACHESW:
-		icachesw = &rcachesw;
-		goto do_cache;
-	    case WCACHESW:
-		icachesw = &wcachesw;
-	    do_cache: ;
-		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
-		switch (*icachesw = smatch (cp, cache_policy)) {
-		case AMBIGSW:
-		    ambigsw (cp, cache_policy);
-		    done (1);
-		case UNKWNSW:
-		    adios (NULL, "%s unknown", cp);
-		default:
-		    break;
-		}
-		continue;
-
-	    case CHECKSW:
-		checksw++;
-		continue;
-	    case NCHECKSW:
-		checksw = 0;
+		autobuild = false;
+		directives = true;
 		continue;
 
 	    case HEADSW:
-		headsw++;
+		headsw = true;
 		continue;
 	    case NHEADSW:
-		headsw = 0;
+		headsw = false;
 		continue;
 
 	    case DIRECTIVES:
-		directives = 1;
+		directives = true;
 		continue;
 	    case NDIRECTIVES:
-		directives = 0;
+		directives = false;
 		continue;
 
 	    case LISTSW:
-		listsw++;
+		listsw = true;
 		continue;
 	    case NLISTSW:
-		listsw = 0;
+		listsw = false;
 		continue;
 
 	    case RFC934SW:
-		rfc934sw++;
+		rfc934sw = true;
 		continue;
 	    case NRFC934SW:
-		rfc934sw = 0;
+		rfc934sw = false;
 		continue;
 
 	    case SIZESW:
-		sizesw++;
+		sizesw = true;
 		continue;
 	    case NSIZESW:
-		sizesw = 0;
+		sizesw = false;
 		continue;
 
 	    case CONTENTIDSW:
-		contentidsw = 1;
+		contentidsw = true;
 		continue;
 	    case NCONTENTIDSW:
-		contentidsw = 0;
+		contentidsw = false;
 		continue;
 
-	    case HEADERENCSW: {
-	    	int encoding;
-
+	    case HEADERENCSW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
-		switch (encoding = smatch (cp, encodingswitches)) {
+		    die("missing argument to %s", argp[-2]);
+		switch (smatch (cp, encodingswitches)) {
 		case AMBIGSW:
 		    ambigsw (cp, encodingswitches);
 		    done (1);
 		case UNKWNSW:
-		    adios (NULL, "%s unknown encoding algorithm", cp);
+		    die("%s unknown encoding algorithm", cp);
 		case BASE64SW:
 		    header_encoding = CE_BASE64;
 		    break;
@@ -242,48 +217,46 @@ main (int argc, char **argv)
 		    header_encoding = CE_8BIT;
 		    break;
 		default:
-		    adios (NULL, "Internal error: algorithm %s", cp);
+		    die("Internal error: algorithm %s", cp);
 		}
 		continue;
-	    }
 
 	    case AUTOHEADERENCSW:
-	    	header_encoding = CE_UNKNOWN;
+		header_encoding = CE_UNKNOWN;
 		continue;
 
 	    case MAXUNENCSW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		if ((maxunencoded = atoi(cp)) < 1)
-		    adios (NULL, "Invalid argument for %s: %s", argp[-2], cp);
+		    die("Invalid argument for %s: %s", argp[-2], cp);
 		if (maxunencoded > 998)
-		    adios (NULL, "limit of -maxunencoded is 998");
+		    die("limit of -maxunencoded is 998");
 		continue;
 
 	    case VERBSW: 
-		verbosw++;
+		verbosw = true;
 		continue;
 	    case NVERBSW: 
-		verbosw = 0;
+		verbosw = false;
 		continue;
 	    case DISPOSW:
-		dispo = 1;
+		dispo = true;
 		continue;
 	    case NDISPOSW:
-		dispo = 0;
+		dispo = false;
 		continue;
 	    case DEBUGSW:
-		debugsw = 1;
+		debugsw = true;
 		continue;
 	    case DISTSW:
-		dist = 1;
+		dist = true;
 		continue;
 	    }
 	}
 	if (compfile)
-	    adios (NULL, "only one composition file allowed");
-	else
-	    compfile = cp;
+	    die("only one composition file allowed");
+        compfile = cp;
     }
 
     /*
@@ -291,7 +264,7 @@ main (int argc, char **argv)
      */
     if ((cp = getenv ("MHBUILD"))) {
 	if ((fp = fopen (cp, "r"))) {
-	    readconfig ((struct node **) 0, fp, cp, 0);
+	    readconfig(NULL, fp, cp, 0);
 	    fclose (fp);
 	} else {
 	    admonish ("", "unable to read $MHBUILD profile (%s)", cp);
@@ -302,50 +275,38 @@ main (int argc, char **argv)
      * Read the standard profile setup
      */
     if ((fp = fopen (cp = etcpath ("mhn.defaults"), "r"))) {
-	readconfig ((struct node **) 0, fp, cp, 0);
+	readconfig(NULL, fp, cp, 0);
 	fclose (fp);
     }
 
-    /* Check for public cache location */
-    if ((cache_public = context_find (nmhcache)) && *cache_public != '/')
-	cache_public = NULL;
-
-    /* Check for private cache location */
-    if (!(cache_private = context_find (nmhprivcache)))
-	cache_private = ".cache";
-    cache_private = getcpy (m_maildir (cache_private));
-
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
 
     /* Check if we have a file to process */
     if (!compfile)
-	adios (NULL, "need to specify a %s composition file", invo_name);
+	die("need to specify a %s composition file", invo_name);
 
     /*
      * Process the composition file from standard input.
      */
     if (compfile[0] == '-' && compfile[1] == '\0') {
 	if ((cp = m_mktemp2(NULL, invo_name, NULL, &fp)) == NULL) {
-	    adios(NULL, "unable to create temporary file in %s",
+	    die("unable to create temporary file in %s",
 		  get_temp_dir());
 	}
-
-	/* save a copy of the name for later removal */
-	strncpy (infile, cp, sizeof(infile));
-	unlink_infile = 1;
+	strncpy (infile, cp, sizeof(infile) - 1);
 
 	/* copy standard input to temporary file */
 	while ((n = fread(buffer, 1, sizeof(buffer), stdin)) > 0) {
 	    if (fwrite(buffer, 1, n, fp) != n) {
-		adios (NULL, "error copying to temporary file");
+		die("error copying to temporary file");
 	    }
 	}
 	fclose (fp);
 
 	/* build the content structures for MIME message */
 	ct = build_mime (infile, autobuild, dist, directives, header_encoding,
-			 maxunencoded, verbosw);
+			 contentidsw, rfc934sw, maxunencoded, listsw, verbosw);
 
 	/*
 	 * If ct == NULL, that means that -auto was set and a MIME version
@@ -354,11 +315,11 @@ main (int argc, char **argv)
 
 	if (!ct) {
 	    if (! (fp = fopen(infile, "r"))) {
-		adios(NULL, "Unable to open %s for reading", infile);
+		die("Unable to open %s for reading", infile);
 	    }
 	    while ((n = fread(buffer, 1, sizeof(buffer), fp)) > 0) {
 		if (fwrite(buffer, 1, n, stdout) != n) {
-		    adios(NULL, "error copying %s to stdout", infile);
+		    die("error copying %s to stdout", infile);
 		}
 	    }
 	} else {
@@ -376,7 +337,7 @@ main (int argc, char **argv)
 
     /* build the content structures for MIME message */
     ct = build_mime (compfile, autobuild, dist, directives, header_encoding,
-    		     maxunencoded, verbosw);
+		     contentidsw, rfc934sw, maxunencoded, listsw, verbosw);
 
     /*
      * If ct == NULL, that means -auto was set and we found a MIME version
@@ -391,10 +352,9 @@ main (int argc, char **argv)
 
     /* output MIME message to this temporary file */
     if ((cp = m_mktemp2(compfile, invo_name, NULL, &fp_out)) == NULL) {
-	adios(NULL, "unable to create temporary file in %s", get_temp_dir());
+	die("unable to create temporary file");
     }
-    strncpy(outfile, cp, sizeof(outfile));
-    unlink_outfile = 1;
+    strncpy(outfile, cp, sizeof(outfile) - 1);
 
     /* output the message */
     output_message_fp (ct, fp_out, outfile);
@@ -408,35 +368,22 @@ main (int argc, char **argv)
 
     /* Rename composition draft */
     snprintf (buffer, sizeof(buffer), "%s.orig", m_backup (compfile));
-    if (rename (compfile, buffer) == NOTOK) {
+    if (rename (compfile, buffer) == -1) {
 	adios (compfile, "unable to rename comp draft %s to", buffer);
     }
 
     /* Rename output file to take its place */
-    if (rename (outfile, compfile) == NOTOK) {
+    if (rename (outfile, compfile) == -1) {
 	advise (outfile, "unable to rename output %s to", compfile);
 	rename (buffer, compfile);
 	done (1);
     }
-    unlink_outfile = 0;
+    /* Remove from atexit(3) list of files to unlink. */
+    if (!(m_unlink(outfile) == -1 && errno == ENOENT)) {
+        adios(outfile, "file exists after rename:");
+    }
 
     free_content (ct);
     done (0);
     return 1;
 }
-
-
-static void NORETURN
-unlink_done (int status)
-{
-    /*
-     * Check if we need to remove stray
-     * temporary files.
-     */
-    if (unlink_infile)
-	(void) m_unlink (infile);
-    if (unlink_outfile)
-	(void) m_unlink (outfile);
-
-    exit (status);
-}
