@@ -5,14 +5,16 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "mhlsbr.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
 
 
 int
 main (int argc, char **argv)
 {
-    if (nmh_init(argv[0], 2)) { return 1; }
+    if (nmh_init(argv[0], true, false)) { return 1; }
 
     done (mhl (argc, argv));
     return 1;
