@@ -1,26 +1,45 @@
-
-/*
- * mhlist.c -- list the contents of MIME messages
+/* mhlist.c -- list the contents of MIME messages
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "sbr/m_name.h"
+#include "sbr/m_gmprot.h"
+#include "sbr/getarguments.h"
+#include "sbr/seq_setprev.h"
+#include "sbr/seq_setcur.h"
+#include "sbr/seq_save.h"
+#include "sbr/smatch.h"
+#include "sbr/m_convert.h"
+#include "sbr/getfolder.h"
+#include "sbr/folder_read.h"
+#include "sbr/context_save.h"
+#include "sbr/context_replace.h"
+#include "sbr/context_find.h"
+#include "sbr/ambigsw.h"
+#include "sbr/path.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
 #include <fcntl.h>
-#include <h/signals.h>
-#include <h/md5.h>
-#include <h/mts.h>
-#include <h/tws.h>
-#include <h/mime.h>
-#include <h/mhparse.h>
-#include <h/mhcachesbr.h>
-#include <h/utils.h>
+#include <signal.h>
+#include "sbr/signals.h"
+#include "sbr/mts.h"
+#include "h/tws.h"
+#include "h/mime.h"
+#include "h/mhparse.h"
+#include "mhlistsbr.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
+#include "mhmisc.h"
+#include "sbr/m_maildir.h"
+#include "mhfree.h"
+#include "sbr/globals.h"
 
 #define MHLIST_SWITCHES \
-    X("check", 0, CHECKSW) \
-    X("nocheck", 0, NCHECKSW) \
     X("headers", 0, HEADSW) \
     X("noheaders", 0, NHEADSW) \
     X("realsize", 0, SIZESW) \
@@ -32,8 +51,8 @@
     X("file file", 0, FILESW) \
     X("part number", 0, PARTSW) \
     X("type content", 0, TYPESW) \
-    X("rcache policy", 0, RCACHESW) \
-    X("wcache policy", 0, WCACHESW) \
+    X("prefer content", 0, PREFERSW) \
+    X("noprefer", 0, NPREFERSW) \
     X("changecur", 0, CHGSW) \
     X("nochangecur", 0, NCHGSW) \
     X("version", 0, VERSIONSW) \
@@ -48,40 +67,14 @@ DEFINE_SWITCH_ENUM(MHLIST);
 DEFINE_SWITCH_ARRAY(MHLIST, switches);
 #undef X
 
-
-/* mhcachesbr.c */
-extern int rcachesw;
-extern int wcachesw;
-extern char *cache_public;
-extern char *cache_private;
-
-/* mhmisc.c */
-extern int npart;
-extern int ntype;
-extern char *parts[NPARTS + 1];
-extern char *types[NTYPES + 1];
-extern int userrs;
-
 /*
  * This is currently needed to keep mhparse happy.
  * This needs to be changed.
  */
-int debugsw = 0;
+bool debugsw;
 
 #define	quitser	pipeser
 
-/* mhparse.c */
-CT parse_mime (char *);
-
-/* mhmisc.c */
-int part_ok (CT, int);
-int type_ok (CT, int);
-void flush_errors (void);
-
-/* mhfree.c */
-extern CT *cts;
-void freects_done (int) NORETURN;
-
 /*
  * static prototypes
  */
@@ -91,8 +84,12 @@ static void pipeser (int);
 int
 main (int argc, char **argv)
 {
-    int sizesw = 1, headsw = 1, chgflag = 1, verbosw = 0, dispo = 0;
-    int msgnum, *icachesw;
+    bool sizesw = true;
+    bool headsw = true;
+    bool chgflag = true;
+    bool verbosw = false;
+    bool dispo = false;
+    int msgnum;
     char *cp, *file = NULL, *folder = NULL;
     char *maildir, buf[100], **argp;
     char **arguments;
@@ -100,9 +97,9 @@ main (int argc, char **argv)
     struct msgs *mp = NULL;
     CT ct, *ctp;
 
-    if (nmh_init(argv[0], 1)) { return 1; }
+    if (nmh_init(argv[0], true, true)) { return 1; }
 
-    done=freects_done;
+    set_done(freects_done);
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -117,7 +114,7 @@ main (int argc, char **argv)
 		ambigsw (cp, switches);
 		done (1);
 	    case UNKWNSW: 
-		adios (NULL, "-%s unknown", cp);
+		die("-%s unknown", cp);
 
 	    case HELPSW: 
 		snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
@@ -128,99 +125,88 @@ main (int argc, char **argv)
 		print_version(invo_name);
 		done (0);
 
-	    case RCACHESW:
-		icachesw = &rcachesw;
-		goto do_cache;
-	    case WCACHESW:
-		icachesw = &wcachesw;
-do_cache:
-		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
-		switch (*icachesw = smatch (cp, caches)) {
-		case AMBIGSW:
-		    ambigsw (cp, caches);
-		    done (1);
-		case UNKWNSW:
-		    adios (NULL, "%s unknown", cp);
-		default:
-		    break;
-		}
-		continue;
-
-	    case CHECKSW:
-		checksw++;
-		continue;
-	    case NCHECKSW:
-		checksw = 0;
-		continue;
-
 	    case HEADSW:
-		headsw = 1;
+		headsw = true;
 		continue;
 	    case NHEADSW:
-		headsw = 0;
+		headsw = false;
 		continue;
 
 	    case SIZESW:
-		sizesw = 1;
+		sizesw = true;
 		continue;
 	    case NSIZESW:
-		sizesw = 0;
+		sizesw = false;
 		continue;
 
 	    case PARTSW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		if (npart >= NPARTS)
-		    adios (NULL, "too many parts (starting with %s), %d max",
+		    die("too many parts (starting with %s), %d max",
 			   cp, NPARTS);
 		parts[npart++] = cp;
 		continue;
 
 	    case TYPESW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		if (ntype >= NTYPES)
-		    adios (NULL, "too many types (starting with %s), %d max",
+		    die("too many types (starting with %s), %d max",
 			   cp, NTYPES);
 		types[ntype++] = cp;
 		continue;
 
+	    case PREFERSW:
+		if (!(cp = *argp++) || *cp == '-')
+		    die("missing argument to %s", argp[-2]);
+		if (npreferred >= NPREFS)
+		    die("too many preferred types (starting with %s), %d max",
+			   cp, NPREFS);
+                mime_preference[npreferred].type = cp;
+		cp = strchr(cp, '/');
+		if (cp) *cp++ = '\0';
+		mime_preference[npreferred++].subtype = cp;
+		continue;
+
+	    case NPREFERSW:
+		npreferred = 0;
+		continue;
+
 	    case FILESW:
 		if (!(cp = *argp++) || (*cp == '-' && cp[1]))
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		file = *cp == '-' ? cp : path (cp, TFILE);
 		continue;
 
 	    case CHGSW:
-	    	chgflag++;
+		chgflag = true;
 		continue;
 	    case NCHGSW:
-	    	chgflag = 0;
+		chgflag = false;
 		continue;
 
 	    case VERBSW: 
-		verbosw = 1;
+		verbosw = true;
 		continue;
 	    case NVERBSW: 
-		verbosw = 0;
+		verbosw = false;
 		continue;
 	    case DISPOSW:
-		dispo = 1;
+		dispo = true;
 		continue;
 	    case NDISPOSW:
-		dispo = 0;
+		dispo = false;
 		continue;
 	    case DEBUGSW:
-		debugsw = 1;
+		debugsw = true;
 		continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
 	    if (folder)
-		adios (NULL, "only one folder at a time!");
-	    else
-		folder = pluspath (cp);
+		die("only one folder at a time!");
+            folder = pluspath (cp);
 	} else
 		app_msgarg(&msgs, cp);
     }
@@ -229,27 +215,17 @@ do_cache:
     parts[npart] = NULL;
     types[ntype] = NULL;
 
-    /* Check for public cache location */
-    if ((cache_public = context_find (nmhcache)) && *cache_public != '/')
-	cache_public = NULL;
-
-    /* Check for private cache location */
-    if (!(cache_private = context_find (nmhprivcache)))
-	cache_private = ".cache";
-    cache_private = getcpy (m_maildir (cache_private));
-
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
 
     if (file && msgs.size)
-	adios (NULL, "cannot specify msg and file at same time!");
+	die("cannot specify msg and file at same time!");
 
     /*
      * check if message is coming from file
      */
     if (file) {
-	if (!(cts = (CT *) calloc ((size_t) 2, sizeof(*cts))))
-	    adios (NULL, "out of memory");
+	cts = mh_xcalloc(2, sizeof *cts);
 	ctp = cts;
 
 	if ((ct = parse_mime (file)))
@@ -264,16 +240,16 @@ do_cache:
 	    folder = getfolder (1);
 	maildir = m_maildir (folder);
 
-	if (chdir (maildir) == NOTOK)
+	if (chdir (maildir) == -1)
 	    adios (maildir, "unable to change directory to");
 
 	/* read folder and create message structure */
 	if (!(mp = folder_read (folder, 0)))
-	    adios (NULL, "unable to read folder %s", folder);
+	    die("unable to read folder %s", folder);
 
 	/* check for empty folder */
 	if (mp->nummsg == 0)
-	    adios (NULL, "no messages in %s", folder);
+	    die("no messages in %s", folder);
 
 	/* parse all the message ranges/sequences and set SELECTED */
 	for (msgnum = 0; msgnum < msgs.size; msgnum++)
@@ -281,8 +257,7 @@ do_cache:
 		done (1);
 	seq_setprev (mp);	/* set the previous-sequence */
 
-	if (!(cts = (CT *) calloc ((size_t) (mp->numsel + 1), sizeof(*cts))))
-	    adios (NULL, "out of memory");
+	cts = mh_xcalloc(mp->numsel + 1, sizeof *cts);
 	ctp = cts;
 
 	for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
@@ -299,7 +274,7 @@ do_cache:
     if (!*cts)
 	done (1);
 
-    userrs = 1;
+    userrs = true;
     SIGNAL (SIGQUIT, quitser);
     SIGNAL (SIGPIPE, pipeser);
 
@@ -311,7 +286,7 @@ do_cache:
 
 	ct = *ctp;
 	if (type_ok (ct, 1) && !ct->c_umask) {
-	    if (stat (ct->c_file, &st) != NOTOK)
+	    if (stat (ct->c_file, &st) != -1)
 		ct->c_umask = ~(st.st_mode & 0777);
 	    else
 		ct->c_umask = ~m_gmprot();
@@ -327,7 +302,7 @@ do_cache:
     for (ctp = cts; *ctp; ctp++)
 	free_content (*ctp);
 
-    free ((char *) cts);
+    free(cts);
     cts = NULL;
 
     /* If reading from a folder, do some updating */
