@@ -5,13 +5,16 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/mime.h>
-#include <h/mhparse.h>
-#include <h/utils.h>
-#include "../sbr/makedir.h"
-
-extern int debugsw;
+#include "h/mh.h"
+#include "sbr/read_yes_or_no_if_tty.h"
+#include "sbr/concat.h"
+#include "sbr/error.h"
+#include "h/mime.h"
+#include "h/mhparse.h"
+#include "sbr/utils.h"
+#include "sbr/makedir.h"
+#include "mhmisc.h"
+#include "sbr/globals.h"
 
 /*
  * limit actions to specified parts or content types
@@ -21,21 +24,11 @@ int ntype = 0;
 char *parts[NPARTS + 1];
 char *types[NTYPES + 1];
 
-int userrs = 0;
+bool userrs;
 
 static char *errs = NULL;
 
 
-/*
- * prototypes
- */
-int part_ok (CT);
-int part_exact(CT ct);
-int type_ok (CT, int);
-void content_error (char *, CT, char *, ...);
-void flush_errors (void);
-
-
 int
 part_ok (CT ct)
 {
@@ -103,7 +96,7 @@ type_ok (CT ct, int sP)
 
 
 /*
- * Returns true if this content is marked as "inline".
+ * Returns true if the content has a disposition of "inline".
  *
  * Technically we should check parent content to see if they have
  * disposition to use as a default, but we don't right now.  Maybe
@@ -133,7 +126,7 @@ make_intermediates (char *file)
 	struct stat st;
 
 	*cp = '\0';
-	if (stat (file, &st) == NOTOK) {
+	if (stat (file, &st) == -1) {
 	    int answer;
 	    char *ep;
 	    if (errno != ENOENT) {
@@ -170,7 +163,7 @@ void
 content_error (char *what, CT ct, char *fmt, ...)
 {
     va_list arglist;
-    int	i, len, buflen;
+    int	len, buflen;
     char *bp, buffer[BUFSIZ];
     CI ci;
 
@@ -204,7 +197,7 @@ content_error (char *what, CT ct, char *fmt, ...)
 	}
 
 	if ((s = strerror (errno)))
-	    snprintf (bp, buflen, "%s", s);
+	    trunccpy(bp, s, buflen);
 	else
 	    snprintf (bp, buflen, "Error %d", errno);
 
@@ -213,10 +206,8 @@ content_error (char *what, CT ct, char *fmt, ...)
 	buflen -= len;
     }
 
-    i = strlen (invo_name) + 2;
-
     /* Now add content type and subtype */
-    snprintf (bp, buflen, "\n%*.*s(content %s/%s", i, i, "",
+    snprintf (bp, buflen, "\n    (content %s/%s",
 	ci->ci_type, ci->ci_subtype);
     len = strlen (bp);
     bp += len;
@@ -251,6 +242,8 @@ content_error (char *what, CT ct, char *fmt, ...)
     } else {
 	inform("%s", buffer);
     }
+
+    va_end(arglist);
 }
 
 
