@@ -1,30 +1,53 @@
-
-/*
- * mhn.c -- display, list, cache, or store the contents of MIME messages
+/* mhn.c -- display, list, or store the contents of MIME messages
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "sbr/charstring.h"
+#include "sbr/m_name.h"
+#include "sbr/m_gmprot.h"
+#include "sbr/getarguments.h"
+#include "sbr/seq_setprev.h"
+#include "sbr/seq_setcur.h"
+#include "sbr/seq_save.h"
+#include "sbr/smatch.h"
+#include "sbr/m_convert.h"
+#include "sbr/getfolder.h"
+#include "sbr/folder_read.h"
+#include "sbr/context_save.h"
+#include "sbr/context_replace.h"
+#include "sbr/context_find.h"
+#include "sbr/readconfig.h"
+#include "sbr/ambigsw.h"
+#include "sbr/path.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
 #include <fcntl.h>
-#include <h/signals.h>
-#include <h/md5.h>
-#include <h/mts.h>
-#include <h/tws.h>
-#include <h/mime.h>
-#include <h/mhparse.h>
-#include <h/mhcachesbr.h>
-#include <h/utils.h>
+#include <signal.h>
+#include "sbr/signals.h"
+#include "sbr/mts.h"
+#include "h/tws.h"
+#include "sbr/fmt_compile.h"
+#include "sbr/fmt_scan.h"
+#include "h/mime.h"
+#include "h/mhparse.h"
+#include "mhlistsbr.h"
+#include "mhstoresbr.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
+#include "mhmisc.h"
+#include "sbr/m_maildir.h"
+#include "mhfree.h"
+#include "mhshowsbr.h"
+#include "sbr/globals.h"
 
 #define MHN_SWITCHES \
     X("auto", 0, AUTOSW) \
     X("noauto", 0, NAUTOSW) \
-    X("cache", 0, CACHESW) \
-    X("nocache", 0, NCACHESW) \
-    X("check", 0, CHECKSW) \
-    X("nocheck", 0, NCHECKSW) \
     X("headers", 0, HEADSW) \
     X("noheaders", 0, NHEADSW) \
     X("list", 0, LISTSW) \
@@ -41,8 +64,6 @@
     X("form formfile", 0, FORMSW) \
     X("part number", 0, PARTSW) \
     X("type content", 0, TYPESW) \
-    X("rcache policy", 0, RCACHESW) \
-    X("wcache policy", 0, WCACHESW) \
     X("version", 0, VERSIONSW) \
     X("help", 0, HELPSW) \
     /*					\
@@ -61,8 +82,8 @@
      */					\
     X("build", -5, BUILDSW) \
     X("nobuild", -7, NBUILDSW) \
-    X("rfc934mode", -10, RFC934SW) \
-    X("norfc934mode", -12, NRFC934SW) \
+    X("rfc934mode", 0, RFC934SW) \
+    X("norfc934mode", 0, NRFC934SW) \
 
 #define X(sw, minchars, id) id,
 DEFINE_SWITCH_ENUM(MHN);
@@ -73,65 +94,24 @@ DEFINE_SWITCH_ARRAY(MHN, switches);
 #undef X
 
 
-/* mhcachesbr.c */
-extern int rcachesw;
-extern int wcachesw;
-extern char *cache_public;
-extern char *cache_private;
-
-/* mhshowsbr.c */
-extern char *progsw;
-extern int nolist;
-extern int nomore;	/* flags for moreproc/header display */
-extern char *formsw;
-
-/* mhmisc.c */
-extern int npart;
-extern int ntype;
-extern char *parts[NPARTS + 1];
-extern char *types[NTYPES + 1];
-extern int userrs;
-
-int debugsw = 0;
+bool debugsw;
 int verbosw = 0;
 
 /*
  * variables for mhbuild (mhn -build)
  */
-static int buildsw  = 0;
+static bool buildsw;
 static int rfc934sw = 0;
 
 /*
  * what action to take?
  */
-static int cachesw = 0;
-static int listsw  = 0;
-static int showsw  = 0;
-static int storesw = 0;
+static bool listsw;
+static bool showsw;
+static bool storesw;
 
 #define	quitser	pipeser
 
-/* mhparse.c */
-CT parse_mime (char *);
-
-/* mhmisc.c */
-int part_ok (CT, int);
-int type_ok (CT, int);
-void flush_errors (void);
-
-/* mhstoresbr.c */
-typedef struct mhstoreinfo *mhstoreinfo_t;
-mhstoreinfo_t mhstoreinfo_create(CT *, char *, const char *, int, int);
-void mhstoreinfo_free(mhstoreinfo_t);
-void store_all_messages (mhstoreinfo_t);
-
-/* mhcachesbr.c */
-void cache_all_messages (CT *);
-
-/* mhfree.c */
-extern CT *cts;
-void freects_done (int) NORETURN;
-
 /*
  * static prototypes
  */
@@ -141,8 +121,10 @@ static void pipeser (int);
 int
 main (int argc, char **argv)
 {
-    int sizesw = 1, headsw = 1, autosw = 0;
-    int msgnum, *icachesw;
+    bool sizesw = true;
+    bool headsw = true;
+    bool autosw = false;
+    int msgnum;
     char *cp, *file = NULL, *folder = NULL;
     char *maildir, buf[100], **argp;
     char **arguments;
@@ -153,9 +135,9 @@ main (int argc, char **argv)
     FILE *fp;
     mhstoreinfo_t info;
 
-    if (nmh_init(argv[0], 1)) { return 1; }
+    if (nmh_init(argv[0], true, true)) { return 1; }
 
-    done=freects_done;
+    set_done(freects_done);
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -170,7 +152,7 @@ main (int argc, char **argv)
 		ambigsw (cp, switches);
 		done (1);
 	    case UNKWNSW: 
-		adios (NULL, "-%s unknown", cp);
+		die("-%s unknown", cp);
 
 	    case HELPSW: 
 		snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
@@ -182,110 +164,76 @@ main (int argc, char **argv)
 		done (0);
 
 	    case AUTOSW:
-		autosw++;
+		autosw = true;
 		continue;
 	    case NAUTOSW:
-		autosw = 0;
-		continue;
-
-	    case CACHESW:
-		cachesw++;
-		continue;
-	    case NCACHESW:
-		cachesw = 0;
-		continue;
-
-	    case RCACHESW:
-		icachesw = &rcachesw;
-		goto do_cache;
-	    case WCACHESW:
-		icachesw = &wcachesw;
-do_cache:
-		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
-		switch (*icachesw = smatch (cp, caches)) {
-		case AMBIGSW:
-		    ambigsw (cp, caches);
-		    done (1);
-		case UNKWNSW:
-		    adios (NULL, "%s unknown", cp);
-		default:
-		    break;
-		}
-		continue;
-
-	    case CHECKSW:
-		checksw++;
-		continue;
-	    case NCHECKSW:
-		checksw = 0;
+		autosw = false;
 		continue;
 
 	    case HEADSW:
-		headsw = 1;
+		headsw = true;
 		continue;
 	    case NHEADSW:
-		headsw = 0;
+		headsw = false;
 		continue;
 
 	    case LISTSW:
-		listsw = 1;
+		listsw = true;
 		continue;
 	    case NLISTSW:
-		listsw = 0;
+		listsw = false;
 		continue;
 
 	    case SHOWSW:
-		showsw = 1;
+		showsw = true;
 		continue;
 	    case NSHOWSW:
-		showsw = 0;
+		showsw = false;
 		continue;
 
 	    case SIZESW:
-		sizesw = 1;
+		sizesw = true;
 		continue;
 	    case NSIZESW:
-		sizesw = 0;
+		sizesw = false;
 		continue;
 
 	    case STORESW:
-		storesw = 1;
+		storesw = true;
 		continue;
 	    case NSTORESW:
-		storesw = 0;
+		storesw = false;
 		continue;
 
 	    case PARTSW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		if (npart >= NPARTS)
-		    adios (NULL, "too many parts (starting with %s), %d max",
+		    die("too many parts (starting with %s), %d max",
 			   cp, NPARTS);
 		parts[npart++] = cp;
 		continue;
 
 	    case TYPESW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		if (ntype >= NTYPES)
-		    adios (NULL, "too many types (starting with %s), %d max",
+		    die("too many types (starting with %s), %d max",
 			   cp, NTYPES);
 		types[ntype++] = cp;
 		continue;
 
 	    case FILESW:
 		if (!(cp = *argp++) || (*cp == '-' && cp[1]))
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		file = *cp == '-' ? cp : path (cp, TFILE);
 		continue;
 
 	    case FORMSW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
-		if (formsw)
-		    free (formsw);
-		formsw = getcpy (etcpath (cp));
+		    die("missing argument to %s", argp[-2]);
+                free(formsw);
+		formsw = mh_xstrdup(etcpath(cp));
 		continue;
 
 	    /*
@@ -293,26 +241,26 @@ do_cache:
 	     */
 	    case PROGSW:
 		if (!(progsw = *argp++) || *progsw == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 	    case NPROGSW:
-		nomore++;
+		nomore = true;
 		continue;
 
 	    case LENSW:
 	    case WIDTHSW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 
 	    /*
 	     * Switches for mhbuild
 	     */
 	    case BUILDSW:
-		buildsw = 1;
+		buildsw = true;
 		continue;
 	    case NBUILDSW:
-		buildsw = 0;
+		buildsw = false;
 		continue;
 	    case RFC934SW:
 		rfc934sw = 1;
@@ -328,15 +276,14 @@ do_cache:
 		verbosw = 0;
 		continue;
 	    case DEBUGSW:
-		debugsw = 1;
+		debugsw = true;
 		continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
 	    if (folder)
-		adios (NULL, "only one folder at a time!");
-	    else
-		folder = pluspath (cp);
+		die("only one folder at a time!");
+            folder = pluspath (cp);
 	} else
 		app_msgarg(&msgs, cp);
     }
@@ -350,7 +297,7 @@ do_cache:
      */
     if ((cp = getenv ("MHN"))) {
 	if ((fp = fopen (cp, "r"))) {
-	    readconfig ((struct node **) 0, fp, cp, 0);
+	    readconfig(NULL, fp, cp, 0);
 	    fclose (fp);
 	} else {
 	    admonish ("", "unable to read $MHN profile (%s)", cp);
@@ -361,23 +308,14 @@ do_cache:
      * Read the standard profile setup
      */
     if ((fp = fopen (cp = etcpath ("mhn.defaults"), "r"))) {
-	readconfig ((struct node **) 0, fp, cp, 0);
+	readconfig(NULL, fp, cp, 0);
 	fclose (fp);
     }
 
-    /* Check for public cache location */
-    if ((cache_public = context_find (nmhcache)) && *cache_public != '/')
-	cache_public = NULL;
-
-    /* Check for private cache location */
-    if (!(cache_private = context_find (nmhprivcache)))
-	cache_private = ".cache";
-    cache_private = getcpy (m_maildir (cache_private));
-
     /*
      * Cache the current directory before we do any chdirs()'s.
      */
-    cwd = add(pwd(), NULL);
+    cwd = mh_xstrdup(pwd());
 
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
@@ -389,12 +327,12 @@ do_cache:
 	char *vec[MAXARGS];
 	int vecp;
 
-	if (showsw || storesw || cachesw)
-	    adios (NULL, "cannot use -build with -show, -store, -cache");
+	if (showsw || storesw)
+	    die("cannot use -build with -show, -store");
 	if (msgs.size < 1)
-	    adios (NULL, "need to specify a %s composition file", invo_name);
+	    die("need to specify a %s composition file", invo_name);
 	if (msgs.size > 1)
-	    adios (NULL, "only one %s composition file at a time", invo_name);
+	    die("only one %s composition file at a time", invo_name);
 
 	vecp = 0;
 	vec[vecp++] = "mhbuild";
@@ -409,13 +347,13 @@ do_cache:
 
 	execvp ("mhbuild", vec);
 	fprintf (stderr, "unable to exec ");
-	_exit (-1);
+	_exit(1);
     }
 
     /*
      * Process a mhn composition file (old MH style)
      */
-    if (msgs.size == 1 && !folder && !npart && !cachesw
+    if (msgs.size == 1 && !folder && !npart
 	&& !showsw && !storesw && !ntype && !file
 	&& (cp = getenv ("mhdraft"))
 	&& strcmp (cp, msgs.msgs[0]) == 0) {
@@ -436,18 +374,17 @@ do_cache:
 
 	execvp ("mhbuild", vec);
 	fprintf (stderr, "unable to exec ");
-	_exit (-1);
+	_exit(1);
     }
 
     if (file && msgs.size)
-	adios (NULL, "cannot specify msg and file at same time!");
+	die("cannot specify msg and file at same time!");
 
     /*
      * check if message is coming from file
      */
     if (file) {
-	if (!(cts = (CT *) calloc ((size_t) 2, sizeof(*cts))))
-	    adios (NULL, "out of memory");
+	cts = mh_xcalloc(2, sizeof *cts);
 	ctp = cts;
 
 	if ((ct = parse_mime (file)))
@@ -462,16 +399,16 @@ do_cache:
 	    folder = getfolder (1);
 	maildir = m_maildir (folder);
 
-	if (chdir (maildir) == NOTOK)
+	if (chdir (maildir) == -1)
 	    adios (maildir, "unable to change directory to");
 
 	/* read folder and create message structure */
 	if (!(mp = folder_read (folder, 1)))
-	    adios (NULL, "unable to read folder %s", folder);
+	    die("unable to read folder %s", folder);
 
 	/* check for empty folder */
 	if (mp->nummsg == 0)
-	    adios (NULL, "no messages in %s", folder);
+	    die("no messages in %s", folder);
 
 	/* parse all the message ranges/sequences and set SELECTED */
 	for (msgnum = 0; msgnum < msgs.size; msgnum++)
@@ -479,8 +416,7 @@ do_cache:
 		done (1);
 	seq_setprev (mp);	/* set the previous-sequence */
 
-	if (!(cts = (CT *) calloc ((size_t) (mp->numsel + 1), sizeof(*cts))))
-	    adios (NULL, "out of memory");
+	cts = mh_xcalloc(mp->numsel + 1, sizeof *cts);
 	ctp = cts;
 
 	for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
@@ -501,14 +437,14 @@ do_cache:
      * You can't give more than one of these flags
      * at a time.
      */
-    if (showsw + listsw + storesw + cachesw > 1)
-	adios (NULL, "can only use one of -show, -list, -store, -cache at same time");
+    if (showsw + listsw + storesw > 1)
+	die("can only use one of -show, -list, -store at same time");
 
     /* If no action is specified, assume -show */
-    if (!listsw && !showsw && !storesw && !cachesw)
-	showsw = 1;
+    if (!listsw && !showsw && !storesw)
+	showsw = true;
 
-    userrs = 1;
+    userrs = true;
     SIGNAL (SIGQUIT, quitser);
     SIGNAL (SIGPIPE, pipeser);
 
@@ -520,7 +456,7 @@ do_cache:
 
 	ct = *ctp;
 	if (type_ok (ct, 1) && !ct->c_umask) {
-	    if (stat (ct->c_file, &st) != NOTOK)
+	    if (stat (ct->c_file, &st) != -1)
 		ct->c_umask = ~(st.st_mode & 0777);
 	    else
 		ct->c_umask = ~m_gmprot();
@@ -537,7 +473,7 @@ do_cache:
      * Store the message content
      */
     if (storesw) {
-	info = mhstoreinfo_create (cts, cwd, "always", autosw, verbosw);;
+	info = mhstoreinfo_create (cts, cwd, "always", autosw, verbosw);
 	store_all_messages (info);
 	mhstoreinfo_free (info);
     }
@@ -550,17 +486,11 @@ do_cache:
 	context_save ();		  /* save the context file  */
     }
 
-    /*
-     * Cache the message content
-     */
-    if (cachesw)
-	cache_all_messages (cts);
-
     /*
      * Show the message content
      */
     if (showsw)
-	show_all_messages (cts, 0, 0, 0, NULL);
+	show_all_messages (cts, 0, 0, 0);
 
     /* Now free all the structures for the content */
     for (ctp = cts; *ctp; ctp++)
