@@ -5,25 +5,47 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "sbr/charstring.h"
+#include "sbr/m_name.h"
+#include "sbr/m_gmprot.h"
+#include "sbr/getarguments.h"
+#include "sbr/seq_setunseen.h"
+#include "sbr/seq_setprev.h"
+#include "sbr/seq_setcur.h"
+#include "sbr/seq_save.h"
+#include "sbr/smatch.h"
+#include "sbr/m_convert.h"
+#include "sbr/getfolder.h"
+#include "sbr/folder_read.h"
+#include "sbr/context_save.h"
+#include "sbr/context_replace.h"
+#include "sbr/context_find.h"
+#include "sbr/readconfig.h"
+#include "sbr/ambigsw.h"
+#include "sbr/path.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
 #include <fcntl.h>
-#include <h/signals.h>
-#include <h/md5.h>
-#include <h/mts.h>
-#include <h/tws.h>
-#include <h/fmt_scan.h>
-#include <h/mime.h>
-#include <h/mhparse.h>
-#include <h/mhcachesbr.h>
-#include <h/utils.h>
-#include "../sbr/m_maildir.h"
-#include "../sbr/m_popen.h"
+#include <signal.h>
+#include "sbr/signals.h"
+#include "sbr/mts.h"
+#include "h/tws.h"
+#include "sbr/fmt_compile.h"
+#include "sbr/fmt_scan.h"
+#include "h/mime.h"
+#include "h/mhparse.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
+#include "mhmisc.h"
+#include "sbr/m_maildir.h"
+#include "sbr/m_popen.h"
 #include "mhfree.h"
 #include "mhshowsbr.h"
+#include "sbr/globals.h"
 
 #define MHSHOW_SWITCHES \
-    X("check", 0, CHECKSW) \
-    X("nocheck", 0, NCHECKSW) \
     X("verbose", 0, VERBSW) \
     X("noverbose", 0, NVERBSW) \
     X("concat", 0, CONCATSW) \
@@ -42,8 +64,6 @@
     X("type content", 0, TYPESW) \
     X("prefer content", 0, PREFERSW) \
     X("noprefer", 0, NPREFERSW) \
-    X("rcache policy", 0, RCACHESW) \
-    X("wcache policy", 0, WCACHESW) \
     X("version", 0, VERSIONSW) \
     X("help", 0, HELPSW) \
     /*					\
@@ -67,31 +87,11 @@ DEFINE_SWITCH_ARRAY(MHSHOW, switches);
 #undef X
 
 
-/* mhmisc.c */
-extern int npart;
-extern int ntype;
-extern char *parts[NPARTS + 1];
-extern char *types[NTYPES + 1];
-extern int userrs;
-
-/* mhparse.c */
-extern char *preferred_types[];
-extern  char *preferred_subtypes[];
-extern int npreferred;
-
-int debugsw = 0;
+bool debugsw;
 int verbosw = 0;
 
 #define	quitser	pipeser
 
-/* mhparse.c */
-CT parse_mime (char *);
-
-/* mhmisc.c */
-int part_ok (CT);
-int type_ok (CT, int);
-void flush_errors (void);
-
 /*
  * static prototypes
  */
@@ -101,7 +101,7 @@ static void pipeser (int);
 int
 main (int argc, char **argv)
 {
-    int msgnum, *icachesw, concatsw = -1, textonly = -1, inlineonly = -1;
+    int msgnum, concatsw = -1, textonly = -1, inlineonly = -1;
     char *cp, *file = NULL;
     char *maildir, buf[100], **argp;
     char **arguments;
@@ -110,9 +110,9 @@ main (int argc, char **argv)
     CT ct, *ctp;
     FILE *fp;
 
-    if (nmh_init(argv[0], 1)) { return 1; }
+    if (nmh_init(argv[0], true, true)) { return 1; }
 
-    done=freects_done;
+    set_done(freects_done);
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -127,7 +127,7 @@ main (int argc, char **argv)
 		ambigsw (cp, switches);
 		done (1);
 	    case UNKWNSW:
-		adios (NULL, "-%s unknown", cp);
+		die("-%s unknown", cp);
 
 	    case HELPSW:
 		snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
@@ -138,32 +138,6 @@ main (int argc, char **argv)
 		print_version(invo_name);
 		done (0);
 
-	    case RCACHESW:
-		icachesw = &rcachesw;
-		goto do_cache;
-	    case WCACHESW:
-		icachesw = &wcachesw;
-do_cache:
-		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
-		switch (*icachesw = smatch (cp, cache_policy)) {
-		case AMBIGSW:
-		    ambigsw (cp, cache_policy);
-		    done (1);
-		case UNKWNSW:
-		    adios (NULL, "%s unknown", cp);
-		default:
-		    break;
-		}
-		continue;
-
-	    case CHECKSW:
-		checksw++;
-		continue;
-	    case NCHECKSW:
-		checksw = 0;
-		continue;
-
 	    case CONCATSW:
 		concatsw = 1;
 		continue;
@@ -185,32 +159,32 @@ do_cache:
 
 	    case PARTSW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		if (npart >= NPARTS)
-		    adios (NULL, "too many parts (starting with %s), %d max",
+		    die("too many parts (starting with %s), %d max",
 			   cp, NPARTS);
 		parts[npart++] = cp;
 		continue;
 
 	    case TYPESW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		if (ntype >= NTYPES)
-		    adios (NULL, "too many types (starting with %s), %d max",
+		    die("too many types (starting with %s), %d max",
 			   cp, NTYPES);
 		types[ntype++] = cp;
 		continue;
 
 	    case PREFERSW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		if (npreferred >= NPREFS)
-		    adios (NULL, "too many preferred types (starting with %s), %d max",
+		    die("too many preferred types (starting with %s), %d max",
 			   cp, NPREFS);
-		preferred_types[npreferred] = cp;
+		mime_preference[npreferred].type = cp;
 		cp = strchr(cp, '/');
 		if (cp) *cp++ = '\0';
-		preferred_subtypes[npreferred++] = cp;
+		mime_preference[npreferred++].subtype = cp;
 		continue;
 
 	    case NPREFERSW:
@@ -219,15 +193,15 @@ do_cache:
 
 	    case FILESW:
 		if (!(cp = *argp++) || (*cp == '-' && cp[1]))
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		file = *cp == '-' ? cp : path (cp, TFILE);
 		continue;
 
 	    case FORMSW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
-                mh_xfree(formsw);
-		formsw = getcpy (etcpath (cp));
+		    die("missing argument to %s", argp[-2]);
+                free(formsw);
+		formsw = mh_xstrdup(etcpath(cp));
 		continue;
 
 	    case HEADSW:
@@ -239,12 +213,12 @@ do_cache:
 
 	    case HEADFORMSW:
 		if (!(headerform = *argp++) || *headerform == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 
 	    case MARKFORMSW:
 		if (!(markerform = *argp++) || *markerform == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 
 	    /*
@@ -252,16 +226,16 @@ do_cache:
 	     */
 	    case PROGSW:
 		if (!(progsw = *argp++) || *progsw == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 	    case NPROGSW:
-		nomore++;
+		nomore = true;
 		continue;
 
 	    case LENSW:
 	    case WIDTHSW:
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 
 	    case VERBSW:
@@ -271,15 +245,14 @@ do_cache:
 		verbosw = 0;
 		continue;
 	    case DEBUGSW:
-		debugsw = 1;
+		debugsw = true;
 		continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
 	    if (folder)
-		adios (NULL, "only one folder at a time!");
-	    else
-		folder = pluspath (cp);
+		die("only one folder at a time!");
+            folder = pluspath (cp);
 	} else
 		app_msgarg(&msgs, cp);
     }
@@ -305,7 +278,7 @@ do_cache:
      */
     if ((cp = getenv ("MHSHOW"))) {
 	if ((fp = fopen (cp, "r"))) {
-	    readconfig ((struct node **) 0, fp, cp, 0);
+	    readconfig(NULL, fp, cp, 0);
 	    fclose (fp);
 	} else {
 	    admonish ("", "unable to read $MHSHOW profile (%s)", cp);
@@ -316,24 +289,15 @@ do_cache:
      * Read the standard profile setup
      */
     if ((fp = fopen (cp = etcpath ("mhn.defaults"), "r"))) {
-	readconfig ((struct node **) 0, fp, cp, 0);
+	readconfig(NULL, fp, cp, 0);
 	fclose (fp);
     }
 
-    /* Check for public cache location */
-    if ((cache_public = context_find (nmhcache)) && *cache_public != '/')
-	cache_public = NULL;
-
-    /* Check for private cache location */
-    if (!(cache_private = context_find (nmhprivcache)))
-	cache_private = ".cache";
-    cache_private = getcpy (m_maildir (cache_private));
-
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
 
     if (file && msgs.size)
-	adios (NULL, "cannot specify msg and file at same time!");
+	die("cannot specify msg and file at same time!");
 
     /*
      * check if message is coming from file
@@ -356,16 +320,16 @@ do_cache:
 	    folder = getfolder (1);
 	maildir = m_maildir (folder);
 
-	if (chdir (maildir) == NOTOK)
+	if (chdir (maildir) == -1)
 	    adios (maildir, "unable to change directory to");
 
 	/* read folder and create message structure */
 	if (!(mp = folder_read (folder, 1)))
-	    adios (NULL, "unable to read folder %s", folder);
+	    die("unable to read folder %s", folder);
 
 	/* check for empty folder */
 	if (mp->nummsg == 0)
-	    adios (NULL, "no messages in %s", folder);
+	    die("no messages in %s", folder);
 
 	/* parse all the message ranges/sequences and set SELECTED */
 	for (msgnum = 0; msgnum < msgs.size; msgnum++)
@@ -404,7 +368,7 @@ do_cache:
     if (!*cts)
 	done (1);
 
-    userrs = 1;
+    userrs = true;
     SIGNAL (SIGQUIT, quitser);
     SIGNAL (SIGPIPE, pipeser);
 
@@ -416,7 +380,7 @@ do_cache:
 
 	ct = *ctp;
 	if (type_ok (ct, 1) && !ct->c_umask) {
-	    if (stat (ct->c_file, &st) != NOTOK)
+	    if (stat (ct->c_file, &st) != -1)
 		ct->c_umask = ~(st.st_mode & 0777);
 	    else
 		ct->c_umask = ~m_gmprot();
