@@ -5,48 +5,53 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "sbr/pidwait.h"
+#include "sbr/charstring.h"
+#include "sbr/fmt_new.h"
+#include "sbr/concat.h"
+#include "sbr/trimcpy.h"
+#include "sbr/check_charset.h"
+#include "sbr/getcpy.h"
+#include "sbr/context_find.h"
+#include "sbr/pidstatus.h"
+#include "sbr/utils.h"
+#include "sbr/arglist.h"
+#include "sbr/error.h"
 #include <fcntl.h>
-#include <h/signals.h>
-#include <h/md5.h>
-#include <h/mts.h>
-#include <h/tws.h>
-#include <h/mime.h>
-#include <h/mhparse.h>
-#include <h/fmt_scan.h>
-#include <h/utils.h>
+#include <signal.h>
+#include "sbr/signals.h"
+#include "sbr/mts.h"
+#include "h/tws.h"
+#include "h/mime.h"
+#include "h/mhparse.h"
+#include "sbr/fmt_compile.h"
+#include "sbr/fmt_scan.h"
+#include "sbr/done.h"
+#include "mhmisc.h"
 #include "mhshowsbr.h"
-#include "../sbr/m_mktemp.h"
+#include "sbr/m_mktemp.h"
+#include "sbr/path.h"
+#include "sbr/globals.h"
 #ifdef HAVE_ICONV
 #   include <iconv.h>
 #endif /* ! HAVE_ICONV */
 
-extern int debugsw;
-extern int npart;
-extern int ntype;
-
-int nolist   = 0;
+extern bool debugsw;
 
 char *progsw = NULL;
 
 /* flags for moreproc/header display */
-int nomore   = 0;
+bool nomore;
 char *formsw = NULL;
 
-/* for output markerss and headers */
+/* for output markers and headers */
 char *folder = NULL;
 char *markerform;
 char *headerform;
 int headersw = -1;
 
 
-/* mhmisc.c */
-int part_ok (CT);
-int part_exact (CT);
-int type_ok (CT, int);
-void content_error (char *, CT, char *, ...);
-void flush_errors (void);
-
 /*
  * static prototypes
  */
@@ -60,7 +65,6 @@ static int show_multi (CT, int, int, int, int, struct format *);
 static int show_multi_internal (CT, int, int, int, int, struct format *);
 static int show_multi_aux (CT, int, char *, struct format *);
 static int show_message_rfc822 (CT, int, struct format *);
-static int show_partial (CT, int);
 static int show_external (CT, int, int, int, int, struct format *);
 static int parse_display_string (CT, char *, int *, int *, char *, char *,
 				 size_t, int multipart);
@@ -101,20 +105,13 @@ void
 show_all_messages(CT *cts, int concat, int textonly, int inlineonly)
 {
     CT ct, *ctp;
-    struct format *hfmt, *mfmt;
 
     /*
      * If form is not specified, then get default form
      * for showing headers of MIME messages.
      */
     if (!formsw)
-	formsw = getcpy (etcpath ("mhl.headers"));
-
-    /*
-     * Compile the content marker and header format lines
-     */
-    mfmt = compile_marker(markerform);
-    hfmt = compile_header(headerform);
+	formsw = mh_xstrdup(etcpath("mhl.headers"));
 
     /*
      * If form is "mhl.null", suppress display of header.
@@ -127,16 +124,30 @@ show_all_messages(CT *cts, int concat, int textonly, int inlineonly)
 
 	/* if top-level type is ok, then display message */
 	if (type_ok (ct, 1)) {
-	    if (headersw) output_header(ct, hfmt);
+	    /*
+	     * Compile the content marker format line
+	     */
+	    struct format *mfmt = compile_marker(markerform);
+
+	    if (headersw) {
+		/*
+		 * Compile the header format line
+		 */
+		struct format *hfmt = compile_header(headerform);
+		output_header(ct, hfmt);
+		fmt_free(hfmt, 0);
+	    }
 
 	    show_single_message (ct, formsw, concat, textonly, inlineonly,
 				 mfmt);
+	    fmt_free(mfmt, 0);
 	}
     }
 
+    /* Reset the format component hashtable, which free's any components which
+       it references. */
+    fmt_free(NULL, 1);
     free_markercomps();
-    fmt_free(hfmt, 1);
-    fmt_free(mfmt, 1);
 }
 
 
@@ -181,7 +192,7 @@ show_single_message (CT ct, char *form, int concatsw, int textonly,
     sigaddset (&set, SIGTERM);
     sigprocmask (SIG_BLOCK, &set, &oset);
 
-    while (!concatsw && wait (&status) != NOTOK) {
+    while (!concatsw && wait (&status) != -1) {
 	pidcheck (status);
 	continue;
     }
@@ -201,7 +212,7 @@ static void
 DisplayMsgHeader (CT ct, char *form, int concatsw)
 {
     pid_t child_id;
-    int i, vecp;
+    int vecp;
     char **vec;
     char *file;
 
@@ -225,9 +236,7 @@ DisplayMsgHeader (CT ct, char *form, int concatsw)
 
     fflush (stdout);
 
-    for (i = 0; (child_id = fork()) == NOTOK && i < 5; i++)
-	sleep (5);
-
+    child_id = fork();
     switch (child_id) {
     case NOTOK:
 	adios ("fork", "unable to");
@@ -237,7 +246,7 @@ DisplayMsgHeader (CT ct, char *form, int concatsw)
 	execvp (file, vec);
 	fprintf (stderr, "unable to exec ");
 	perror (mhlproc);
-	_exit (-1);
+	_exit(1);
 	/* NOTREACHED */
 
     default:
@@ -265,9 +274,6 @@ show_switch (CT ct, int alternate, int concatsw, int textonly, int inlineonly,
 
 	case CT_MESSAGE:
 	    switch (ct->c_subtype) {
-		case MESSAGE_PARTIAL:
-		    return show_partial (ct, alternate);
-
 		case MESSAGE_EXTERNAL:
 		    return show_external (ct, alternate, concatsw, textonly,
 					  inlineonly, fmt);
@@ -281,7 +287,7 @@ show_switch (CT ct, int alternate, int concatsw, int textonly, int inlineonly,
 		 */
 		default:
 		    return show_content (ct, alternate, textonly,
-		    			 inlineonly, fmt);
+					 inlineonly, fmt);
 	    }
 
 	case CT_TEXT:
@@ -379,12 +385,12 @@ show_content_aux (CT ct, int alternate, char *cp, char *cracked, struct format *
     }
 
     if (cracked) {
-	strncpy (buffer, cp, sizeof(buffer));
+	strncpy (buffer, cp, sizeof(buffer) - 1);
 	goto got_command;
     }
 
     if (parse_display_string (ct, cp, &xstdin, &xlist, file, buffer,
-    			      sizeof(buffer) - 1, 0)) {
+			      sizeof(buffer) - 1, 0)) {
 	inform("Buffer overflow constructing show command, continuing...");
 	return NOTOK;
     }
@@ -404,7 +410,7 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
                    int fd, int xlist, int xstdin, struct format *fmt)
 {
     pid_t child_id;
-    int i, vecp;
+    int vecp;
     char **vec, *file;
 
     if (debugsw || cracked) {
@@ -441,7 +447,7 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
 	}
 
 	while ((cc = read(fd, readbuf, sizeof(readbuf))) > 0) {
-	    if ((ssize_t) fwrite(readbuf, sizeof(char), cc, stdout) < cc) {
+	    if ((ssize_t) fwrite(readbuf, 1, cc, stdout) < cc) {
 		advise ("putline", "fwrite");
 	    }
 	    lastchar = readbuf[cc - 1];
@@ -449,7 +455,7 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
 
 	if (cc < 0) {
 	    advise("read", "while reading text content");
-	    return NOTOK;
+	    return -1;
 	}
 
 	/*
@@ -470,12 +476,10 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
     }
 
     vec = argsplit(buffer, &file, &vecp);
-    vec[vecp++] = NULL;
 
     fflush (stdout);
 
-    for (i = 0; (child_id = fork()) == NOTOK && i < 5; i++)
-	sleep (5);
+    child_id = fork();
     switch (child_id) {
 	case NOTOK:
 	    advise ("fork", "unable to");
@@ -494,7 +498,7 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
 	    execvp (file, vec);
 	    fprintf (stderr, "unable to exec ");
 	    perror (buffer);
-	    _exit (-1);
+	    _exit(1);
 	    /* NOTREACHED */
 
 	default: {
@@ -510,7 +514,7 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
             arglist_free(file, vec);
 	    if (fd != NOTOK)
 		(*ct->c_ceclosefnx) (ct);
-	    return (alternate ? OK : status);
+	    return alternate ? OK : status;
         }
     }
 }
@@ -592,9 +596,9 @@ show_multi_internal (CT ct, int alternate, int concatsw, int textonly,
     int	alternating, nowalternate, result;
     struct multipart *m = (struct multipart *) ct->c_ctparams;
     struct part *part;
-    int request_matched;
-    int display_success;
-    int mult_alt_done;
+    bool request_matched;
+    bool display_success;
+    bool mult_alt_done;
     int ret;
     CT p;
 
@@ -612,9 +616,9 @@ show_multi_internal (CT ct, int alternate, int concatsw, int textonly,
      */
 
     result = NOTOK;
-    request_matched = 0;
-    display_success = 0;
-    mult_alt_done = 0;
+    request_matched = false;
+    display_success = false;
+    mult_alt_done = false;
 
     for (part = m->mp_parts; part; part = part->mp_next) {
 	p = part->mp_part;
@@ -635,7 +639,7 @@ show_multi_internal (CT ct, int alternate, int concatsw, int textonly,
 				      inlineonly, fmt);
 	    switch (inneresult) {
 		case NOTOK:  /* hard display error */
-		    request_matched = 1;
+		    request_matched = true;
 		    if (alternate && !alternating) {
 			result = NOTOK;
 			goto out;
@@ -646,8 +650,8 @@ show_multi_internal (CT ct, int alternate, int concatsw, int textonly,
 		    continue;
 
 		case OK:  /* display successful */
-		    request_matched = 1;
-		    display_success = 1;
+		    request_matched = true;
+		    display_success = true;
 		    result = OK;
 
 		    /* if we got success on a sub-part of
@@ -655,7 +659,7 @@ show_multi_internal (CT ct, int alternate, int concatsw, int textonly,
 		     * there's a chance an explicit part should be
 		     * matched later in the alternatives.  */
 		    if (alternating) {
-			mult_alt_done = 1;
+			mult_alt_done = true;
 		    } else if (alternate) {
 			alternate = nowalternate = 0;
 		    }
@@ -714,7 +718,7 @@ show_multi_aux (CT ct, int alternate, char *cp, struct format *fmt)
 	    if ((*p->c_ceopenfnx) (p, &file) == NOTOK)
 		return NOTOK;
 
-	    p->c_storage = add (file, NULL);
+	    p->c_storage = mh_xstrdup(FENDNULL(file));
 
 	    if (p->c_showproc && !strcmp (p->c_showproc, "true"))
 		return OK;
@@ -763,21 +767,6 @@ show_message_rfc822 (CT ct, int alternate, struct format *fmt)
 }
 
 
-/*
- * Show content of type "message/partial".
- */
-
-static int
-show_partial (CT ct, int alternate)
-{
-    NMH_UNUSED (alternate);
-
-    content_error (NULL, ct,
-	"in order to display this message, you must reassemble it");
-    return NOTOK;
-}
-
-
 /*
  * Show content of type "message/external".
  *
@@ -801,8 +790,10 @@ show_external (CT ct, int alternate, int concatsw, int textonly, int inlineonly,
 static int
 parse_display_string (CT ct, char *cp, int *xstdin, int *xlist,
                       char *file, char *buffer, size_t buflen,
-                      int multipart) {
-    int len, quoted = 0;
+                      int multipart)
+{
+    int len;
+    bool quoted = false;
     char *bp = buffer, *pp;
     CI ci = &ct->c_ctinfo;
 
@@ -869,7 +860,7 @@ parse_display_string (CT ct, char *cp, int *xstdin, int *xlist,
 		    }
 		} else {
 		    /* insert filename containing content */
-		    snprintf (bp, buflen, "%s", file);
+		    abortcpy(bp, file, buflen);
 
 		    /*
 		     * Old comments below are left here for posterity.
@@ -906,7 +897,7 @@ parse_display_string (CT ct, char *cp, int *xstdin, int *xlist,
 
 	    case 'l':
 		/* display listing prior to displaying content */
-		*xlist = !nolist;
+		*xlist = 1;
 		break;
 
 	    case 's':
@@ -973,7 +964,7 @@ parse_display_string (CT ct, char *cp, int *xstdin, int *xlist,
 		    *pp++ = '\'';
 		    buflen--;
 		    bp++;
-		    quoted = 1;
+		    quoted = true;
 		}
 		/* Escape existing quotes */
 		while ((pp = strchr (pp, '\'')) && buflen > 3) {
@@ -988,7 +979,7 @@ parse_display_string (CT ct, char *cp, int *xstdin, int *xlist,
 			*pp++ = '\'';
 			buflen -= 2;
 			bp += 2;
-			quoted = 0;
+			quoted = false;
 		    } else {
 			/* Not quoted.  This should not be reached with
 			   the current code, but handle the condition
@@ -1027,7 +1018,7 @@ parse_display_string (CT ct, char *cp, int *xstdin, int *xlist,
 		    if (! found_quote) {
 			*bp++ = '\'';
 			buflen--;
-			quoted = 0;
+			quoted = false;
 		    }
 		}
 	    }
@@ -1056,7 +1047,7 @@ raw:
     if (ct->c_termproc) {
 	char term[BUFSIZ];
 
-	strncpy (term, buffer, sizeof(term));
+	strncpy (term, buffer, sizeof(term) - 1);
 	snprintf (buffer, buflen, ct->c_termproc, term);
     }
 
@@ -1065,7 +1056,8 @@ raw:
 
 
 int
-convert_charset (CT ct, char *dest_charset, int *message_mods) {
+convert_charset (CT ct, char *dest_charset, int *message_mods)
+{
     char *src_charset = content_charset (ct);
     int status = OK;
 
@@ -1078,7 +1070,7 @@ convert_charset (CT ct, char *dest_charset, int *message_mods) {
         FILE **fp = NULL;
         size_t begin;
         size_t end;
-        int opened_input_file = 0;
+        bool opened_input_file = false;
         char src_buffer[BUFSIZ];
 	size_t dest_buffer_size = BUFSIZ;
 	char *dest_buffer = mh_xmalloc(dest_buffer_size);
@@ -1094,7 +1086,7 @@ convert_charset (CT ct, char *dest_charset, int *message_mods) {
         }
 
         if ((tempfile = m_mktemp2 (NULL, invo_name, &fd, NULL)) == NULL) {
-            adios (NULL, "unable to create temporary file in %s",
+            die("unable to create temporary file in %s",
                    get_temp_dir());
         }
         dest = mh_xstrdup(tempfile);
@@ -1116,7 +1108,7 @@ convert_charset (CT ct, char *dest_charset, int *message_mods) {
                     advise (*file, "unable to open for reading");
                     status = NOTOK;
                 } else {
-                    opened_input_file = 1;
+                    opened_input_file = true;
                 }
             }
         }
@@ -1220,7 +1212,7 @@ iconv_start:
 		replace_param(&ct->c_ctinfo.ci_first_pm,
 			      &ct->c_ctinfo.ci_last_pm, "charset",
 			      dest_charset, 0);
-		outline = output_params(strlen(TYPE_FIELD) + 1 + strlen(ctline),
+		outline = output_params(LEN(TYPE_FIELD) + 1 + strlen(ctline),
 					ct->c_ctinfo.ci_first_pm, NULL, 0);
 		if (outline) {
 		    ctline = add(outline, ctline);
@@ -1261,7 +1253,8 @@ iconv_start:
 
 
 static int
-convert_content_charset (CT ct, char **file) {
+convert_content_charset (CT ct, char **file)
+{
     int status = OK;
 
 #ifdef HAVE_ICONV
@@ -1313,7 +1306,7 @@ compile_header(char *form)
 
     fmtstring = new_fs(form, NULL, DEFAULT_HEADER);
 
-    (void) fmt_compile(fmtstring, &fmt, 1);
+    (void) fmt_compile(fmtstring, &fmt, 0);
     free_fs();
 
     while ((comp = fmt_nextcomp(comp, &bucket)) != NULL) {
@@ -1336,7 +1329,7 @@ compile_marker(char *form)
 
     fmtstring = new_fs(form, NULL, DEFAULT_MARKER);
 
-    (void) fmt_compile(fmtstring, &fmt, 1);
+    (void) fmt_compile(fmtstring, &fmt, 0);
     free_fs();
 
     /*
