@@ -11,22 +11,35 @@
 #endif /* ! NMH */
 
 #ifdef HAVE_CONFIG_H
-#   include <config.h>
+#   include "config.h"
 #endif /* HAVE_CONFIG_H */
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdbool.h>
 #include <string.h>
 #include <stdio.h>
+#include <assert.h>
+
+#if NMH
+#include "h/mh.h"
+#include "sbr/getarguments.h"
+#include "sbr/smatch.h"
+#include "sbr/ambigsw.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
+#include "sbr/globals.h"
+#endif
 
-#include <h/utils.h>
 
 #if ! defined HAVE_MKSTEMPS
 #   define HAVE_MKSTEMPS 0
 #endif /* ! HAVE_MKSTEMPS */
 
-char *build_template(const char *, const char *, const char *);
-void process_args(int, char **, const char **, const char **, const char **);
+static char *build_template(const char *, const char *, const char *);
+static void process_args(int, char **, const char **, const char **, const char **);
 
 /*
  * Use a template of the form:
@@ -36,7 +49,8 @@ void process_args(int, char **, const char **, const char **, const char **);
  */
 
 int
-main(int argc, char *argv[]) {
+main(int argc, char *argv[])
+{
     const char *directory = "", *prefix = "", *suffix = "";
     size_t suffix_len;
     int fd;
@@ -44,7 +58,7 @@ main(int argc, char *argv[]) {
 
     process_args(argc, argv, &directory, &prefix, &suffix);
     if ((template = build_template(directory, prefix, suffix)) == NULL) {
-        return -1;
+        return 1;
     }
 
     if ((suffix_len = strlen(suffix)) > 0) {
@@ -64,12 +78,13 @@ main(int argc, char *argv[]) {
 
     free(template);
 
-    return fd >= 0  ?  0  :  -1;
+    return fd >= 0  ?  0  :  1;
 }
 
 
-char *
-build_template(const char *directory, const char *prefix, const char *suffix) {
+static char *
+build_template(const char *directory, const char *prefix, const char *suffix)
+{
     const char pattern[] = "XXXXXX";
     size_t len, directory_len, pathsep_len, prefix_len, suffix_len;
     char *template;
@@ -87,39 +102,42 @@ build_template(const char *directory, const char *prefix, const char *suffix) {
     }
     prefix_len = strlen(prefix);
     suffix_len = strlen(suffix);
-    /* sizeof pattern includes its final NULL, so don't add another. */
-    len = directory_len  +  pathsep_len  +  prefix_len  +   sizeof pattern  +
-        suffix_len;
+    len = directory_len + pathsep_len +
+        prefix_len + sizeof pattern + suffix_len;
 
-    if ((template = malloc(len))) {
-        char *tp = template;
-
-        (void) strncpy(tp, directory, directory_len);
-        tp += directory_len;
-
-        if (pathsep_len == 1) { *tp++ = '/'; }
-
-        (void) strncpy(tp, prefix, prefix_len);
-        tp += prefix_len;
-
-        (void) strncpy(tp, pattern, sizeof pattern - 1);
-        tp += sizeof pattern - 1;
-
-        (void) strncpy(tp, suffix, suffix_len);
-        /* tp += suffix_len; */
-
-        template[len-1] = '\0';
-
-        return template;
+    if (!(template = malloc(len))) {
+#if NMH
+        advise("", "build_template: malloc(%zu) failed", len);   /* "" prints errno! */
+#else
+        perror("malloc");
+#endif
+        return NULL;
     }
 
-    perror("malloc");
-    return NULL;
+    char *tp = template;
+
+    memcpy(tp, directory, directory_len);
+    tp += directory_len;
+
+    if (pathsep_len == 1) { *tp++ = '/'; }
+
+    memcpy(tp, prefix, prefix_len);
+    tp += prefix_len;
+
+    memcpy(tp, pattern, sizeof pattern - 1);
+    tp += sizeof pattern - 1;
+
+    memcpy(tp, suffix, suffix_len);
+    tp += suffix_len;
+
+    *tp = '\0';
+    assert(tp == template + len - 1);
+
+    return template;
 }
 
 
 #if NMH
-#include <h/mh.h>
 
 #if HAVE_MKSTEMPS
 #   define MHFIXMSG_SWITCHES \
@@ -144,15 +162,16 @@ DEFINE_SWITCH_ENUM(MHFIXMSG);
 DEFINE_SWITCH_ARRAY(MHFIXMSG, switches);
 #undef X
 
-void
+static void
 process_args(int argc, char **argv, const char **directory,
-             const char **prefix, const char **suffix) {
+             const char **prefix, const char **suffix)
+{
     char **argp, **arguments, *cp, buf[100];
 #   if ! HAVE_MKSTEMPS
     NMH_UNUSED(suffix);
 #   endif /* ! HAVE_MKSTEMPS */
 
-    if (nmh_init(argv[0], 2)) { done(NOTOK); }
+    if (nmh_init(argv[0], true, false)) { done(1); }
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
@@ -164,12 +183,12 @@ process_args(int argc, char **argv, const char **directory,
             switch (smatch(++cp, switches)) {
             case AMBIGSW:
                 ambigsw(cp, switches);
-                done(NOTOK);
+                done(1);
             case UNKWNSW:
                 inform("-%s unknown", cp);
                 (void) snprintf(buf, sizeof buf, "%s [switches]", invo_name);
                 print_help(buf, switches, 1);
-                done(NOTOK);
+                done(1);
             case HELPSW:
                 (void) snprintf(buf, sizeof buf, "%s [switches]", invo_name);
                 print_help(buf, switches, 1);
@@ -181,7 +200,7 @@ process_args(int argc, char **argv, const char **directory,
             case DIRECTORYSW:
                 /* Allow the directory to start with '-'. */
                 if ((cp = *argp++) == NULL) {
-                    adios(NULL, "missing argument to %s", argp[-2]);
+                    die("missing argument to %s", argp[-2]);
                 }
                 *directory = cp;
                 continue;
@@ -189,7 +208,7 @@ process_args(int argc, char **argv, const char **directory,
             case PREFIXSW:
                 /* Allow the prefix to start with '-'. */
                 if ((cp = *argp++) == NULL) {
-                    adios(NULL, "missing argument to %s", argp[-2]);
+                    die("missing argument to %s", argp[-2]);
                 }
                 *prefix = cp;
                 continue;
@@ -198,7 +217,7 @@ process_args(int argc, char **argv, const char **directory,
             case SUFFIXSW:
                 /* Allow the suffix to start with '-'. */
                 if ((cp = *argp++) == NULL) {
-                    adios(NULL, "missing argument to %s", argp[-2]);
+                    die("missing argument to %s", argp[-2]);
                 }
                 *suffix = cp;
                 continue;
@@ -208,9 +227,10 @@ process_args(int argc, char **argv, const char **directory,
     }
 }
 #else  /* ! NMH */
-void
+static void
 process_args(int argc, char **argv, const char **directory,
-             const char **prefix, const char **suffix) {
+             const char **prefix, const char **suffix)
+{
 #   if HAVE_MKSTEMPS
     const char usage[] =
         "usage: %s [-h] [-d directory] [-p prefix] [-s suffix]\n";
@@ -237,7 +257,7 @@ process_args(int argc, char **argv, const char **directory,
             exit(0);
         default:
             (void) fprintf(stderr, usage, argv[0]);
-            exit(-1);
+            exit(1);
         }
     }
 }
