@@ -5,13 +5,22 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/mts.h>
-#include <h/tws.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "sbr/dtime.h"
+#include "sbr/getarguments.h"
+#include "sbr/smatch.h"
+#include "sbr/ambigsw.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
+#include "sbr/mts.h"
+#include "h/tws.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
 #include <pwd.h>
 
-#include <h/popsbr.h>
+#include "popsbr.h"
+#include "sbr/globals.h"
 
 #ifndef CYRUS_SASL
 # define SASLminc(a) (a)
@@ -40,6 +49,7 @@
     X("nosasl", SASLminc(6), NOSASLSW) \
     X("saslmech", SASLminc(5), SASLMECHSW) \
     X("authservice", SASLminc(0), AUTHSERVICESW) \
+    X("tls", TLSminc(-3), TLSSW) \
     X("initialtls", TLSminc(-10), INITTLSSW) \
     X("notls", TLSminc(-5), NOTLSSW) \
     X("certverify", TLSminc(-10), CERTVERSW) \
@@ -60,26 +70,15 @@ DEFINE_SWITCH_ARRAY(MSGCHK, switches);
  */
 #define MAXVEC  51
 
-#define	NT_NONE	0x0
-#ifdef NT_NONE
-#endif /* Use NT_NONE to prevent warning from gcc -Wunused-macros. */
 #define	NT_MAIL	0x1
 #define	NT_NMAI	0x2
 #define	NT_ALL	(NT_MAIL | NT_NMAI)
 
-#define	NONEOK	0x0
-#define	UUCPOLD	0x1
-#define	UUCPNEW	0x2
-#define	UUCPOK	(UUCPOLD | UUCPNEW)
-#define	MMDFOLD	0x4
-#define	MMDFNEW	0x8
-#define	MMDFOK	(MMDFOLD | MMDFNEW)
-
 
 /*
  * static prototypes
  */
-static int donote (char *, int);
+static int donote (char *, int) PURE;
 static int checkmail (char *, char *, int, int, int);
 static int remotemail (char *, char *, char *, char *, int, int, int, int,
 		       char *, int, const char *);
@@ -88,15 +87,20 @@ static int remotemail (char *, char *, char *, char *, int, int, int, int,
 int
 main (int argc, char **argv)
 {
-    int datesw = 1, notifysw = NT_ALL;
-    int status = 0, sasl = 0, tls = 0, noverify = 0;
-    int snoop = 0, vecp = 0;
+    bool datesw = true;
+    int notifysw = NT_ALL;
+    int status = 0;
+    bool sasl = false;
+    int tls = 0;
+    bool noverify = false;
+    bool snoop = false;
+    int vecp = 0;
     char *cp, *host = NULL, *port = NULL, *user = NULL, *proxy = NULL;
     char buf[BUFSIZ], *saslmech = NULL, *auth_svc = NULL;
     char **argp, **arguments, *vec[MAXVEC];
     struct passwd *pw;
 
-    if (nmh_init(argv[0], 1)) { return 1; }
+    if (nmh_init(argv[0], true, true)) { return 1; }
 
     mts_init ();
 
@@ -110,7 +114,7 @@ main (int argc, char **argv)
 		    ambigsw (cp, switches);
 		    done (1);
 		case UNKWNSW: 
-		    adios (NULL, "-%s unknown", cp);
+		    die("-%s unknown", cp);
 
 		case HELPSW: 
 		    snprintf (buf, sizeof(buf), "%s [switches] [users ...]",
@@ -122,61 +126,64 @@ main (int argc, char **argv)
 		    done (0);
 
 		case DATESW:
-		    datesw++;
+		    datesw = true;
 		    continue;
 		case NDATESW:
-		    datesw = 0;
+		    datesw = false;
 		    continue;
 
 		case NOTESW:
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    notifysw |= donote (cp, 1);
 		    continue;
 		case NNOTESW:
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    notifysw &= ~donote (cp, 0);
 		    continue;
 
 		case HOSTSW: 
 		    if (!(host = *argp++) || *host == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 
 		case PORTSW:
 		    if (!(port = *argp++) || *port == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		continue;
 
 		case USERSW: 
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    if (vecp >= MAXVEC-1)
-			adios (NULL, "you can only check %d users at a time", MAXVEC-1);
-		    else
-			user = vec[vecp++] = cp;
+			die("you can only check %d users at a time", MAXVEC-1);
+                    user = vec[vecp++] = cp;
 		    continue;
 
 		case SNOOPSW:
-		    snoop++;
+		    snoop = true;
 		    continue;
 
 		case SASLSW:
-		    sasl++;
+		    sasl = true;
 		    continue;
 		
 		case NOSASLSW:
-		    sasl = 0;
+		    sasl = false;
 		    continue;
 
 		case SASLMECHSW:
 		    if (!(saslmech = *argp++) || *saslmech == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
+		    continue;
+
+		case TLSSW:
+		    tls = 1;
 		    continue;
 
 		case INITTLSSW:
-		    tls++;
+		    tls = 2;
 		    continue;
 
 		case NOTLSSW:
@@ -184,32 +191,31 @@ main (int argc, char **argv)
 		    continue;
 
 		case CERTVERSW:
-		    noverify = 0;
+		    noverify = false;
 		    continue;
 
 		case NOCERTVERSW:
-		    noverify++;
+		    noverify = true;
 		    continue;
 
 		case AUTHSERVICESW:
 #ifdef OAUTH_SUPPORT
 		    if (!(auth_svc = *argp++) || *auth_svc == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 #else
-		    adios (NULL, "not built with OAuth support");
+		    die("not built with OAuth support");
 #endif
 		    continue;
 
 		case PROXYSW:
 		    if (!(proxy = *argp++) || *proxy == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 	    }
 	}
 	if (vecp >= MAXVEC-1)
-	    adios (NULL, "you can only check %d users at a time", MAXVEC-1);
-	else
-	    vec[vecp++] = cp;
+	    die("you can only check %d users at a time", MAXVEC-1);
+        vec[vecp++] = cp;
     }
 
     /*
@@ -232,8 +238,10 @@ main (int argc, char **argv)
     if (host) {
 	int tlsflag = 0;
 
-	if (tls)
-	    tlsflag |= P_INITTLS;
+	if (tls == 1)
+	    tlsflag = P_STARTTLS;
+	else if (tls == 2)
+	    tlsflag = P_INITTLS;
 
 	if (noverify)
 	    tlsflag |= P_NOVERIFY;
@@ -247,7 +255,7 @@ main (int argc, char **argv)
 				      snoop, sasl, saslmech, tlsflag, auth_svc);
 	}
     } else {
-	if (user == NULL) user = getusername ();
+	if (user == NULL) user = getusername (1);
 	if (vecp == 0) {
 	    char *home;
 
@@ -255,7 +263,7 @@ main (int argc, char **argv)
 	    if (!geteuid() || NULL == (home = getenv("HOME"))) {
 		pw = getpwnam (user);
 		if (pw == NULL)
-		    adios (NULL, "unable to get information about user");
+		    die("unable to get information about user");
 		home = pw->pw_dir;
 	    }
 	    status = checkmail (user, home, datesw, notifysw, 1);
@@ -296,7 +304,7 @@ donote (char *cp, int ntflag)
 	    ambigsw (cp, ntswitches);
 	    done (1);
 	case UNKWNSW: 
-	    adios (NULL, "-%snotify %s unknown", ntflag ? "" : "no", cp);
+	    die("-%snotify %s unknown", ntflag ? "" : "no", cp);
 
 	case NALLSW: 
 	    return NT_ALL;
@@ -313,73 +321,57 @@ donote (char *cp, int ntflag)
 static int
 checkmail (char *user, char *home, int datesw, int notifysw, int personal)
 {
-    int mf, status;
     char buffer[BUFSIZ];
+    snprintf(buffer, sizeof buffer, "%s/%s", *mmdfldir ? mmdfldir : home,
+        *mmdflfil ? mmdflfil : user);
+
     struct stat st;
+    bool statok = stat(buffer, &st) != -1;
+    bool empty = !statok || st.st_size == 0;
 
-    snprintf (buffer, sizeof(buffer), "%s/%s", mmdfldir[0] ? mmdfldir : home, mmdflfil[0] ? mmdflfil : user);
-    if (datesw) {
-	st.st_size = 0;
-	st.st_atime = st.st_mtime = 0;
-    }
-    mf = (stat (buffer, &st) == NOTOK || st.st_size == 0) ? NONEOK
-	: st.st_atime <= st.st_mtime ? MMDFNEW : MMDFOLD;
-
-    if ((mf & UUCPOK) || (mf & MMDFOK)) {
-	if (notifysw & NT_MAIL) {
-	    if (personal)
-		printf ("You have ");
-	    else
-		printf ("%s has ", user);
-	    if (mf & UUCPOK)
-		printf ("%s old-style bell", mf & UUCPOLD ? "old" : "new");
-	    if ((mf & UUCPOK) && (mf & MMDFOK))
-		printf (" and ");
-	    if (mf & MMDFOK)
-		printf ("%s%s", mf & MMDFOLD ? "old" : "new",
-			mf & UUCPOK ? " Internet" : "");
-	    printf (" mail waiting");
-	} else {
-	    notifysw = 0;
-	}
-	status = 0;
-    }
-    else {
-	if (notifysw & NT_NMAI)
-	    printf (personal ? "You don't %s%s" : "%s doesn't %s",
-		    personal ? "" : user, "have any mail waiting");
-	else
-	    notifysw = 0;
-
-	status = 1;
+    if ((empty && !(notifysw & NT_NMAI)) ||
+        (!empty && !(notifysw & NT_MAIL))) {
+        return empty;
     }
 
-    if (notifysw)
-	if (datesw && st.st_atime)
-	    printf ("; last read on %s", dtime (&st.st_atime, 1));
-    if (notifysw)
-	putchar('\n');
+    if (empty) {
+        if (personal) {
+            fputs("You don't have any mail waiting", stdout);
+        } else {
+            printf("%s doesn't have any mail waiting", user);
+        }
+    } else {
+        char *kind = st.st_mtime < st.st_atime ? "old" : "new";
+        if (personal) {
+            printf("You have %s mail waiting", kind);
+        } else {
+            printf("%s has %s mail waiting", user, kind);
+        }
+    }
 
-    return status;
+    if (datesw && statok && st.st_atime)
+        printf("; last read on %s", dtime(&st.st_atime, 1));
+    putchar('\n');
+
+    return empty;
 }
 
 
-extern char response[];
-
 static int
 remotemail (char *host, char *port, char *user, char *proxy, int notifysw,
 	    int personal, int snoop, int sasl, char *saslmech, int tls,
 	    const char *auth_svc)
 {
-    int nmsgs, nbytes, status;
+    int nmsgs, nbytes;
+    bool status;
 
     if (auth_svc == NULL) {
 	if (saslmech  &&  ! strcasecmp(saslmech, "xoauth2")) {
-	    adios (NULL, "must specify -authservice with -saslmech xoauth2");
+	    die("must specify -authservice with -saslmech xoauth2");
 	}
     } else {
 	if (user == NULL) {
-	    adios (NULL, "must specify -user with -saslmech xoauth2");
+	    die("must specify -user with -saslmech xoauth2");
 	}
     }
 
@@ -395,24 +387,23 @@ remotemail (char *host, char *port, char *user, char *proxy, int notifysw,
     if (nmsgs) {
 	if (notifysw & NT_MAIL) {
 	    if (personal)
-		printf ("You have ");
+		fputs("You have ", stdout);
 	    else
 		printf ("%s has ", user);
 
 	    printf ("%d message%s (%d bytes)",
 		    nmsgs, PLURALS(nmsgs), nbytes);
-	}
-	else
+	} else
 	    notifysw = 0;
 
-	status = 0;
+	status = false;
     } else {
 	if (notifysw & NT_NMAI)
 	    printf (personal ? "You don't %s%s" : "%s doesn't %s",
 		    personal ? "" : user, "have any mail waiting");
 	else
 	    notifysw = 0;
-	status = 1;
+	status = true;
     }
     if (notifysw)
 	printf (" on %s\n", host);
