@@ -1,16 +1,22 @@
-
-/*
- * picksbr.c -- routines to help pick along...
+/* picksbr.c -- routines to help pick along...
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/tws.h>
-#include <h/picksbr.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "picksbr.h"
+#include "sbr/dtime.h"
+#include "sbr/dtime.h"
+#include "sbr/m_getfld.h"
+#include "sbr/smatch.h"
+#include "sbr/fmt_rfc2047.h"
+#include "sbr/brkstring.h"
+#include "sbr/ambigsw.h"
+#include "sbr/error.h"
+#include "h/tws.h"
+#include "sbr/utils.h"
 
 #ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
@@ -64,7 +70,7 @@ DEFINE_SWITCH_ARRAY(PARSE, parswit);
 
 #define	STAR	01
 
-#define LBSIZE  1024
+#define LBSIZE  NMH_BUFSIZ
 #define	ESIZE	1024
 
 
@@ -115,10 +121,10 @@ static unsigned char cc[] = {
 #define	nxtarg()	(*argp ? *argp++ : NULL)
 #define	prvarg()	argp--
 
-#define	padvise		if (!talked++) advise
+#define	pinform		if (!talked++) inform
 
 struct nexus {
-    int (*n_action)();
+    int (*n_action)(struct nexus *n, FILE *fp, int msgnum, long start, long stop);
 
     union {
 	/* for {OR,AND,NOT}action */
@@ -157,7 +163,6 @@ struct nexus {
 #define	n_tws	 un.st3.un_tws
 
 static int talked;
-static int pdebug = 0;
 
 static char *datesw;
 static char **argp;
@@ -183,33 +188,34 @@ static struct nexus *parse(void);
 static struct nexus *nexp1(void);
 static struct nexus *nexp2(void);
 static struct nexus *nexp3(void);
-static struct nexus *newnexus(int (*)());
+static struct nexus *newnexus(int (*action)(struct nexus *n,
+    FILE *fp, int msgnum, long start, long stop));
 
-static int ORaction();
-static int ANDaction();
-static int NOTaction();
-static int GREPaction();
-static int TWSaction();
+static int ORaction(struct nexus *n, FILE *fp, int msgnum,
+    long start, long stop);
+static int ANDaction(struct nexus *n, FILE *fp, int msgnum,
+    long start, long stop);
+static int NOTaction(struct nexus *n, FILE *fp, int msgnum,
+    long start, long stop);
+static int GREPaction(struct nexus *n, FILE *fp, int msgnum,
+    long start, long stop);
+static int TWSaction(struct nexus *n, FILE *fp, int msgnum,
+    long start, long stop);
 
 
 int
 pcompile (char **vec, char *date)
 {
-    register char *cp;
-
-    if ((cp = getenv ("MHPDEBUG")) && *cp)
-	pdebug++;
-
     argp = vec;
     if ((datesw = date) == NULL)
 	datesw = "date";
     talked = 0;
 
     if ((head = parse ()) == NULL)
-	return (talked ? 0 : 1);
+        return !talked;
 
     if (*argp) {
-	padvise (NULL, "%s unexpected", *argp);
+	inform("%s unexpected", *argp);
 	return 0;
     }
 
@@ -220,14 +226,14 @@ pcompile (char **vec, char *date)
 static struct nexus *
 parse (void)
 {
-    register char  *cp;
-    register struct nexus *n, *o;
+    char  *cp;
+    struct nexus *n, *o;
 
     if ((n = nexp1 ()) == NULL || (cp = nxtarg ()) == NULL)
 	return n;
 
     if (*cp != '-') {
-	padvise (NULL, "%s unexpected", cp);
+	pinform("%s unexpected", cp);
 	return NULL;
     }
 
@@ -248,7 +254,8 @@ parse (void)
 	    o->n_L_child = n;
 	    if ((o->n_R_child = parse ()))
 		return o;
-	    padvise (NULL, "missing disjunctive");
+	    pinform("missing disjunctive");
+	    free (o);
 	    return NULL;
 
 header: ;
@@ -261,14 +268,15 @@ header: ;
 static struct nexus *
 nexp1 (void)
 {
-    register char *cp;
-    register struct nexus *n, *o;
+    char *cp;
+    struct nexus *n, *o;
 
     if ((n = nexp2 ()) == NULL || (cp = nxtarg ()) == NULL)
 	return n;
 
     if (*cp != '-') {
-	padvise (NULL, "%s unexpected", cp);
+	pinform("%s unexpected", cp);
+	free (n);
 	return NULL;
     }
 
@@ -278,10 +286,12 @@ nexp1 (void)
 	case AMBIGSW: 
 	    ambigsw (cp, parswit);
 	    talked++;
+	    free (n);
 	    return NULL;
 	case UNKWNSW: 
 	    fprintf (stderr, "-%s unknown\n", cp);
 	    talked++;
+	    free (n);
 	    return NULL;
 
 	case PRAND: 
@@ -289,7 +299,8 @@ nexp1 (void)
 	    o->n_L_child = n;
 	    if ((o->n_R_child = nexp1 ()))
 		return o;
-	    padvise (NULL, "missing conjunctive");
+	    pinform("missing conjunctive");
+	    free (o);
 	    return NULL;
 
 header: ;
@@ -303,8 +314,8 @@ header: ;
 static struct nexus *
 nexp2 (void)
 {
-    register char *cp;
-    register struct nexus *n;
+    char *cp;
+    struct nexus *n;
 
     if ((cp = nxtarg ()) == NULL)
 	return NULL;
@@ -330,7 +341,8 @@ nexp2 (void)
 	    n = newnexus (NOTaction);
 	    if ((n->n_L_child = nexp3 ()))
 		return n;
-	    padvise (NULL, "missing negation");
+	    pinform("missing negation");
+	    free (n);
 	    return NULL;
 
 header: ;
@@ -344,15 +356,15 @@ static struct nexus *
 nexp3 (void)
 {
     int i;
-    register char *cp, *dp;
+    char *cp, *dp;
     char buffer[BUFSIZ], temp[64];
-    register struct nexus *n;
+    struct nexus *n;
 
     if ((cp = nxtarg ()) == NULL)
 	return NULL;
 
     if (*cp != '-') {
-	padvise (NULL, "%s unexpected", cp);
+	pinform("%s unexpected", cp);
 	return NULL;
     }
 
@@ -372,16 +384,16 @@ nexp3 (void)
 
 	case PRLBR: 
 	    if ((n = parse ()) == NULL) {
-		padvise (NULL, "missing group");
+		pinform("missing group");
 		return NULL;
 	    }
 	    if ((cp = nxtarg ()) == NULL) {
-		padvise (NULL, "missing -rbrace");
+		pinform("missing -rbrace");
 		return NULL;
 	    }
 	    if (*cp++ == '-' && smatch (cp, parswit) == PRRBR)
 		return n;
-	    padvise (NULL, "%s unexpected", --cp);
+	    pinform("%s unexpected", --cp);
 	    return NULL;
 
 	default: 
@@ -398,7 +410,7 @@ nexp3 (void)
 	    dp = *brkstring (temp, " ", NULL);
     header: ;
 	    if (!(cp = nxtarg ())) {/* allow -xyz arguments */
-		padvise (NULL, "missing argument to %s", argp[-2]);
+		pinform("missing argument to %s", argp[-2]);
 		return NULL;
 	    }
 	    n = newnexus (GREPaction);
@@ -411,25 +423,27 @@ nexp3 (void)
 	    n = newnexus (GREPaction);
 	    n->n_header = 0;
 	    if (!(cp = nxtarg ())) {/* allow -xyz arguments */
-		padvise (NULL, "missing argument to %s", argp[-2]);
+		pinform("missing argument to %s", argp[-2]);
+		free (n);
 		return NULL;
 	    }
 	    dp = cp;
     pattern: ;
 	    if (!gcompile (n, dp)) {
-		padvise (NULL, "pattern error in %s %s", argp[-2], cp);
+		pinform("pattern error in %s %s", argp[-2], cp);
+		free (n);
 		return NULL;
 	    }
-	    n->n_patbuf = getcpy (dp);
+	    n->n_patbuf = mh_xstrdup(dp);
 	    return n;
 
 	case PROTHR: 
-	    padvise (NULL, "internal error!");
+	    pinform("internal error!");
 	    return NULL;
 
 	case PRDATF: 
 	    if (!(datesw = nxtarg ()) || *datesw == '-') {
-		padvise (NULL, "missing argument to %s", argp[-2]);
+		pinform("missing argument to %s", argp[-2]);
 		return NULL;
 	    }
 	    return nexp3 ();
@@ -437,13 +451,14 @@ nexp3 (void)
 	case PRAFTR: 
 	case PRBEFR: 
 	    if (!(cp = nxtarg ())) {/* allow -xyz arguments */
-		padvise (NULL, "missing argument to %s", argp[-2]);
+		pinform("missing argument to %s", argp[-2]);
 		return NULL;
 	    }
 	    n = newnexus (TWSaction);
 	    n->n_datef = datesw;
 	    if (!tcompile (cp, &n->n_tws, n->n_after = i == PRAFTR)) {
-		padvise (NULL, "unable to parse %s %s", argp[-2], cp);
+		pinform("unable to parse %s %s", argp[-2], cp);
+		free (n);
 		return NULL;
 	    }
 	    return n;
@@ -452,44 +467,34 @@ nexp3 (void)
 
 
 static struct nexus *
-newnexus (int (*action)())
+newnexus(int (*action)(struct nexus *n, FILE *fp, int msgnum,
+    long start, long stop))
 {
-    register struct nexus *p;
-
-    if ((p = (struct nexus *) calloc ((size_t) 1, sizeof *p)) == NULL)
-	adios (NULL, "unable to allocate component storage");
+    struct nexus *p;
 
+    NEW0(p);
     p->n_action = action;
     return p;
 }
 
 
-#define	args(a)	a, fp, msgnum, start, stop
-#define	params	args (n)
-#define	plist	\
-	    register struct nexus  *n; \
-	    register FILE *fp; \
-	    int	msgnum; \
-	    long    start, \
-		    stop;
-
 int
-pmatches (FILE *fp, int msgnum, long start, long stop)
+pmatches (FILE *fp, int msgnum, long start, long stop, int debug)
 {
     if (!head)
 	return 1;
 
-    if (!talked++ && pdebug)
+    if (!talked++ && debug)
 	PRaction (head, 0);
 
-    return (*head->n_action) (args (head));
+    return (*head->n_action)(head, fp, msgnum, start, stop);
 }
 
 
 static void
 PRaction (struct nexus *n, int level)
 {
-    register int i;
+    int i;
 
     for (i = 0; i < level; i++)
 	fprintf (stderr, "| ");
@@ -522,53 +527,51 @@ PRaction (struct nexus *n, int level)
 		dasctime (&n->n_tws, TW_NULL));
 	return;
     }
-    fprintf (stderr, "UNKNOWN(0x%x)\n",
-	     (unsigned int)(unsigned long) (*n->n_action));
+
+    generic_pointer gp;
+    gp.f = (void (*)(void))n->n_action;
+    fprintf(stderr, "UNKNOWN(%p)\n", gp.v);
 }
 
 
 static int
-ORaction (params)
-plist
+ORaction(struct nexus *n, FILE *fp, int msgnum, long start, long stop)
 {
-    if ((*n->n_L_child->n_action) (args (n->n_L_child)))
+    if ((*n->n_L_child->n_action)(n->n_L_child, fp, msgnum, start, stop))
 	return 1;
-    return (*n->n_R_child->n_action) (args (n->n_R_child));
+    return (*n->n_R_child->n_action)(n->n_R_child, fp, msgnum, start, stop);
 }
 
 
 static int
-ANDaction (params)
-plist
+ANDaction(struct nexus *n, FILE *fp, int msgnum, long start, long stop)
 {
-    if (!(*n->n_L_child->n_action) (args (n->n_L_child)))
+    if (!(*n->n_L_child->n_action)(n->n_L_child, fp, msgnum, start, stop))
 	return 0;
-    return (*n->n_R_child->n_action) (args (n->n_R_child));
+    return (*n->n_R_child->n_action)(n->n_R_child, fp, msgnum, start, stop);
 }
 
 
 static int
-NOTaction (params)
-plist
+NOTaction(struct nexus *n, FILE *fp, int msgnum, long start, long stop)
 {
-    return (!(*n->n_L_child->n_action) (args (n->n_L_child)));
+    return (!(*n->n_L_child->n_action)(n->n_L_child, fp, msgnum, start, stop));
 }
 
 
 static int
 gcompile (struct nexus *n, char *astr)
 {
-    register int c;
+    int c;
     int cclcnt;
-    register unsigned char *ep, *dp, *sp, *lastep = 0;
+    unsigned char *ep, *dp, *sp, *lastep = 0;
 
     dp = (ep = (unsigned char *) n->n_expbuf) + sizeof n->n_expbuf;
     sp = (unsigned char *) astr;
     if (*sp == '^') {
 	n->n_circf = 1;
 	sp++;
-    }
-    else
+    } else
 	n->n_circf = 0;
     for (;;) {
 	if (ep >= dp)
@@ -612,8 +615,8 @@ gcompile (struct nexus *n, char *astr)
 		do {
 		    if (c == '-' && *sp != '\0' && *sp != ']') {
 			for (c = ep[-1]+1; c < *sp; c++) {
-		    	    *ep++ = c;
-		    	    cclcnt++;
+			    *ep++ = c;
+			    cclcnt++;
 			    if (c == '\0' || ep >= dp)
 				goto cerror;
 			}
@@ -632,6 +635,7 @@ gcompile (struct nexus *n, char *astr)
 	    case '\\': 
 		if ((c = *sp++) == '\0')
 		    goto cerror;
+		/* FALLTHRU */
 	defchar: 
 	    default: 
 		*ep++ = CCHR;
@@ -645,24 +649,25 @@ cerror: ;
 
 
 static int
-GREPaction (params)
-plist
+GREPaction(struct nexus *n, FILE *fp, int msgnum, long start, long stop)
 {
-    int c, body, lf;
+    int c;
+    bool body;
+    bool lf;
     long pos = start;
-    register char *p1, *p2, *ebp, *cbp;
+    char *p1, *p2, *ebp, *cbp;
     char ibuf[BUFSIZ];
     NMH_UNUSED (msgnum);
 
     fseek (fp, start, SEEK_SET);
-    body = 0;
+    body = false;
     ebp = cbp = ibuf;
     for (;;) {
 	if (body && n->n_header)
 	    return 0;
 	p1 = linebuf;
 	p2 = cbp;
-	lf = 0;
+	lf = false;
 	for (;;) {
 	    if (p2 >= ebp) {
 		if (fgets (ibuf, sizeof ibuf, fp) == NULL
@@ -681,21 +686,18 @@ plist
 		    --p2;
 		    break;
 		}
-		else
-		    lf = 0;
+                lf = false;
 	    }
 	    if (c == '\n') {
 		if (body)
 		    break;
-		else {
-		    if (lf) {
-			body++;
-			break;
-		    }
-		    lf++;
-		    /* Unfold by skipping the newline. */
-		    c = 0;
-		}
+                if (lf) {
+                    body = true;
+                    break;
+                }
+                lf = true;
+                /* Unfold by skipping the newline. */
+                c = 0;
 	    }
 	    if (c && p1 < &linebuf[LBSIZE - 1])
 		*p1++ = c;
@@ -708,8 +710,9 @@ plist
 
 	/* Attempt to decode as a MIME header.	If it's the last header,
 	   body will be 1 and lf will be at least 1. */
-	if ((body == 0 || lf > 0)  &&
-	    decode_rfc2047 (linebuf, decoded_linebuf, sizeof decoded_linebuf)) {
+	if ((!body || lf)  &&
+	    decode_rfc2047 (linebuf, decoded_linebuf, sizeof decoded_linebuf,
+			    NULL) > 0) {
 	    p1 = decoded_linebuf;
 	}
 
@@ -740,7 +743,7 @@ plist
 static int
 advance (char *alp, char *aep)
 {
-    register unsigned char *lp, *ep, *curlp;
+    unsigned char *lp, *ep, *curlp;
 
     lp = (unsigned char *)alp;
     ep = (unsigned char *)aep;
@@ -803,12 +806,12 @@ advance (char *alp, char *aep)
 		do {
 		    lp--;
 		    if (advance ((char *) lp, (char *) ep))
-			return (1);
+			return 1;
 		} while (lp > curlp);
 		return 0;
 
 	    default: 
-		admonish (NULL, "advance() botch -- you lose big");
+		inform("advance() botch -- you lose big, continuing...");
 		return 0;
 	}
 }
@@ -817,31 +820,31 @@ advance (char *alp, char *aep)
 static int
 cclass (unsigned char *aset, int ac, int af)
 {
-    register unsigned int    n;
-    register unsigned char   c, *set;
+    unsigned int    n;
+    unsigned char   c, *set;
 
     set = aset;
     if ((c = ac) == 0)
-	return (0);
+	return 0;
 
     n = *set++;
     while (n--)
 	if (*set++ == c || set[-1] == cc[c])
-	    return (af);
+	    return af;
 
-    return (!af);
+    return !af;
 }
 
 
 static int
 tcompile (char *ap, struct tws *tb, int isafter)
 {
-    register struct tws *tw;
+    struct tws *tw;
 
     if ((tw = tws_parse (ap, isafter)) == NULL)
 	return 0;
 
-    twscopy (tb, tw);
+    *tb = *tw;
     return 1;
 }
 
@@ -850,7 +853,7 @@ static struct tws *
 tws_parse (char *ap, int isafter)
 {
     char buffer[BUFSIZ];
-    register struct tws *tw, *ts;
+    struct tws *tw, *ts;
 
     if ((tw = tws_special (ap)) != NULL) {
 	tw->tw_sec = tw->tw_min = isafter ? 59 : 0;
@@ -892,7 +895,7 @@ tws_special (char *ap)
 {
     int i;
     time_t clock;
-    register struct tws *tw;
+    struct tws *tw;
 
     time (&clock);
     if (!strcasecmp (ap, "today"))
@@ -914,12 +917,12 @@ tws_special (char *ap)
 	    return NULL;
 	if ((i -= tw->tw_wday) > 0)
 	    i -= 7;
-    }
-    else
+    } else {
 	if (*ap != '-')
 	    return NULL;
-	else			/* -ddd days ago */
-	    i = atoi (ap);	/* we should error check this */
+	/* -ddd days ago */
+	i = atoi (ap);	/* we should error check this */
+    }
 
     clock += (long) ((60 * 60 * 24) * i);
     return dlocaltime (&clock);
@@ -927,28 +930,27 @@ tws_special (char *ap)
 
 
 static int
-TWSaction (params)
-plist
+TWSaction(struct nexus *n, FILE *fp, int msgnum, long start, long stop)
 {
     int state;
-    register char *bp;
-    char buf[BUFSIZ], name[NAMESZ];
-    register struct tws *tw;
-    m_getfld_state_t gstate = 0;
+    char *bp;
+    char buf[NMH_BUFSIZ], name[NAMESZ];
+    struct tws *tw;
+    m_getfld_state_t gstate;
     NMH_UNUSED (stop);
 
     fseek (fp, start, SEEK_SET);
+    gstate = m_getfld_state_init(fp);
     for (bp = NULL;;) {
 	int bufsz = sizeof buf;
-	switch (state = m_getfld (&gstate, name, buf, &bufsz, fp)) {
+	switch (state = m_getfld2(&gstate, name, buf, &bufsz)) {
 	    case FLD: 
 	    case FLDPLUS: 
-		if (bp != NULL)
-		    free (bp), bp = NULL;
-		bp = add (buf, NULL);
+                free(bp);
+		bp = mh_xstrdup(buf);
 		while (state == FLDPLUS) {
 		    bufsz = sizeof buf;
-		    state = m_getfld (&gstate, name, buf, &bufsz, fp);
+		    state = m_getfld2(&gstate, name, buf, &bufsz);
 		    bp = add (buf, bp);
 		}
 		if (!strcasecmp (name, n->n_datef))
@@ -960,26 +962,24 @@ plist
 	    case LENERR: 
 	    case FMTERR: 
 		if (state == LENERR || state == FMTERR)
-		    advise (NULL, "format error in message %d", msgnum);
-		if (bp != NULL)
-		    free (bp);
+		    inform("format error in message %d", msgnum);
+                free(bp);
 		return 0;
 
 	    default: 
-		adios (NULL, "internal error -- you lose");
+		die("internal error -- you lose");
 	}
 	break;
     }
     m_getfld_state_destroy (&gstate);
 
     if ((tw = dparsetime (bp)) == NULL)
-	advise (NULL, "unable to parse %s field in message %d, matching...",
+	inform("unable to parse %s field in message %d, matching...",
 		n->n_datef, msgnum), state = 1;
     else
 	state = n->n_after ? (twsort (tw, &n->n_tws) > 0)
 	    : (twsort (tw, &n->n_tws) < 0);
 
-    if (bp != NULL)
-	free (bp);
+    free(bp);
     return state;
 }
