@@ -5,25 +5,45 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "sbr/pidwait.h"
+#include "sbr/dtime.h"
+#include "sbr/m_getfld.h"
+#include "sbr/getarguments.h"
+#include "sbr/concat.h"
+#include "sbr/smatch.h"
+#include "sbr/cpydata.h"
+#include "sbr/cpydgst.h"
+#include "sbr/uprf.h"
+#include "sbr/context_foil.h"
+#include "sbr/readconfig.h"
+#include "sbr/ambigsw.h"
+#include "sbr/pidstatus.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/utils.h"
+#include "sbr/arglist.h"
+#include "sbr/error.h"
 #include <fcntl.h>
-#include <h/signals.h>
-#include <h/addrsbr.h>
-#include <h/aliasbr.h>
-#include <h/dropsbr.h>
-#include <h/mime.h>
-#include <h/utils.h>
-#include <h/tws.h>
-#include <h/mts.h>
-#include "../sbr/m_mktemp.h"
-#include "../sbr/message_id.h"
+#include <signal.h>
+#include "sbr/signals.h"
+#include "sbr/addrsbr.h"
+#include "aliasbr.h"
+#include "dropsbr.h"
+#include "h/mime.h"
+#include "sbr/done.h"
+#include "h/tws.h"
+#include "sbr/mts.h"
+#include "sbr/m_mktemp.h"
+#include "sbr/message_id.h"
 
 #ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
 #endif
 #include <time.h>
 
-#include <mts/smtp/smtp.h>
+#include "mts/smtp/smtp.h"
+#include "sbr/globals.h"
 
 #ifndef CYRUS_SASL
 # define SASLminc(a) (a)
@@ -224,27 +244,27 @@ static short fccind = 0;	/* index into fccfold[] */
 static short outputlinelen = OUTPUTLINELEN;
 
 static int pfd = NOTOK;		/* fd to write annotation list to        */
-static int recipients = 0;	/* how many people will get a copy       */
+static bool recipients;		/* how many people will get a copy	 */
 static int unkadr = 0;		/* how many of those were unknown        */
 static int badadr = 0;		/* number of bad addrs                   */
-static int badmsg = 0;		/* message has bad semantics             */
-static int verbose = 0;		/* spell it out                          */
-static int format = 1;		/* format addresses                      */
-static int mime = 0;		/* use MIME-style encapsulations for Bcc */
-static int msgid = 0;		/* add msgid                             */
-static int debug = 0;		/* debugging post                        */
-static int watch = 0;		/* watch the delivery process            */
-static int whomsw = 0;		/* we are whom not post                  */
-static int checksw = 0;		/* whom -check                           */
+static bool badmsg;		/* message has bad semantics             */
+static bool verbose;		/* spell it out                          */
+static bool format = true;	/* format addresses                      */
+static bool mime;		/* use MIME-style encapsulations for Bcc */
+static bool msgid;		/* add msgid                             */
+static bool debug;		/* debugging post                        */
+static bool watch;		/* watch the delivery process            */
+static bool whomsw;		/* we are whom not post                  */
+static bool checksw;		/* whom -check                           */
 static int linepos=0;		/* putadr()'s position on the line       */
 static int nameoutput=0;	/* putadr() has output header name       */
-static int sasl=0;		/* Use SASL auth for SMTP                */
+static bool sasl;		/* Use SASL auth for SMTP                */
 static char *saslmech=NULL;	/* Force use of particular SASL mech     */
 static char *user=NULL;		/* Authenticate as this user             */
 static char *port="submission";	/* Name of server port for SMTP submission */
 static int tlsflag=0;		/* Flags to control TLS settings	 */
 static int fromcount=0;		/* Count of addresses on From: header	 */
-static int seensender=0;	/* Have we seen a Sender: header?	 */
+static bool seensender;		/* Have we seen a Sender: header?	 */
 
 static unsigned msgflags = 0;	/* what we've seen */
 
@@ -276,7 +296,7 @@ static struct mailname netaddrs;		/* network addrs   */
 static struct mailname uuaddrs;			/* uucp addrs      */
 static struct mailname tmpaddrs;		/* temporary queue */
 
-static int snoop      = 0;
+static bool snoop;
 static char *clientsw = NULL;
 static char *serversw = NULL;
 
@@ -290,7 +310,7 @@ static char *partno = NULL;
 static void putfmt (char *, char *, int *, FILE *);
 static void start_headers (void);
 static void finish_headers (FILE *);
-static int get_header (char *, struct headers *);
+static int get_header (char *, struct headers *) PURE;
 static int putadr (char *, char *, struct mailname *, FILE *, unsigned int,
 		   char *, unsigned int);
 static void putgrp (char *, char *, FILE *, unsigned int);
@@ -306,7 +326,7 @@ static void sigon (void);
 static void sigoff (void);
 static void p_refile (char *);
 static void fcc (char *, char *);
-static void die (char *, char *, ...);
+static void fatal (char *, char *, ...) CHECK_PRINTF(2, 3);
 static void post (char *, int, int, int, char *, int, char *);
 static void do_text (char *file, int fd);
 static void do_an_address (struct mailname *, int);
@@ -317,15 +337,17 @@ static int find_prefix (void);
 int
 main (int argc, char **argv)
 {
-    int state, compnum, dashstuff = 0, swnum, oauth_flag = 0, tls = -1;
-    int noverify = 0;
+    int state, compnum, dashstuff = 0, swnum;
+    bool oauth_flag = false;
+    int tls = -1;
+    bool noverify = false;
     int eai = 0; /* use Email Address Internationalization (EAI) (SMTPUTF8) */
     char *cp, *msg = NULL, **argp, **arguments, *envelope;
     char buf[NMH_BUFSIZ], name[NAMESZ], *auth_svc = NULL;
     FILE *in, *out;
-    m_getfld_state_t gstate = 0;
+    m_getfld_state_t gstate;
 
-    if (nmh_init(argv[0], 0 /* use context_foil() */)) { return 1; }
+    if (nmh_init(argv[0], false, false)) { return 1; }
 
     mts_init ();
     arguments = getarguments (invo_name, argc, argv, 0);
@@ -338,7 +360,7 @@ main (int argc, char **argv)
 		    ambigsw (cp, switches);
 		    done (1);
 		case UNKWNSW: 
-		    adios (NULL, "-%s unknown", cp);
+		    die("-%s unknown", cp);
 
 		case HELPSW: 
 		    snprintf (buf, sizeof(buf), "%s [switches] file", invo_name);
@@ -350,7 +372,7 @@ main (int argc, char **argv)
 
 		case LIBSW:
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    /* create a minimal context */
 		    if (context_foil (cp) == -1)
 			done (1);
@@ -358,21 +380,21 @@ main (int argc, char **argv)
 
 		case ALIASW: 
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    if ((state = alias (cp)) != AK_OK)
-			adios (NULL, "aliasing error in %s - %s",
+			die("aliasing error in %s - %s",
 				cp, akerror (state));
 		    continue;
 
 		case CHKSW: 
-		    checksw++;
+		    checksw = true;
 		    continue;
 		case NCHKSW: 
-		    checksw = 0;
+		    checksw = false;
 		    continue;
 
 		case DEBUGSW: 
-		    debug++;
+		    debug = true;
 		    continue;
 
 		case DISTSW:
@@ -381,18 +403,18 @@ main (int argc, char **argv)
 
 		case FILTSW:
 		    if (!(filter = *argp++) || *filter == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
-		    mime = 0;
+			die("missing argument to %s", argp[-2]);
+		    mime = false;
 		    continue;
 		case NFILTSW:
 		    filter = NULL;
 		    continue;
 		
 		case FRMTSW: 
-		    format++;
+		    format = true;
 		    continue;
 		case NFRMTSW: 
-		    format = 0;
+		    format = false;
 		    continue;
 
 		case BITSTUFFSW:
@@ -403,86 +425,86 @@ main (int argc, char **argv)
 		    continue;
 
 		case MIMESW:
-		    mime++;
+		    mime = true;
 		    filter = NULL;
 		    continue;
 		case NMIMESW: 
-		    mime = 0;
+		    mime = false;
 		    continue;
 
 		case MSGDSW: 
-		    msgid++;
+		    msgid = true;
 		    continue;
 		case NMSGDSW: 
-		    msgid = 0;
+		    msgid = false;
 		    continue;
 
 		case VERBSW: 
-		    verbose++;
+		    verbose = true;
 		    continue;
 		case NVERBSW: 
-		    verbose = 0;
+		    verbose = false;
 		    continue;
 
 		case WATCSW: 
-		    watch++;
+		    watch = true;
 		    continue;
 		case NWATCSW: 
-		    watch = 0;
+		    watch = false;
 		    continue;
 
 		case WHOMSW: 
-		    whomsw++;
+		    whomsw = true;
 		    continue;
 
 		case WIDTHSW: 
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    if ((outputlinelen = atoi (cp)) < 10)
-			adios (NULL, "impossible width %d", outputlinelen);
+			die("impossible width %d", outputlinelen);
 		    continue;
 
 		case ANNOSW: 
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    if ((pfd = atoi (cp)) <= 2)
-			adios (NULL, "bad argument %s %s", argp[-2], cp);
+			die("bad argument %s %s", argp[-2], cp);
 		    continue;
 
 		case CLIESW:
 		    if (!(clientsw = *argp++) || *clientsw == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 		case SERVSW:
 		    if (!(serversw = *argp++) || *serversw == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 		case SNOOPSW:
-		    snoop++;
+		    snoop = true;
 		    continue;
 
 		case PARTSW:
 		    if (!(partno = *argp++) || *partno == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 
 		case SASLSW:
-		    sasl++;
+		    sasl = true;
 		    continue;
 
 		case NOSASLSW:
-		    sasl = 0;
+		    sasl = false;
 		    continue;
 
 		case SASLMECHSW:
 		    if (!(saslmech = *argp++) || *saslmech == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 
 		case AUTHSERVICESW:
 		    if (!(auth_svc = *argp++) || *auth_svc == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
-		    oauth_flag++;
+			die("missing argument to %s", argp[-2]);
+		    oauth_flag = true;
 		    continue;
 
 		case OAUTHCREDFILESW:
@@ -496,7 +518,7 @@ main (int argc, char **argv)
 		    int i;
 
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 
 		    for (i = 0; oauthswitches[i].profname != NULL; i++) {
 			if (oauthswitches[i].switchnum == swnum) {
@@ -506,21 +528,21 @@ main (int argc, char **argv)
 		    }
 
 		    if (oauthswitches[i].profname == NULL)
-			adios (NULL, "internal error: cannot map switch %s "
+			die("internal error: cannot map switch %s "
 			       "to profile entry", argp[-2]);
 
-		    oauth_flag++;
+		    oauth_flag = true;
 		    continue;
 		}
 
 		case USERSW:
 		    if (!(user = *argp++) || *user == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 
 		case PORTSW:
 		    if (!(port = *argp++) || *port == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 
 		case TLSSW:
@@ -536,72 +558,71 @@ main (int argc, char **argv)
 		    continue;
 
 		case CERTVERSW:
-		    noverify = 0;
+		    noverify = false;
 		    continue;
 
 		case NOCERTVERSW:
-		    noverify++;
+		    noverify = true;
 		    continue;
 
 		case FILEPROCSW:
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    fileproc = cp;
 		    continue;
 
 		case MHLPROCSW:
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    mhlproc = cp;
 		    continue;
 
 		case MTSSM:
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
                     sendmail = cp;
 		    continue;
 
 		case MTSSW:
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
                     save_mts_method (cp);
 		    continue;
 
 		case CREDENTIALSSW: {
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    add_profile_entry ("credentials", cp);
 		    continue;
 		}
 
 		case MESSAGEIDSW:
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
                     if (save_message_id_style (cp) != 0)
-			adios (NULL, "unsupported messageid \"%s\"", cp);
+			die("unsupported messageid \"%s\"", cp);
 		    continue;
 	    }
 	}
 	if (msg)
-	    adios (NULL, "only one message at a time!");
-	else
-	    msg = cp;
+	    die("only one message at a time!");
+        msg = cp;
     }
 
     alias (AliasFile);
 
     if (!msg)
-	adios (NULL, "usage: %s [switches] file", invo_name);
+	die("usage: %s [switches] file", invo_name);
 
     if (outputlinelen < 10)
-	adios (NULL, "impossible width %d", outputlinelen);
+	die("impossible width %d", outputlinelen);
 
     if ((in = fopen (msg, "r")) == NULL)
 	adios (msg, "unable to open");
 
     start_headers ();
     if (debug) {
-	verbose++;
+	verbose = true;
 	out = stdout;
     } else {
 	if (whomsw) {
@@ -610,25 +631,26 @@ main (int argc, char **argv)
 	} else {
 	    char *cp = m_mktemp2(NULL, invo_name, NULL, &out);
 	    if (cp == NULL) {
-		adios(NULL, "unable to create temporary file in %s",
+		die("unable to create temporary file in %s",
 		      get_temp_dir());
 	    }
-            strncpy(tmpfil, cp, sizeof(tmpfil));
+            strncpy(tmpfil, cp, sizeof(tmpfil) - 1);
 	}
     }
 
     hdrtab = msgstate == NORMAL ? NHeaders : RHeaders;
 
+    gstate = m_getfld_state_init(in);
     for (compnum = 1;;) {
 	int bufsz = sizeof buf;
-	switch (state = m_getfld (&gstate, name, buf, &bufsz, in)) {
+	switch (state = m_getfld2(&gstate, name, buf, &bufsz)) {
 	    case FLD: 
 	    case FLDPLUS: 
                 compnum++;
 		cp = mh_xstrdup(buf);
 		while (state == FLDPLUS) {
 		    bufsz = sizeof buf;
-		    state = m_getfld (&gstate, name, buf, &bufsz, in);
+		    state = m_getfld2(&gstate, name, buf, &bufsz);
 		    cp = add (buf, cp);
 		}
 		putfmt (name, cp, &eai, out);
@@ -642,7 +664,7 @@ main (int argc, char **argv)
 		fprintf (out, "\n%s", buf);
 		while (state == BODY) {
 		    bufsz = sizeof buf;
-		    state = m_getfld (&gstate, name, buf, &bufsz, in);
+		    state = m_getfld2(&gstate, name, buf, &bufsz);
 		    fputs (buf, out);
 		}
 		break;
@@ -653,10 +675,10 @@ main (int argc, char **argv)
 
 	    case LENERR: 
 	    case FMTERR: 
-		adios (NULL, "message format error in component #%d", compnum);
+		die("message format error in component #%d", compnum);
 
 	    default: 
-		adios (NULL, "getfld() returned %d", state);
+		die("getfld() returned %d", state);
 	}
 	break;
     }
@@ -669,9 +691,8 @@ main (int argc, char **argv)
     if (debug) {
 	pl ();
 	done (0);
-    } else {
-	fclose (out);
     }
+    fclose (out);
 
     /*
      * Here's how we decide which address to use as the envelope-from
@@ -683,11 +704,11 @@ main (int argc, char **argv)
      */
 
     if (msgflags & MEFM) {
-    	envelope = efrom;
+	envelope = efrom;
     } else if (seensender) {
-    	envelope = sender;
+	envelope = sender;
     } else {
-    	envelope = from;
+	envelope = from;
     }
 
     if (tls == -1) {
@@ -727,7 +748,7 @@ main (int argc, char **argv)
 	char sbuf[128];
 
 	if (auth_svc == NULL) {
-	    adios(NULL, "No authentication service given with -authservice");
+	    die("No authentication service given with -authservice");
 	}
 
 	for (i = 0; oauthswitches[i].profname != NULL; i++) {
@@ -742,7 +763,12 @@ main (int argc, char **argv)
 
     /* If we are doing a "whom" check */
     if (whomsw) {
-	/* This won't work with MTS_SENDMAIL_PIPE. */
+        /* Note that this will call sendmail directly using SMTP, just like
+           sendmail/smtp, even if the sendmail/pipe mts is specified.
+           It doesn't use sendmail -bv, which spost(8) used to do, because:
+             Notice: -bv may give misleading output for non-privileged user
+           and because -bv works differently with postfix.
+           And see the BUGS section of the whom(1) man page re. -check. */
         verify_all_addresses (1, eai, envelope, oauth_flag, auth_svc);
 	done (0);
     }
@@ -751,10 +777,9 @@ main (int argc, char **argv)
 	make_bcc_file (dashstuff);
 	if (msgflags & MVIS) {
 	    if (sm_mts != MTS_SENDMAIL_PIPE) {
-		/* It would be nice to have support to call
-		   verify_all_addresses with MTS_SENDMAIL_PIPE, but
-		   that might require running sendmail as root.  Note
-		   that spost didn't verify addresses. */
+                /* Addresses aren't verified here with sendmail/pipe because
+                   spost didn't.  They could be because verify_all_address()
+                   uses SMTP, see above. */
 		verify_all_addresses (verbose, eai, envelope, oauth_flag,
                                       auth_svc);
 	    }
@@ -770,7 +795,7 @@ main (int argc, char **argv)
     (void) m_unlink (tmpfil);
 
     if (verbose) {
-    	if (partno)
+	if (partno)
 	    printf ("Partial Message #%s Processed\n", partno);
 	else
 	    puts("Message Processed");
@@ -788,7 +813,9 @@ main (int argc, char **argv)
 static void
 putfmt (char *name, char *str, int *eai, FILE *out)
 {
-    int count, grp, i, keep;
+    int count;
+    bool grp;
+    int i, keep;
     char *cp, *pp, *qp;
     char namep[BUFSIZ], error[BUFSIZ];
     char *savehdr = NULL;
@@ -801,7 +828,7 @@ putfmt (char *name, char *str, int *eai, FILE *out)
 
     if (msgstate == NORMAL && uprf (name, "resent")) {
 	inform("illegal header line -- %s:", name);
-	badmsg++;
+	badmsg = true;
 	return;
     }
 
@@ -845,9 +872,15 @@ putfmt (char *name, char *str, int *eai, FILE *out)
     }
     if (hdr->flags & HBAD) {
 	inform("illegal header line -- %s:", name);
-	badmsg++;
+	badmsg = true;
 	return;
     }
+    if (hdr->flags & HDCC && sm_mts == MTS_SENDMAIL_PIPE)
+    {
+       inform("Dcc header is not supported with sendmail/pipe");
+       badmsg = true;
+       return;
+    }
     msgflags |= (hdr->set & ~(MVIS | MINV));
 
     if (hdr->flags & HSUB)
@@ -893,18 +926,18 @@ putfmt (char *name, char *str, int *eai, FILE *out)
 
     for (count = 0; (cp = getname (str)); count++) {
 	if ((mp = getm (cp, NULL, 0, error, sizeof(error)))) {
-	    if (tmpaddrs.m_next)
+	    if (tmpaddrs.m_next) {
+		assert (np != NULL);
 		np->m_next = mp;
-	    else
+	    } else
 		tmpaddrs.m_next = mp;
 	    np = mp;
-	}
-	else {
+	} else {
 	    admonish(cp, "%s", error);
 	    if (hdr->flags & HTRY)
 		badadr++;
 	    else
-		badmsg++;
+		badmsg = true;
 	}
     }
 
@@ -918,11 +951,11 @@ putfmt (char *name, char *str, int *eai, FILE *out)
 	    if ((msgstate == RESENT) ? (hdr->set & MRSN)
 					: (hdr->set & MSND)) {
 		inform("%s: field requires one address", name);
-		badmsg++;
+		badmsg = true;
 	    }
 #ifdef notdef
 	    inform("%s: field requires at least one address", name);
-	    badmsg++;
+	    badmsg = true;
 #endif /* notdef */
 	}
 	return;
@@ -930,7 +963,7 @@ putfmt (char *name, char *str, int *eai, FILE *out)
 
     if (count > 1 && (hdr->flags & HONE)) {
 	inform("%s: field only permits one address", name);
-	badmsg++;
+	badmsg = true;
 	return;
     }
 
@@ -938,7 +971,7 @@ putfmt (char *name, char *str, int *eai, FILE *out)
     snprintf (namep, sizeof(namep), "%s%s",
 		(hdr->flags & HMNG) ? "Original-" : "", name);
 
-    for (grp = 0, mp = tmpaddrs.m_next; mp; mp = np)
+    for (grp = false, mp = tmpaddrs.m_next; mp; mp = np)
 	if (mp->m_nohost) {	/* also used to test (hdr->flags & HTRY) */
 	    /* The address doesn't include a host, so it might be an alias. */
 	    pp = akvalue (mp->m_mbox);  /* do mh alias substitution */
@@ -973,7 +1006,7 @@ putfmt (char *name, char *str, int *eai, FILE *out)
 						: (hdr->set & MSND)) {
 		    strncpy(sender, auxformat(mp, 0), sizeof(sender) - 1);
 		    sender[sizeof(sender) - 1] = '\0';
-		    seensender++;
+		    seensender = true;
 		}
 
 		/*
@@ -989,9 +1022,8 @@ putfmt (char *name, char *str, int *eai, FILE *out)
 		    mp->m_bcc++;
 		if (np->m_ingrp)
 		    mp->m_ingrp = np->m_ingrp;
-		else
-		    if (mp->m_gname)
-			putgrp (namep, mp->m_gname, out, hdr->flags);
+		else if (mp->m_gname)
+                    putgrp (namep, mp->m_gname, out, hdr->flags);
 		if (mp->m_ingrp) {
 		    if (sm_mts == MTS_SENDMAIL_PIPE) {
 			/* Catch this before sendmail chokes with:
@@ -1001,12 +1033,10 @@ putfmt (char *name, char *str, int *eai, FILE *out)
 			   aliases and put them in Bcc:, but then
 			   they'd have the Blind-Carbon-Copy
 			   indication. */
-			adios (NULL,
-			       "blind lists not compatible with"
-			       " sendmail/pipe");
+			die("blind lists not compatible with sendmail/pipe");
 		    }
 
-		    grp++;
+		    grp = true;
 		}
 		if (putadr (namep, qp, mp, out, hdr->flags, savehdr,
 			    savehdrlen))
@@ -1017,8 +1047,7 @@ putfmt (char *name, char *str, int *eai, FILE *out)
 	    mp = np;
 	    np = np->m_next;
 	    mnfree (mp);
-	}
-	else {
+	} else {
 	    /* Address includes a host, so no alias substitution is needed. */
 
 	    /*
@@ -1040,7 +1069,7 @@ putfmt (char *name, char *str, int *eai, FILE *out)
 					    : (hdr->set & MSND)) {
 	        strncpy(sender, auxformat(mp, 0), sizeof(sender) - 1);
 		sender[sizeof(sender) - 1] = '\0';
-		seensender++;
+		seensender = true;
 	    }
 
 	    /*
@@ -1057,14 +1086,13 @@ putfmt (char *name, char *str, int *eai, FILE *out)
 	    if (mp->m_gname)
 		putgrp (namep, mp->m_gname, out, hdr->flags);
 	    if (mp->m_ingrp)
-		grp++;
+		grp = true;
 	    keep = putadr (namep, "", mp, out, hdr->flags, savehdr, savehdrlen);
 	    np = mp->m_next;
 	    if (keep) {
 		mp->m_next = NULL;
 		msgflags |= (hdr->set & (MVIS | MINV));
-	    }
-	    else
+	    } else
 		mnfree (mp);
 	}
 
@@ -1079,9 +1107,9 @@ putfmt (char *name, char *str, int *eai, FILE *out)
 	}
     }
 
-    if (grp > 0 && (hdr->flags & HNGR)) {
+    if (grp && (hdr->flags & HNGR)) {
 	inform("%s: field does not allow groups", name);
-	badmsg++;
+	badmsg = true;
     }
     if (linepos) {
 	putc ('\n', out);
@@ -1122,14 +1150,14 @@ finish_headers (FILE *out)
 		 */
 		inform("message has no From: header");
 		inform("See default components files for examples");
-		badmsg++;
+		badmsg = true;
 		break;
 	    }
 
-	    if (fromcount > 1 && (seensender == 0 && !(msgflags & MEFM))) {
+	    if (fromcount > 1 && (!seensender && !(msgflags & MEFM))) {
 		inform("A Sender: or Envelope-From: header is required "
 			"with multiple\nFrom: addresses");
-		badmsg++;
+		badmsg = true;
 		break;
 	    }
 
@@ -1145,12 +1173,12 @@ finish_headers (FILE *out)
 	     * from Envelope-From: (which in this case, cannot be blank)
 	     */
 
-	    if (fromcount > 1 && seensender == 0) {
-	    	if (efrom[0] == '\0') {
+	    if (fromcount > 1 && !seensender) {
+		if (efrom[0] == '\0') {
 		    inform("Envelope-From cannot be blank when there "
-		    	    "is multiple From: addresses\nand no Sender: "
+			    "is multiple From: addresses\nand no Sender: "
 			    "header");
-		    badmsg++;
+		    badmsg = true;
 		} else {
 		    fprintf (out, "Sender: %s\n", efrom);
 		}
@@ -1163,22 +1191,22 @@ finish_headers (FILE *out)
 	case RESENT: 
 	    if (!(msgflags & MDAT)) {
 		inform("message has no Date: header");
-		badmsg++;
+		badmsg = true;
 	    }
 	    if (!(msgflags & MFRM)) {
 		inform("message has no From: header");
-		badmsg++;
+		badmsg = true;
 	    }
 	    if (!(msgflags & MRFM)) {
 		inform("message has no Resent-From: header");
 		inform("See default components files for examples");
-		badmsg++;
+		badmsg = true;
 		break;
 	    }
-	    if (fromcount > 1 && (seensender == 0 && !(msgflags & MEFM))) {
+	    if (fromcount > 1 && (!seensender && !(msgflags & MEFM))) {
 		inform("A Resent-Sender: or Envelope-From: header is "
 			"required with multiple\nResent-From: addresses");
-		badmsg++;
+		badmsg = true;
 		break;
 	    }
 
@@ -1195,12 +1223,12 @@ finish_headers (FILE *out)
 	     * from Envelope-From (which in this case, cannot be blank)
 	     */
 
-	    if (fromcount > 1 && seensender == 0) {
-	    	if (efrom[0] == '\0') {
+	    if (fromcount > 1 && !seensender) {
+		if (efrom[0] == '\0') {
 		    inform("Envelope-From cannot be blank when there "
-		    	    "is multiple Resent-From: addresses and no "
+			    "is multiple Resent-From: addresses and no "
 			    "Resent-Sender: header");
-		    badmsg++;
+		    badmsg = true;
 		} else {
 		    fprintf (out, "Resent-Sender: %s\n", efrom);
 		}
@@ -1212,9 +1240,9 @@ finish_headers (FILE *out)
     }
 
     if (badmsg)
-	adios (NULL, "re-format message and try again");
+	die("re-format message and try again");
     if (!recipients)
-	adios (NULL, "no addressees");
+	die("no addressees");
 }
 
 
@@ -1223,9 +1251,10 @@ get_header (char *header, struct headers *table)
 {
     struct headers *h;
 
+    header = FENDNULL(header);
     for (h = table; h->value; h++)
-	if (!strcasecmp (FENDNULL(header), FENDNULL(h->value)))
-	    return (h - table);
+	if (!strcasecmp(FENDNULL(h->value), header))
+	    return h - table;
 
     return NOTOK;
 }
@@ -1235,15 +1264,15 @@ static int
 putadr (char *name, char *aka, struct mailname *mp, FILE *out,
 	unsigned int flags, char *savehdr, unsigned int savehdrsize)
 {
-    int len, saveappend = 0;
+    int len;
+    bool saveappend = false;
     unsigned int shlen;
     char *cp;
     char buffer[BUFSIZ];
 
     if (mp->m_mbox == NULL || ((flags & HTRY) && !insert (mp)))
 	return 0;
-    if (sm_mts != MTS_SENDMAIL_PIPE &&
-        ((flags & (HBCC | HDCC | HEFM)) || mp->m_ingrp))
+    if ((flags & (HBCC | HDCC | HEFM)) || mp->m_ingrp)
 	return 1;
 
     if (!nameoutput) {
@@ -1253,7 +1282,7 @@ putadr (char *name, char *aka, struct mailname *mp, FILE *out,
 
     if (savehdr) {
 	shlen = strlen(savehdr);
-	saveappend = 1;
+	saveappend = true;
     }
 
     if (*aka && mp->m_type != UUCPHOST && !mp->m_pers)
@@ -1275,7 +1304,7 @@ putadr (char *name, char *aka, struct mailname *mp, FILE *out,
 	    fprintf (out, ",\n%*s", linepos = nameoutput, "");
 	    if (saveappend) {
 		if (shlen + 2 + nameoutput + len >= savehdrsize) {
-		    saveappend = 0;
+		    saveappend = false;
 		} else {
 		    snprintf(savehdr + shlen, savehdrsize - shlen, ",\n%*s",
 			     linepos, "");
@@ -1286,7 +1315,7 @@ putadr (char *name, char *aka, struct mailname *mp, FILE *out,
 	    linepos += 2;
 	    if (saveappend) {
 		if (shlen + 2 + len >= savehdrsize) {
-		    saveappend = 0;
+		    saveappend = false;
 		} else {
 		    strncat(savehdr, ", ", savehdrsize - shlen);
 		}
@@ -1301,7 +1330,7 @@ putadr (char *name, char *aka, struct mailname *mp, FILE *out,
 
     linepos += len;
 
-    return (flags & HTRY);
+    return flags & HTRY;
 }
 
 
@@ -1326,8 +1355,7 @@ putgrp (char *name, char *group, FILE *out, unsigned int flags)
 	if (len + linepos + 2 > outputlinelen) {
 	    fprintf (out, ",\n%*s", nameoutput, "");
 	    linepos = nameoutput;
-	}
-	else {
+	} else {
 	    fputs (", ", out);
 	    linepos += 2;
 	}
@@ -1359,7 +1387,7 @@ insert (struct mailname *np)
 	    return 0;
 
     mp->m_next = np;
-    recipients++;
+    recipients = true;
     return 1;
 }
 
@@ -1370,26 +1398,26 @@ pl (void)
     int i;
     struct mailname *mp;
 
-    printf ("-------\n\t-- Addresses --\nlocal:\t");
+    fputs("-------\n\t-- Addresses --\nlocal:\t", stdout);
     for (mp = localaddrs.m_next; mp; mp = mp->m_next)
 	printf ("%s%s%s", mp->m_mbox,
 		mp->m_bcc ? "[BCC]" : "",
 		mp->m_next ? ",\n\t" : "");
 
-    printf ("\nnet:\t");
+    fputs("\nnet:\t", stdout);
     for (mp = netaddrs.m_next; mp; mp = mp->m_next)
 	printf ("%s%s@%s%s%s", FENDNULL(mp->m_path),
 		mp->m_mbox, mp->m_host,
 		mp->m_bcc ? "[BCC]" : "",
 		mp->m_next ? ",\n\t" : "");
 
-    printf ("\nuucp:\t");
+    fputs("\nuucp:\t", stdout);
     for (mp = uuaddrs.m_next; mp; mp = mp->m_next)
 	printf ("%s!%s%s%s", mp->m_host, mp->m_mbox,
 		mp->m_bcc ? "[BCC]" : "",
 		mp->m_next ? ",\n\t" : "");
 
-    printf ("\n\t-- Folder Copies --\nfcc:\t");
+    fputs("\n\t-- Folder Copies --\nfcc:\t", stdout);
     for (i = 0; i < fccind; i++)
 	printf ("%s%s", fccfold[i], i + 1 < fccind ? ",\n\t" : "");
     putchar('\n');
@@ -1428,7 +1456,7 @@ annoaux (struct mailname *mp)
     snprintf (buffer, sizeof(buffer), "%s\n", adrformat (mp));
     i = strlen (buffer);
 
-    return (write (pfd, buffer, i) == i ? OK : NOTOK);
+    return write(pfd, buffer, i) == i ? OK : NOTOK;
 }
 
 
@@ -1447,7 +1475,7 @@ insert_fcc (struct headers *hdr, char *pp)
 	return;
 
     if (fccind >= FCCS)
-	adios (NULL, "too many %ss", hdr->value);
+	die("too many %ss", hdr->value);
     fccfold[fccind++] = mh_xstrdup(cp);
 }
 
@@ -1465,9 +1493,9 @@ make_bcc_file (int dashstuff)
     char *tfile = NULL, *program;
 
     if ((tfile = m_mktemp2(NULL, "bccs", NULL, &out)) == NULL) {
-	adios(NULL, "unable to create temporary file in %s", get_temp_dir());
+	die("unable to create temporary file in %s", get_temp_dir());
     }
-    strncpy (bccfil, tfile, sizeof(bccfil));
+    strncpy (bccfil, tfile, sizeof(bccfil) - 1);
 
     fprintf (out, "From: %s\n", fullfrom);
     fprintf (out, "Date: %s\n", dtime (&tclock, 0));
@@ -1475,7 +1503,31 @@ make_bcc_file (int dashstuff)
 	fprintf (out, "Message-ID: %s\n", message_id (tclock, 0));
     if (subject)
 	fprintf (out, "Subject: %s", subject);
-    fprintf (out, "BCC:\n");
+
+    /* for sendmail/pipe, insert all bcc recipients here so that the email can be routed based on the bcc: header */
+    if (sm_mts == MTS_SENDMAIL_PIPE)
+    {
+       char *allbcc = NULL;
+       struct mailname *lp;
+
+       for (lp = localaddrs.m_next; lp; lp = lp->m_next)
+	  if (lp->m_bcc)
+	     allbcc = allbcc? add(concat(", ", lp->m_mbox, NULL), allbcc)
+		: mh_xstrdup(lp->m_mbox);
+       for (lp = netaddrs.m_next; lp; lp = lp->m_next)
+	  if (lp->m_bcc)
+	     allbcc = allbcc? add(
+		concat(", ", lp->m_mbox, "@", lp->m_host, NULL),
+		allbcc)
+		: concat(lp->m_mbox, "@", lp->m_host, NULL);
+       if (allbcc)
+       {
+	  fprintf (out, "BCC: %s\n",allbcc);
+	  free(allbcc);
+       }
+    } else {
+       fprintf (out, "BCC:\n");
+    }
 
     /*
      * Use MIME encapsulation for Bcc messages
@@ -1490,15 +1542,14 @@ make_bcc_file (int dashstuff)
 	 * prefix and check again.
 	 */
 	if ((cp = strchr(prefix, 'a')) == NULL)
-	    adios (NULL, "lost prefix start");
+	    die("lost prefix start");
 	while (find_prefix () == NOTOK) {
 	    if (*cp < 'z')
 		(*cp)++;
-	    else
-		if (*++cp == 0)
-		    adios (NULL, "can't find a unique delimiter string");
-		else
-		    (*cp)++;
+	    else if (*++cp == 0)
+                die("can't find a unique delimiter string");
+            else
+                (*cp)++;
 	}
 
 	fprintf (out, "%s: %s\n%s: multipart/digest; boundary=\"",
@@ -1515,8 +1566,7 @@ make_bcc_file (int dashstuff)
      * of MIME encapsulation.
      */
     if (filter != NULL) {
-	for (i = 0; (child_id = fork()) == NOTOK && i < 5; i++)
-	    sleep (5);
+	child_id = fork();
 	switch (child_id) {
 	    case NOTOK: 
 		adios ("fork", "unable to");
@@ -1540,14 +1590,14 @@ make_bcc_file (int dashstuff)
 		execvp (program, vec);
 		fprintf (stderr, "unable to exec ");
 		perror (mhlproc);
-		_exit (-1);
+		_exit(1);
 
 	    default: 
 		pidXwait (child_id, mhlproc);
 		break;
 	}
     } else {
-	if ((fd = open (tmpfil, O_RDONLY)) == NOTOK)
+	if ((fd = open (tmpfil, O_RDONLY)) == -1)
 	    adios (tmpfil, "unable to re-open");
 
 	/*
@@ -1609,12 +1659,12 @@ static void
 chkadr (void)
 {
     if (badadr && unkadr)
-	die (NULL, "%d address%s unparsable, %d addressee%s undeliverable",
+	fatal (NULL, "%d address%s unparsable, %d addressee%s undeliverable",
 		badadr, PLURALS(badadr), unkadr, PLURALS(badadr));
     if (badadr)
-	die (NULL, "%d address%s unparsable", badadr, PLURALS(badadr));
+	fatal (NULL, "%d address%s unparsable", badadr, PLURALS(badadr));
     if (unkadr)
-	die (NULL, "%d addressee%s undeliverable", unkadr, PLURALS(unkadr));
+	fatal (NULL, "%d addressee%s undeliverable", unkadr, PLURALS(unkadr));
 }
 
 
@@ -1655,7 +1705,7 @@ do_addresses (int bccque, int talk)
     chkadr ();
 
     if (rp_isbad (retval = sm_waend ()))
-	die (NULL, "problem ending addresses; %s", rp_string (retval));
+	fatal (NULL, "problem ending addresses; %s", rp_string (retval));
 }
 
 
@@ -1672,7 +1722,7 @@ static void
 post (char *file, int bccque, int talk, int eai, char *envelope,
       int oauth_flag, char *auth_svc)
 {
-    int	retval, i;
+    int retval;
     pid_t child_id;
 
     if (verbose) {
@@ -1689,8 +1739,7 @@ post (char *file, int bccque, int talk, int eai, char *envelope,
 	char **argp, *program;
 	int argc;
 
-	for (i = 0; (child_id = fork()) == NOTOK && i < 5; i++)
-	    sleep (5);
+	child_id = fork();
 	switch (child_id) {
 	    case NOTOK: 
 		adios ("fork", "unable to");
@@ -1718,12 +1767,11 @@ post (char *file, int bccque, int talk, int eai, char *envelope,
 	}
     } else {
         const int fd = open (file, O_RDONLY);
-        int eightbit = 0;
-
-        if (fd == NOTOK) {
-          die (file, "unable to re-open");
+        if (fd == -1) {
+          fatal (file, "unable to re-open");
         }
 
+        bool eightbit;
         if (msgflags & MMIM  &&  cte != UNKNOWN) {
             /* MIME message with C-T-E header.  (BINARYMIME isn't
                supported, use 8BITMIME instead for binary.) */
@@ -1731,7 +1779,7 @@ post (char *file, int bccque, int talk, int eai, char *envelope,
         } else {
             if (scan_input (fd, &eightbit) == NOTOK) {
                 close (fd);
-                die (file, "problem reading from");
+                fatal (file, "problem reading from");
             }
         }
 
@@ -1740,7 +1788,7 @@ post (char *file, int bccque, int talk, int eai, char *envelope,
 					oauth_flag ? auth_svc : NULL, tlsflag))
 		|| rp_isbad (retval = sm_winit (envelope, eai, eightbit))) {
 	    close (fd);
-	    die (NULL, "problem initializing server; %s", rp_string (retval));
+	    fatal (NULL, "problem initializing server; %s", rp_string (retval));
 	}
 
         do_addresses (bccque, talk && verbose);
@@ -1783,7 +1831,7 @@ verify_all_addresses (int talk, int eai, char *envelope, int oauth_flag,
 					verbose, snoop, sasl, saslmech, user,
 					oauth_flag ? auth_svc : NULL, tlsflag))
 		|| rp_isbad (retval = sm_winit (envelope, eai, eightbit))) {
-	    die (NULL, "problem initializing server; %s", rp_string (retval));
+	    fatal (NULL, "problem initializing server; %s", rp_string (retval));
 	}
     }
 
@@ -1827,7 +1875,7 @@ do_an_address (struct mailname *lp, int talk)
 	case LOCALHOST: 
 	    mbox = lp->m_mbox;
 	    host = lp->m_host;
-	    strncpy (addr, mbox, sizeof(addr));
+	    strncpy (addr, mbox, sizeof(addr) - 1);
 	    break;
 
 	case UUCPHOST: 
@@ -1836,7 +1884,7 @@ do_an_address (struct mailname *lp, int talk)
 	    snprintf (addr, sizeof(addr), "%s!%s", lp->m_host, lp->m_mbox);
 	    break;
 
-	default: 		/* let SendMail decide if the host is bad  */
+	default:		/* let SendMail decide if the host is bad  */
 	    mbox = lp->m_mbox;
 	    host = lp->m_host;
 	    snprintf (addr, sizeof(addr), "%s at %s", mbox, host);
@@ -1851,7 +1899,7 @@ do_an_address (struct mailname *lp, int talk)
 	return;
     }
     if (talk)
-	printf (": ");
+	fputs(": ", stdout);
     fflush (stdout);
 
     switch (retval = sm_wadr (mbox, host,
@@ -1873,7 +1921,7 @@ do_an_address (struct mailname *lp, int talk)
 	default: 
 	    if (!talk)
 		fprintf (stderr, "  %s: ", addr);
-	    die (NULL, "unexpected response; %s", rp_string (retval));
+	    fatal (NULL, "unexpected response; %s", rp_string (retval));
     }
 
     fflush (stdout);
@@ -1890,11 +1938,11 @@ do_text (char *file, int fd)
 
     while ((state = read (fd, buf, sizeof(buf))) > 0) {
 	if (rp_isbad (retval = sm_wtxt (buf, state)))
-	    die (NULL, "problem writing text; %s\n", rp_string (retval));
+	    fatal (NULL, "problem writing text; %s\n", rp_string (retval));
     }
 
-    if (state == NOTOK)
-	die (file, "problem reading from");
+    if (state == -1)
+	fatal (file, "problem reading from");
 
     switch (retval = sm_wtend ()) {
 	case RP_OK: 
@@ -1902,11 +1950,11 @@ do_text (char *file, int fd)
 
 	case RP_NO: 
 	case RP_NDEL: 
-	    die (NULL, "posting failed; %s", rp_string (retval));
+	    fatal (NULL, "posting failed; %s", rp_string (retval));
 	    break;
 
 	default: 
-	    die (NULL, "unexpected response; %s", rp_string (retval));
+	    fatal (NULL, "unexpected response; %s", rp_string (retval));
     }
 }
 
@@ -1985,7 +2033,7 @@ static void
 fcc (char *file, char *folder)
 {
     pid_t child_id;
-    int i, status, argp;
+    int status, argp;
     char fold[BUFSIZ];
     char **arglist, *program;
 
@@ -1993,9 +2041,7 @@ fcc (char *file, char *folder)
 	printf ("  %sFcc %s: ", msgstate == RESENT ? "Resent-" : "", folder);
     fflush (stdout);
 
-    for (i = 0; (child_id = fork ()) == NOTOK && i < 5; i++)
-	sleep (5);
-
+    child_id = fork();
     switch (child_id) {
 	case NOTOK: 
 	    if (!verbose)
@@ -2018,7 +2064,7 @@ fcc (char *file, char *folder)
 	    arglist[argp++] = fold;
 	    arglist[argp] = NULL;
 	    execvp (program, arglist);
-	    _exit (-1);
+	    _exit(1);
 
 	default: 
 	    if ((status = pidwait (child_id, OK))) {
@@ -2040,7 +2086,7 @@ fcc (char *file, char *folder)
  */
 
 static void
-die (char *what, char *fmt, ...)
+fatal (char *what, char *fmt, ...)
 {
     int err;
     va_list ap;
