@@ -5,14 +5,28 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "sbr/m_gmprot.h"
+#include "sbr/m_getfld.h"
+#include "sbr/getarguments.h"
+#include "sbr/smatch.h"
+#include "sbr/cpydata.h"
+#include "sbr/m_atoi.h"
+#include "sbr/context_save.h"
+#include "sbr/ambigsw.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
 #include <fcntl.h>
-#include <h/signals.h>
-#include <h/utils.h>
-#include "../sbr/m_mktemp.h"
+#include <signal.h>
+#include "sbr/signals.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
+#include "sbr/m_mktemp.h"
 #include <setjmp.h>
 
 #include <termios.h>
+#include "sbr/globals.h"
 
 #define	QUOTE '\\'
 
@@ -40,8 +54,8 @@ DEFINE_SWITCH_ARRAY(PROMPTER, switches);
 
 static struct termios tio;
 
-static int wtuser = 0;
-static int sigint = 0;
+static bool wtuser;
+static bool sigint;
 static jmp_buf sigenv;
 
 /*
@@ -56,17 +70,20 @@ static void intrser (int);
 int
 main (int argc, char **argv)
 {
-    int body = 1, prepend = 1, rapid = 0;
-    int doteof = 0, fdi, fdo, i, state;
+    bool body = true;
+    bool prepend = true;
+    bool rapid = false;
+    bool doteof = false;
+    int fdi, fdo, i, state;
     char *cp, *drft = NULL, *erasep = NULL;
     char *killp = NULL, name[NAMESZ], field[NMH_BUFSIZ];
     char buffer[BUFSIZ];
     char **arguments, **argp;
     FILE *in, *out;
     char *tmpfil;
-    m_getfld_state_t gstate = 0;
+    m_getfld_state_t gstate;
 
-    if (nmh_init(argv[0], 2)) { return 1; }
+    if (nmh_init(argv[0], true, false)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -78,7 +95,7 @@ main (int argc, char **argv)
 		    ambigsw (cp, switches);
 		    done (1);
 		case UNKWNSW: 
-		    adios (NULL, "-%s unknown", cp);
+		    die("-%s unknown", cp);
 
 		case HELPSW: 
 		    snprintf (buffer, sizeof(buffer), "%s [switches] file",
@@ -91,39 +108,39 @@ main (int argc, char **argv)
 
 		case ERASESW: 
 		    if (!(erasep = *argp++) || *erasep == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 		case KILLSW: 
 		    if (!(killp = *argp++) || *killp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 
 		case PREPSW: 
-		    prepend++;
+		    prepend = true;
 		    continue;
 		case NPREPSW: 
-		    prepend = 0;
+		    prepend = false;
 		    continue;
 
 		case RAPDSW: 
-		    rapid++;
+		    rapid = true;
 		    continue;
 		case NRAPDSW: 
-		    rapid = 0;
+		    rapid = false;
 		    continue;
 
 		case BODYSW: 
-		    body++;
+		    body = true;
 		    continue;
 		case NBODYSW: 
-		    body = 0;
+		    body = false;
 		    continue;
 
 		case DOTSW: 
-		    doteof++;
+		    doteof = true;
 		    continue;
 		case NDOTSW: 
-		    doteof = 0;
+		    doteof = false;
 		    continue;
 	    }
 	} else {
@@ -132,12 +149,12 @@ main (int argc, char **argv)
 	}
 
     if (!drft)
-	adios (NULL, "usage: %s [switches] file", invo_name);
+	die("usage: %s [switches] file", invo_name);
     if ((in = fopen (drft, "r")) == NULL)
 	adios (drft, "unable to open");
 
     if ((tmpfil = m_mktemp2(NULL, invo_name, NULL, &out)) == NULL) {
-	adios(NULL, "unable to create temporary file in %s", get_temp_dir());
+	die("unable to create temporary file in %s", get_temp_dir());
     }
 
     /*
@@ -176,15 +193,16 @@ main (int argc, char **argv)
 	tio.c_cc[VERASE] = save_erase;
     }
 
-    sigint = 0;
+    sigint = false;
     SIGNAL2 (SIGINT, intrser);
 
     /*
      * Loop through the lines of the draft skeleton.
      */
+    gstate = m_getfld_state_init(in);
     for (;;) {
 	int fieldsz = sizeof field;
-	switch (state = m_getfld (&gstate, name, field, &fieldsz, in)) {
+	switch (state = m_getfld2(&gstate, name, field, &fieldsz)) {
 	    case FLD: 
 	    case FLDPLUS: 
 		/*
@@ -201,7 +219,7 @@ main (int argc, char **argv)
 		    fprintf (out, "%s:%s", name, field);
 		    while (state == FLDPLUS) {
 			fieldsz = sizeof field;
-			state = m_getfld (&gstate, name, field, &fieldsz, in);
+			state = m_getfld2(&gstate, name, field, &fieldsz);
 			fputs(field, stdout);
 			fputs(field, out);
 		    }
@@ -242,7 +260,7 @@ abort:
 		    puts("--------");
 		if (field[0]) {
 		    if (prepend && body) {
-			printf ("\n--------Enter initial text\n\n");
+			puts("\n--------Enter initial text\n");
 			fflush (stdout);
 			for (;;) {
 			    getln (buffer, sizeof(buffer));
@@ -260,11 +278,10 @@ abort:
 			    fputs(field, stdout);
 		    } while (state == BODY &&
 			    (fieldsz = sizeof field,
-			     state = m_getfld (&gstate, name, field, &fieldsz, in)));
+			     state = m_getfld2(&gstate, name, field, &fieldsz)));
 		    if (prepend || !body)
 			break;
-		    else
-			printf ("\n--------Enter additional text\n\n");
+                    puts("\n--------Enter additional text\n");
 		}
 
 		fflush (stdout);
@@ -279,7 +296,7 @@ abort:
 		break;
 
 	    default: 
-		adios (NULL, "skeleton is poorly formatted");
+		die("skeleton is poorly formatted");
 	}
 	break;
     }
@@ -297,9 +314,9 @@ abort:
 	 tcsetattr(0, TCSADRAIN, &tio);
     }
 
-    if ((fdi = open (tmpfil, O_RDONLY)) == NOTOK)
+    if ((fdi = open (tmpfil, O_RDONLY)) == -1)
 	adios (tmpfil, "unable to re-open");
-    if ((fdo = creat (drft, m_gmprot ())) == NOTOK)
+    if ((fdo = creat (drft, m_gmprot ())) == -1)
 	adios (drft, "unable to write");
     cpydata (fdi, fdo, tmpfil, drft);
     close (fdi);
@@ -317,21 +334,21 @@ getln (char *buffer, int n)
 {
     int c;
     char *cp;
-    static int quoting = 0;
+    static bool quoting;
 
     *buffer = 0;
 
     switch (setjmp (sigenv)) {
 	case OK: 
-	    wtuser = 1;
+	    wtuser = true;
 	    break;
 
 	case DONE: 
-	    wtuser = 0;
+	    wtuser = false;
 	    return 0;
 
 	default: 
-	    wtuser = 0;
+	    wtuser = false;
 	    return NOTOK;
     }
 
@@ -341,27 +358,27 @@ getln (char *buffer, int n)
     for (;;) {
 	switch (c = getchar ()) {
 	    case EOF: 
-		quoting = 0;
+		quoting = false;
 		clearerr (stdin);
 		longjmp (sigenv, DONE);
 
 	    case '\n': 
 		if (quoting) {
 		    *(cp - 1) = c;
-		    quoting = 0;
-		    wtuser = 0;
+		    quoting = false;
+		    wtuser = false;
 		    return 1;
 		}
 		*cp++ = c;
 		*cp = 0;
-		wtuser = 0;
+		wtuser = false;
 		return 0;
 
 	    default: 
 		if (c == QUOTE) {
-		    quoting = 1;
+		    quoting = true;
 		} else {
-		    quoting = 0;
+		    quoting = false;
 		}
 		if (cp < buffer + n)
 		    *cp++ = c;
@@ -378,14 +395,14 @@ intrser (int i)
 
     if (wtuser)
 	longjmp (sigenv, NOTOK);
-    sigint++;
+    sigint = true;
 }
 
 
 static int
 chrcnv (char *cp)
 {
-    return (*cp != QUOTE ? *cp : m_atoi (++cp));
+    return *cp != QUOTE ? *cp : m_atoi(++cp);
 }
 
 
