@@ -5,13 +5,31 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/fmt_scan.h>
-#include <h/rcvmail.h>
-#include <h/tws.h>
-#include <h/mts.h>
-#include <h/utils.h>
-#include "../sbr/m_mktemp.h"
+#include "h/mh.h"
+#include "sbr/pidwait.h"
+#include "sbr/charstring.h"
+#include "sbr/fmt_new.h"
+#include "distsbr.h"
+#include "sbr/m_gmprot.h"
+#include "sbr/m_getfld.h"
+#include "sbr/getarguments.h"
+#include "sbr/smatch.h"
+#include "sbr/cpydata.h"
+#include "sbr/context_find.h"
+#include "sbr/ambigsw.h"
+#include "sbr/pidstatus.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/utils.h"
+#include "sbr/arglist.h"
+#include "sbr/error.h"
+#include "sbr/fmt_compile.h"
+#include "sbr/fmt_scan.h"
+#include "h/tws.h"
+#include "sbr/mts.h"
+#include "sbr/done.h"
+#include "sbr/m_mktemp.h"
+#include "sbr/globals.h"
 
 #define RCVDIST_SWITCHES \
     X("form formfile", 4, FORMSW) \
@@ -89,15 +107,15 @@ int
 main (int argc, char **argv)
 {
     pid_t child_id;
-    int i, vecp;
+    int vecp;
     char *addrs = NULL, *cp, *form = NULL, buf[BUFSIZ], *program;
     char **argp, **arguments, **vec;
     FILE *fp;
     char *tfile = NULL;
 
-    if (nmh_init(argv[0], 2)) { return 1; }
+    if (nmh_init(argv[0], true, false)) { return 1; }
 
-    done=unlink_done;
+    set_done(unlink_done);
 
     /*
      * Configure this now, since any unknown switches to rcvdist get
@@ -122,7 +140,7 @@ main (int argc, char **argv)
 		    if (argno != AMBIGSW && argno != UNKWNSW) {
 			/* It's a post switch that does take an argument. */
 			if (!(cp = *argp) || *cp == '-') {
-			    adios(NULL, "missing argument to %s", argp[-1]);
+			    die("missing argument to %s", argp[-1]);
 			}
 			vec[vecp++] = cp;
 			++argp;
@@ -141,7 +159,7 @@ main (int argc, char **argv)
 
 		case FORMSW: 
 		    if (!(form = *argp++) || *form == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 	    }
 	}
@@ -149,23 +167,23 @@ main (int argc, char **argv)
     }
 
     if (addrs == NULL)
-	adios (NULL, "usage: %s [switches] [switches for postproc] address ...",
+	die("usage: %s [switches] [switches for postproc] address ...",
 	    invo_name);
 
     umask (~m_gmprot ());
 
     if ((tfile = m_mktemp2(NULL, invo_name, NULL, &fp)) == NULL) {
-	adios(NULL, "unable to create temporary file in %s", get_temp_dir());
+	die("unable to create temporary file in %s", get_temp_dir());
     }
-    strncpy (tmpfil, tfile, sizeof(tmpfil));
+    strncpy (tmpfil, tfile, sizeof(tmpfil) - 1);
 
     cpydata (fileno (stdin), fileno (fp), "message", tmpfil);
     fseek (fp, 0L, SEEK_SET);
 
     if ((tfile = m_mktemp2(NULL, invo_name, NULL, NULL)) == NULL) {
-	adios(NULL, "unable to create temporary file in %s", get_temp_dir());
+	die("unable to create temporary file in %s", get_temp_dir());
     }
-    strncpy (drft, tfile, sizeof(tmpfil));
+    strncpy (drft, tfile, sizeof(drft) - 1);
 
     rcvdistout (fp, form, addrs);
     fclose (fp);
@@ -181,12 +199,11 @@ main (int argc, char **argv)
     vec[vecp++] = drft;
     vec[vecp] = NULL;
 
-    for (i = 0; (child_id = fork()) == NOTOK && i < 5; i++)
-	sleep (5);
+    child_id = fork();
     switch (child_id) {
 	case NOTOK: 
-	    inform("unable to fork, continuing...");
-	    /* FALLTHRU */
+            adios("fork", "failed:");
+
 	case OK: 
 	    execvp (program, vec);
 	    fprintf (stderr, "unable to exec ");
@@ -234,7 +251,7 @@ rcvdistout (FILE *inb, char *form, char *addrs)
     charstring_t scanl;
     struct comp *cptr;
     FILE *out;
-    m_getfld_state_t gstate = 0;
+    m_getfld_state_t gstate;
 
     if (!(out = fopen (drft, "w")))
 	adios (drft, "unable to create");
@@ -254,17 +271,18 @@ rcvdistout (FILE *inb, char *form, char *addrs)
     if (cptr)
 	cptr->c_text = addrs;
 
+    gstate = m_getfld_state_init(inb);
     for (;;) {
 	int msg_count = sizeof tmpbuf;
-	switch (state = m_getfld (&gstate, name, tmpbuf, &msg_count, inb)) {
+	switch (state = m_getfld2(&gstate, name, tmpbuf, &msg_count)) {
 	    case FLD: 
 	    case FLDPLUS: 
-	    	i = fmt_addcomptext(name, tmpbuf);
+		i = fmt_addcomptext(name, tmpbuf);
 		if (i != -1) {
 		    char_read += msg_count;
 		    while (state == FLDPLUS) {
 			msg_count = sizeof tmpbuf;
-			state = m_getfld (&gstate, name, tmpbuf, &msg_count, inb);
+			state = m_getfld2(&gstate, name, tmpbuf, &msg_count);
 			fmt_appendcomp(i, name, tmpbuf);
 			char_read += msg_count;
 		    }
@@ -272,7 +290,7 @@ rcvdistout (FILE *inb, char *form, char *addrs)
 
 		while (state == FLDPLUS) {
 		    msg_count = sizeof tmpbuf;
-		    state = m_getfld (&gstate, name, tmpbuf, &msg_count, inb);
+		    state = m_getfld2(&gstate, name, tmpbuf, &msg_count);
 		}
 		break;
 
@@ -283,7 +301,7 @@ rcvdistout (FILE *inb, char *form, char *addrs)
 		goto finished;
 
 	    default: 
-		adios (NULL, "m_getfld() returned %d", state);
+		die("m_getfld2() returned %d", state);
 	}
     }
 finished: ;
@@ -315,5 +333,5 @@ unlink_done (int status)
     if (tmpfil[0])
 	(void) m_unlink (tmpfil);
 
-    exit (status ? RCV_MBX : RCV_MOK);
+    exit(status ? 1 : 0);
 }
