@@ -1,6 +1,4 @@
-
-/*
- * rcvtty.c -- a rcvmail program (a lot like rcvalert) handling IPC ttys
+/* rcvtty.c -- a rcvmail program (a lot like rcvalert) handling IPC ttys
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -12,13 +10,28 @@
  * Ruud de Rooij <ruud@ruud.org>  Sun, 28 May 2000 17:28:55 +0200
  */
 
-#include <h/mh.h>
-#include <h/signals.h>
+#include "h/mh.h"
+#include "sbr/pidwait.h"
+#include "sbr/charstring.h"
+#include "sbr/fmt_new.h"
+#include "scansbr.h"
+#include "sbr/getarguments.h"
+#include "sbr/smatch.h"
+#include "sbr/ambigsw.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
+#include <signal.h>
+#include "sbr/signals.h"
 #include <setjmp.h>
-#include <h/rcvmail.h>
-#include <h/scansbr.h>
-#include <h/tws.h>
-#include <h/mts.h>
+#include "sbr/fmt_compile.h"
+#include "sbr/fmt_scan.h"
+#include "h/tws.h"
+#include "sbr/mts.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
+#include "sbr/m_mktemp.h"
+#include "sbr/globals.h"
 #include <fcntl.h>
 
 #ifdef HAVE_GETUTXENT
@@ -51,18 +64,13 @@ DEFINE_SWITCH_ARRAY(RCVTTY, switches);
 #undef X
 
 static jmp_buf myctx;
-static int bell = 1;
-static int newline = 1;
-static int biff = 0;
-static int width = 0;
+static bool bell = true;
+static bool newline = true;
+static bool biff;
+static int width = -1;
 static char *form = NULL;
 static char *format = NULL;
 
-/*
- * external prototypes
- */
-char *getusername(void);
-
 /*
  * static prototypes
  */
@@ -78,27 +86,27 @@ int
 main (int argc, char **argv)
 {
     int md, vecp = 0;
-    char *cp, *user, buf[BUFSIZ], tty[BUFSIZ];
+    char *cp, *user, buf[BUFSIZ];
     char **argp, **arguments, *vec[MAXARGS];
     struct utmpx *utp;
 
-    if (nmh_init(argv[0], 1)) { return 1; }
+    if (nmh_init(argv[0], true, false)) { return 1; }
 
-    mts_init (invo_name);
+    mts_init ();
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
-		case AMBIGSW: 
+		case AMBIGSW:
 		    ambigsw (cp, switches);
 		    done (1);
-		case UNKWNSW: 
+		case UNKWNSW:
 		    vec[vecp++] = --cp;
 		    continue;
 
-		case HELPSW: 
+		case HELPSW:
 		    snprintf (buf, sizeof(buf), "%s [command ...]", invo_name);
 		    print_help (buf, switches, 1);
 		    done (0);
@@ -107,36 +115,36 @@ main (int argc, char **argv)
 		    done (0);
 
 		case BIFFSW:
-		    biff = 1;
+		    biff = true;
 		    continue;
 
-		case FORMSW: 
+		case FORMSW:
 		    if (!(form = *argp++) || *form == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    format = NULL;
 		    continue;
-		case FMTSW: 
+		case FMTSW:
 		    if (!(format = *argp++) || *format == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    form = NULL;
 		    continue;
 
 		case WIDTHSW:
 		    if (!(cp = *argp++) || *cp == '-')
-			adios(NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    width = atoi(cp);
 		    continue;
                 case NLSW:
-                    newline = 1;
+                    newline = true;
                     continue;
                 case NNLSW:
-                    newline = 0;
+                    newline = false;
                     continue;
                 case BELSW:
-                    bell = 1;
+                    bell = true;
                     continue;
                 case NBELSW:
-                    bell = 0;
+                    bell = false;
                     continue;
 
 	    }
@@ -146,27 +154,25 @@ main (int argc, char **argv)
     vec[vecp] = 0;
 
     if ((md = vecp ? message_fd (vec) : header_fd ()) == NOTOK)
-	exit (RCV_MBX);
-
-    user = getusername();
+	exit(1);
 
 #if HAVE_GETUTXENT
+    user = getusername(1);
+
     setutxent();
     while ((utp = getutxent()) != NULL) {
         if (utp->ut_type == USER_PROCESS && utp->ut_user[0] != 0
                && utp->ut_line[0] != 0
                && strncmp (user, utp->ut_user, sizeof(utp->ut_user)) == 0) {
-            strncpy (tty, utp->ut_line, sizeof(utp->ut_line));
-	    alert (tty, md);
+	    alert(utp->ut_line, md);
         }
     }
     endutxent();
 #else
-    NMH_UNUSED (tty);
     NMH_UNUSED (utp);
 #endif /* HAVE_GETUTXENT */
 
-    exit (RCV_MOK);
+    exit(0);
 }
 
 
@@ -189,20 +195,21 @@ message_fd (char **vec)
     struct stat st;
 
     if ((tfile = m_mktemp2(NULL, invo_name, &fd, NULL)) == NULL) {
-	advise(NULL, "unable to create temporary file in %s", get_temp_dir());
+	inform("unable to create temporary file in %s", get_temp_dir());
 	return NOTOK;
     }
     (void) m_unlink(tfile);  /* Use fd, no longer need the file name. */
 
-    if ((child_id = fork()) == NOTOK) {
+    if ((child_id = fork()) == -1) {
 	/* fork error */
 	close (fd);
 	return header_fd ();
-    } else if (child_id) {
+    }
+    if (child_id) {
 	/* parent process */
 	if (!setjmp (myctx)) {
 	    SIGNAL (SIGALRM, alrmser);
-	    bytes = fstat(fileno (stdin), &st) != NOTOK ? (int) st.st_size : 100;
+	    bytes = fstat(fileno (stdin), &st) != -1 ? (int) st.st_size : 100;
 
 	    /* amount of time to wait depends on message size */
 	    if (bytes <= 100) {
@@ -218,7 +225,7 @@ message_fd (char **vec)
 	    pidwait(child_id, OK);
 	    alarm (0);
 
-	    if (fstat (fd, &st) != NOTOK && st.st_size > (off_t) 0)
+	    if (fstat (fd, &st) != -1 && st.st_size > 0)
 		return fd;
 	} else {
 	    /*
@@ -233,12 +240,11 @@ message_fd (char **vec)
 
     /* child process */
     rewind (stdin);
-    if (dup2 (fd, 1) == NOTOK || dup2 (fd, 2) == NOTOK)
-	_exit (-1);
-    closefds (3);
-    setpgid ((pid_t) 0, getpid ());	/* put in own process group */
-    if (execvp (vec[0], vec) == NOTOK) {
-        _exit (-1);
+    if (dup2 (fd, 1) == -1 || dup2 (fd, 2) == -1)
+	_exit(1);
+    setpgid(0, getpid());	/* put in own process group */
+    if (execvp (vec[0], vec) == -1) {
+        _exit(1);
     }
 
     return NOTOK;
@@ -251,9 +257,10 @@ header_fd (void)
     int fd;
     char *nfs;
     char *tfile = NULL;
+    charstring_t scanl = NULL;
 
     if ((tfile = m_mktemp2(NULL, invo_name, &fd, NULL)) == NULL) {
-	advise(NULL, "unable to create temporary file in %s", get_temp_dir());
+	inform("unable to create temporary file in %s", get_temp_dir());
         return NOTOK;
     }
     (void) m_unlink(tfile);  /* Use fd, no longer need the file name. */
@@ -262,13 +269,22 @@ header_fd (void)
 
     /* get new format string */
     nfs = new_fs (form, format, SCANFMT);
-    scan (stdin, 0, 0, nfs, width, 0, 0, NULL, 0L, 0);
+    scan (stdin, 0, 0, nfs, width, 0, 0, NULL, 0L, 0, &scanl);
     scan_finished ();
-    if (newline)
-        write (fd, "\n\r", 2);
-    write (fd, scanl, strlen (scanl));
-    if (bell)
-        write (fd, "\007", 1);
+    if (newline) {
+	if (write (fd, "\n\r", 2) < 0) {
+	    advise (tfile, "write LF/CR");
+	}
+    }
+    if (write (fd, charstring_buffer (scanl), charstring_bytes (scanl)) < 0) {
+	advise (tfile, "write");
+    }
+    charstring_free (scanl);
+    if (bell) {
+        if (write (fd, "\007", 1) < 0) {
+	    advise (tfile, "write BEL");
+        }
+    }
 
     return fd;
 }
@@ -289,22 +305,21 @@ alert (char *tty, int md)
      * write permission based on `biff' or `mesg'.
      */
     mask = biff ? S_IEXEC : (S_IWRITE >> 3);
-    if (stat (ttyspec, &st) == NOTOK || (st.st_mode & mask) == 0)
+    if (stat (ttyspec, &st) == -1 || (st.st_mode & mask) == 0)
 	return;
 
-    if (!setjmp (myctx)) {
-	SIGNAL (SIGALRM, alrmser);
-	alarm (2);
-	td = open (ttyspec, O_WRONLY);
-	alarm (0);
-	if (td == NOTOK)
-	    return;
-    } else {
+    if (setjmp (myctx)) {
 	alarm (0);
 	return;
     }
+    SIGNAL (SIGALRM, alrmser);
+    alarm (2);
+    td = open (ttyspec, O_WRONLY);
+    alarm (0);
+    if (td == -1)
+        return;
 
-    lseek (md, (off_t) 0, SEEK_SET);
+    lseek(md, 0, SEEK_SET);
 
     while ((i = read (md, buffer, sizeof(buffer))) > 0)
 	if (write (td, buffer, i) != i)
