@@ -5,16 +5,34 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/addrsbr.h>
-#include <h/fmt_scan.h>
-#include <h/utils.h>
-#include <sys/file.h>		/* L_SET */
+#include "h/mh.h"
+#include "sbr/pidwait.h"
+#include "sbr/charstring.h"
+#include "sbr/fmt_new.h"
+#include "sbr/m_name.h"
+#include "sbr/m_gmprot.h"
+#include "sbr/m_getfld.h"
+#include "sbr/read_switch.h"
+#include "sbr/concat.h"
+#include "sbr/uprf.h"
+#include "sbr/escape_addresses.h"
+#include "sbr/pidstatus.h"
+#include "sbr/utils.h"
+#include "sbr/arglist.h"
+#include "sbr/error.h"
+#include "sbr/addrsbr.h"
+#include "sbr/fmt_compile.h"
+#include "sbr/fmt_rfc2047.h"
+#include "sbr/fmt_scan.h"
+#include "sbr/done.h"
+#include <sys/file.h>
+#include "replsbr.h"
+#include "sbr/globals.h"
 
-extern short ccto;		/* from repl.c */
-extern short cccc;
-extern short ccme;
-extern short querysw;
+short ccto = -1;
+short cccc = -1;
+short ccme = -1;
+bool querysw;
 
 static int dftype=0;
 
@@ -22,7 +40,7 @@ static char *badaddrs = NULL;
 static char *dfhost = NULL;
 
 static struct mailname mq;
-static int nodupcheck = 0;		/* If set, no check for duplicates */
+static bool nodupcheck;		/* If set, no check for duplicates */
 
 static char *addrcomps[] = {
     "from",
@@ -63,7 +81,8 @@ replout (FILE *inb, char *msg, char *drft, struct msgs *mp, int outputlinelen,
     char name[NAMESZ], *cp;
     charstring_t scanl;
     static int dat[5];			/* aux. data for format routine */
-    m_getfld_state_t gstate = 0;
+    char decoded_buf[NMH_BUFSIZ + 1];
+    m_getfld_state_t gstate;
     struct fmt_callbacks cb;
 
     FILE *out;
@@ -110,9 +129,10 @@ replout (FILE *inb, char *msg, char *drft, struct msgs *mp, int outputlinelen,
     /*
      * pick any interesting stuff out of msg "inb"
      */
+    gstate = m_getfld_state_init(inb);
     for (;;) {
 	int msg_count = sizeof tmpbuf;
-	state = m_getfld (&gstate, name, tmpbuf, &msg_count, inb);
+	state = m_getfld2(&gstate, name, tmpbuf, &msg_count);
 	switch (state) {
 	    case FLD:
 	    case FLDPLUS:
@@ -128,7 +148,7 @@ replout (FILE *inb, char *msg, char *drft, struct msgs *mp, int outputlinelen,
 		    char_read += msg_count;
 		    while (state == FLDPLUS) {
 			msg_count= sizeof tmpbuf;
-			state = m_getfld (&gstate, name, tmpbuf, &msg_count, inb);
+			state = m_getfld2(&gstate, name, tmpbuf, &msg_count);
 			fmt_appendcomp(i, name, tmpbuf);
 			char_read += msg_count;
 		    }
@@ -136,7 +156,7 @@ replout (FILE *inb, char *msg, char *drft, struct msgs *mp, int outputlinelen,
 
 		while (state == FLDPLUS) {
 		    msg_count= sizeof tmpbuf;
-		    state = m_getfld (&gstate, name, tmpbuf, &msg_count, inb);
+		    state = m_getfld2(&gstate, name, tmpbuf, &msg_count);
 		}
 		break;
 
@@ -147,7 +167,7 @@ replout (FILE *inb, char *msg, char *drft, struct msgs *mp, int outputlinelen,
 		goto finished;
 
 	    default:
-		adios (NULL, "m_getfld() returned %d", state);
+		die("m_getfld2() returned %d", state);
 	}
     }
 
@@ -160,7 +180,7 @@ finished:
     /* set up the "fcc" pseudo-component */
     cptr = fmt_findcomp ("fcc");
     if (cptr) {
-	mh_xfree(cptr->c_text);
+	free(cptr->c_text);
 	if (fcc)
 	    cptr->c_text = mh_xstrdup(fcc);
 	else
@@ -168,7 +188,7 @@ finished:
     }
     cptr = fmt_findcomp ("user");
     if (cptr) {
-	mh_xfree(cptr->c_text);
+	free(cptr->c_text);
 	if ((cp = getenv("USER")))
 	    cptr->c_text = mh_xstrdup(cp);
 	else
@@ -179,8 +199,10 @@ finished:
      * if there's a "Subject" component, strip any "Re:"s off it
      */
     cptr = fmt_findcomp ("subject");
-    if (cptr && (cp = cptr->c_text)) {
-	char *sp = cp;
+    if (cptr && (cp = decode_rfc2047(cptr->c_text, decoded_buf,
+                                     sizeof decoded_buf, NULL) > 0
+                 ? decoded_buf : cptr->c_text)) {
+        char *sp = cp;
 
 	for (;;) {
 	    while (isspace((unsigned char) *cp))
@@ -204,7 +226,7 @@ finished:
     dat[2] = 0;
     dat[3] = outputlinelen;
     dat[4] = 0;
-    memset(&cb, 0, sizeof(cb));
+    ZERO(&cb);
     cb.formataddr = replformataddr;
     cb.concataddr = replconcataddr;
     fmt_scan (fmt, scanl, i, dat, &cb);
@@ -279,8 +301,8 @@ static char *
 replformataddr (char *orig, char *str)
 {
     int len;
-    char baddr[BUFSIZ], error[BUFSIZ];
-    int isgroup;
+    char baddr[BUFSIZ+6], error[BUFSIZ];
+    bool isgroup;
     char *dst;
     char *cp;
     char *sp;
@@ -310,7 +332,7 @@ replformataddr (char *orig, char *str)
     }
 
     /* concatenate all the new addresses onto 'buf' */
-    for (isgroup = 0; (cp = getname (fixed_str)); ) {
+    for (isgroup = false; (cp = getname (fixed_str)); ) {
 	if ((mp = getm (cp, dfhost, dftype, error, sizeof(error))) == NULL) {
 	    snprintf (baddr, sizeof(baddr), "\t%s -- %s\n", cp, error);
 	    badaddrs = add (baddr, badaddrs);
@@ -318,7 +340,7 @@ replformataddr (char *orig, char *str)
 	}
 	if (isgroup && (mp->m_gname || !mp->m_ingrp)) {
 	    *dst++ = ';';
-	    isgroup = 0;
+	    isgroup = false;
 	}
 	if (insert (mp)) {
 	    /* if we get here we're going to add an address */
@@ -329,7 +351,7 @@ replformataddr (char *orig, char *str)
 	    if (mp->m_gname) {
 		CHECKMEM (mp->m_gname);
 		CPY (mp->m_gname);
-		isgroup++;
+		isgroup = true;
 	    }
 	    sp = adrformat (mp);
 	    CHECKMEM (sp);
@@ -344,7 +366,7 @@ replformataddr (char *orig, char *str)
 
     *dst = '\0';
     last_dst = dst;
-    return (buf);
+    return buf;
 }
 
 
@@ -365,9 +387,9 @@ replconcataddr(char *orig, char *str)
 {
     char *cp;
 
-    nodupcheck = 1;
+    nodupcheck = true;
     cp = replformataddr(orig, str);
-    nodupcheck = 0;
+    nodupcheck = false;
     return cp;
 }
 
@@ -422,7 +444,7 @@ replfilter (FILE *in, FILE *out, char *filter, int fmtproc)
     if (filter == NULL)
 	return;
 
-    if (access (filter, R_OK) == NOTOK)
+    if (access (filter, R_OK) == -1)
 	adios (filter, "unable to read");
 
     rewind (in);
@@ -437,7 +459,6 @@ replfilter (FILE *in, FILE *out, char *filter, int fmtproc)
 	case OK:
 	    dup2 (fileno (in), fileno (stdin));
 	    dup2 (fileno (out), fileno (stdout));
-	    closefds (3);
 
 	    /*
 	     * We're not allocating the memory for the extra arguments,
@@ -454,7 +475,7 @@ replfilter (FILE *in, FILE *out, char *filter, int fmtproc)
 		arglist[argnum++] = formatproc;
 		break;
 	    case 0:
-	    	arglist[argnum++] = "-nofmtproc";
+		arglist[argnum++] = "-nofmtproc";
 		break;
 	    }
 
@@ -469,22 +490,23 @@ replfilter (FILE *in, FILE *out, char *filter, int fmtproc)
 		write(2, "\n", 1) < 0) {
 		advise ("stderr", "write");
 	    }
-	    _exit (-1);
+	    _exit(1);
 
 	default:
 	    if (pidXwait (pid, mhl))
 		done (1);
 	    fseek (out, 0L, SEEK_END);
+            arglist_free(mhl, arglist);
 	    break;
     }
 }
 
 
-static
-char *
-fix_addresses (char *str) {
+static char *
+fix_addresses (char *str)
+{
     char *fixed_str = NULL;
-    int fixed_address = 0;
+    bool fixed_address = false;
 
     if (str) {
         /*
@@ -521,7 +543,7 @@ fix_addresses (char *str) {
 
             /* With AD_NAME, errors are not reported to user. */
             if ((mp = getm (cp, dfhost, dftype, error,
-	    		    sizeof(error))) == NULL) {
+			    sizeof(error))) == NULL) {
                 const char *no_at_sign = "no at-sign after local-part";
 
                 adr_nodep->escape_local_part =
@@ -561,7 +583,7 @@ fix_addresses (char *str) {
                 adr = getname (new_adr);
                 if (adr != NULL  &&
                     (mp = getm (adr, dfhost, dftype, NULL, 0)) != NULL) {
-                    fixed_address = 1;
+                    fixed_address = true;
                     mnfree (mp);
                 }
                 free (angle_addr);
@@ -570,7 +592,7 @@ fix_addresses (char *str) {
                 np->adr = mh_xstrdup (adr);
 
                 /* Need to flush getname() */
-                while ((cp = getname (""))) continue;
+                while (getname ("")) continue;
             } /* else the np->adr is OK, so use it as-is. */
 
             free (display_name);
