@@ -8,7 +8,7 @@
 /*
  *  Under sendmail, users should add the line
  *
- * 	"| /usr/local/nmh/lib/slocal"
+ *	"| /usr/local/nmh/lib/slocal"
  *
  *  to their $HOME/.forward file.
  *
@@ -19,16 +19,33 @@
  * Ruud de Rooij <ruud@ruud.org>  Sun, 28 May 2000 17:28:55 +0200
  */
 
-#include <h/mh.h>
-#include <h/dropsbr.h>
-#include <h/rcvmail.h>
-#include <h/signals.h>
+#include <inttypes.h>
+#include "h/mh.h"
+#include "sbr/pidwait.h"
+#include "sbr/dtime.h"
+#include "sbr/m_gmprot.h"
+#include "sbr/m_getfld.h"
+#include "sbr/getarguments.h"
+#include "sbr/smatch.h"
+#include "sbr/stringdex.h"
+#include "sbr/trimcpy.h"
+#include "sbr/getcpy.h"
+#include "sbr/ambigsw.h"
+#include "sbr/pidstatus.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
+#include "dropsbr.h"
+#include <signal.h>
+#include "sbr/signals.h"
 #include <setjmp.h>
-#include <h/tws.h>
-#include <h/mts.h>
-#include <h/utils.h>
-#include "../sbr/lock_file.h"
-#include "../sbr/m_mktemp.h"
+#include "h/tws.h"
+#include "sbr/mts.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
+#include "sbr/lock_file.h"
+#include "sbr/m_mktemp.h"
+#include "sbr/globals.h"
 
 #include <pwd.h>
 #include <sys/ioctl.h>
@@ -82,10 +99,10 @@ DEFINE_SWITCH_ARRAY(SLOCAL, switches);
 static int globbed = 0;		/* have we built "vars" table yet?        */
 static int parsed = 0;		/* have we built header field table yet   */
 static int utmped = 0;		/* have we scanned umtp(x) file yet       */
-static int suppressdup = 0;	/* are we suppressing duplicate messages? */
+static bool suppressdup;	/* are we suppressing duplicate messages? */
 
-static int verbose = 0;
-static int debug = 0;
+static bool verbose;
+static bool debug;
 
 static char *addr = NULL;
 static char *user = NULL;
@@ -163,7 +180,7 @@ static int split (char *, char **);
 static int parse (int);
 static void expand (char *, char *, int);
 static void glob (int);
-static struct pair *lookup (struct pair *, char *);
+static struct pair *lookup (struct pair *, char *) PURE;
 static int logged_in (void);
 static int timely (char *, char *);
 static int usr_file (int, char *, int);
@@ -172,9 +189,9 @@ static int usr_folder (int, char *);
 static void alrmser (int);
 static void get_sender (char *, char **);
 static int copy_message (int, char *, int);
-static void verbose_printf (char *fmt, ...);
-static void adorn (char *, char *, ...);
-static void debug_printf (char *fmt, ...);
+static void verbose_printf (char *fmt, ...) CHECK_PRINTF(1, 2);
+static void adorn (char *, char *, ...) CHECK_PRINTF(2, 3);
+static void debug_printf (char *fmt, ...) CHECK_PRINTF(1, 2);
 static int suppress_duplicates (int, char *);
 static char *trim (char *);
 
@@ -183,12 +200,11 @@ int
 main (int argc, char **argv)
 {
     int fd, status;
-    FILE *fp;
     char *cp, *mdlvr = NULL, buf[BUFSIZ];
     char mailbox[BUFSIZ], tmpfil[BUFSIZ];
     char **argp, **arguments;
 
-    if (nmh_init(argv[0], 0 /* use context_foil() */)) { return 1; }
+    if (nmh_init(argv[0], false, false)) { return 1; }
 
     mts_init ();
     arguments = getarguments (invo_name, argc, argv, 0);
@@ -202,7 +218,7 @@ main (int argc, char **argv)
 		    ambigsw (cp, switches);
 		    done (1);
 		case UNKWNSW: 
-		    adios (NULL, "-%s unknown", cp);
+		    die("-%s unknown", cp);
 
 		case HELPSW: 
 		    snprintf (buf, sizeof(buf), "%s [switches]", invo_name);
@@ -214,70 +230,70 @@ main (int argc, char **argv)
 
 		case ADDRSW: 
 		    if (!(addr = *argp++))/* allow -xyz arguments */
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 		case INFOSW: 
 		    if (!(info = *argp++))/* allow -xyz arguments */
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 		case USERSW: 
 		    if (!(user = *argp++))/* allow -xyz arguments */
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 		case FILESW: 
 		    if (!(file = *argp++) || *file == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 		case SENDERSW: 
 		    if (!(sender = *argp++))/* allow -xyz arguments */
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 		case MAILBOXSW: 
 		    if (!(mbox = *argp++) || *mbox == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 		case HOMESW: 
 		    if (!(home = *argp++) || *home == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 
 		case MAILSW: 
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    if (mdlvr)
-			adios (NULL, "only one maildelivery file at a time!");
+			die("only one maildelivery file at a time!");
 		    mdlvr = cp;
 		    continue;
 
 		case VERBSW: 
-		    verbose++;
+		    verbose = true;
 		    continue;
 		case NVERBSW: 
-		    verbose = 0;
+		    verbose = false;
 		    continue;
 
 		case SUPPRESSDUP:
-		    suppressdup++;
+		    suppressdup = true;
 		    continue;
 		case NSUPPRESSDUP:
-		    suppressdup = 0;
+		    suppressdup = false;
 		    continue;
 		case DEBUGSW: 
-		    debug++;
+		    debug = true;
 		    continue;
 	    }
 	} else {
-		adios (NULL, "only switch arguments are supported");
+		die("only switch arguments are supported");
 	}
     }
 
     if (addr == NULL)
-	addr = getusername ();
+	addr = getusername (1);
     if (user == NULL) {
-	user = getusername ();
+	user = getusername (1);
     }
     if ((pw = getpwnam (user)) == NULL)
-	adios (NULL, "no such local user as %s", user);
+	die("no such local user as %s", user);
 
     if (chdir (pw->pw_dir) == -1)
 	if (chdir ("/") < 0) {
@@ -302,7 +318,7 @@ main (int argc, char **argv)
 
     /* Record the delivery time */
     if ((now = dlocaltimenow ()) == NULL)
-	adios (NULL, "unable to ascertain local time");
+	die("unable to ascertain local time");
     snprintf (ddate, sizeof(ddate), "Delivery-Date: %s\n", dtimenow (0));
 
     /*
@@ -317,7 +333,7 @@ main (int argc, char **argv)
 	if (debug)
 	    debug_printf ("retrieving message from file \"%s\"\n", file);
 	if ((fd = copy_message (tempfd, tmpfil, 1)) == -1)
-	    adios(NULL, "unable to create temporary file in %s",
+	    die("unable to create temporary file in %s",
                   get_temp_dir());
 	close (tempfd);
     } else {
@@ -325,7 +341,7 @@ main (int argc, char **argv)
 	if (debug)
 	    debug_printf ("retrieving message from stdin\n");
 	if ((fd = copy_message (fileno (stdin), tmpfil, 1)) == -1)
-	    adios(NULL, "unable to create temporary file in %s",
+	    die("unable to create temporary file in %s",
                   get_temp_dir());
     }
 
@@ -345,9 +361,6 @@ main (int argc, char **argv)
        functionality (they'll have to watch out for cases where we adios()). */
     (void) m_unlink (tmpfil);
 
-    if (!(fp = fdopen (fd, "r+")))
-	adios (NULL, "unable to access temporary file");
-
     /*
      * If no sender given, extract it
      * from envelope information.  */
@@ -378,7 +391,7 @@ main (int argc, char **argv)
     /* deliver the message */
     status = localmail (fd, mdlvr);
 
-    done (status != -1 ? RCV_MOK : RCV_MBX);
+    done(status != -1 ? 0 : 1);
     return 1;
 }
 
@@ -420,7 +433,12 @@ localmail (int fd, char *mdlvr)
 static int
 usr_delivery (int fd, char *delivery, int su)
 {
-    int i, accept, status=1, won, vecp, next;
+    int i;
+    bool accept;
+    int status=1;
+    bool won;
+    int vecp;
+    bool next;
     char *field, *pattern, *action, *result, *string;
     char buffer[BUFSIZ], tmpbuf[BUFSIZ];
     char *vec[NVEC];
@@ -437,14 +455,15 @@ usr_delivery (int fd, char *delivery, int su)
 	    || (st.st_uid != 0 && (su || st.st_uid != pw->pw_uid))
 	    || st.st_mode & (S_IWGRP|S_IWOTH)) {
 	if (verbose) {
-	    verbose_printf ("WARNING: %s has bad ownership/modes (su=%d,uid=%d,owner=%d,mode=0%o)\n",
-		    delivery, su, (int) pw->pw_uid, (int) st.st_uid, (int) st.st_mode);
+	    verbose_printf("WARNING: %s has bad ownership/modes "
+                "(su=%d,uid=%d,owner=%d,mode=0%" PRIoMAX ")\n",
+		delivery, su, (int)pw->pw_uid, (int)st.st_uid, (uintmax_t)st.st_mode);
 	}
 	return -1;
     }
 
-    won = 0;
-    next = 1;
+    won = false;
+    next = true;
 
     /* read and process delivery file */
     while (fgets (buffer, sizeof(buffer), fp)) {
@@ -475,6 +494,7 @@ usr_delivery (int fd, char *delivery, int su)
 	result  = vec[3];
 	string  = vec[4];
 
+        assert (result != NULL);
 	/* find out how to perform the action */
 	switch (result[0]) {
 	    case 'N':
@@ -483,7 +503,7 @@ usr_delivery (int fd, char *delivery, int su)
 		 * If previous condition failed, don't
 		 * do this - else fall through
 		 */
- 		if (!next)
+		if (!next)
 		    continue;
 		/* FALLTHRU */
 
@@ -502,7 +522,7 @@ usr_delivery (int fd, char *delivery, int su)
 		 * Take action, and consider delivered if
 		 * action is successful.
 		 */
-		accept = 1;
+		accept = true;
 		break;
 
 	    case 'R': 
@@ -512,7 +532,7 @@ usr_delivery (int fd, char *delivery, int su)
 		 * Take action, but don't consider delivered, even
 		 * if action is successful
 		 */
-		accept = 0;
+		accept = false;
 		break;
 	}
 
@@ -525,6 +545,7 @@ usr_delivery (int fd, char *delivery, int su)
 	    }
 	}
 
+        assert (field != NULL);
 	/* check if the field matches */
 	switch (*field) {
 	    case '*': 
@@ -555,9 +576,9 @@ usr_delivery (int fd, char *delivery, int su)
 		 */
 		if ((p = lookup (hdrs, field)) && (p->p_value != NULL)
 			&& matches (p->p_value, pattern)) {
-		    next = 1;
+		    next = true;
 		} else {
-		    next = 0;
+		    next = false;
 		    continue;
 		}
 		break;
@@ -629,14 +650,14 @@ usr_delivery (int fd, char *delivery, int su)
 		break;
 	}
 
-	if (status) next = 0;	/* action failed, mark for 'N' result */
+	if (status) next = false; /* action failed, mark for 'N' result */
 
 	if (accept && status == 0)
-	    won++;
+	    won = true;
     }
 
     fclose (fp);
-    return (won ? 0 : -1);
+    return won ? 0 : -1;
 }
 
 
@@ -713,7 +734,7 @@ parse (int fd)
     char name[NAMESZ], field[NMH_BUFSIZ];
     struct pair *p, *q;
     FILE  *in;
-    m_getfld_state_t gstate = 0;
+    m_getfld_state_t gstate;
 
     if (parsed++)
 	return 0;
@@ -737,15 +758,16 @@ parse (int fd)
      * Scan the headers of the message and build
      * a lookup table.
      */
+    gstate = m_getfld_state_init(in);
     for (i = 0;;) {
 	int fieldsz = sizeof field;
-	switch (state = m_getfld (&gstate, name, field, &fieldsz, in)) {
+	switch (state = m_getfld2(&gstate, name, field, &fieldsz)) {
 	    case FLD: 
 	    case FLDPLUS: 
 		lp = mh_xstrdup(field);
 		while (state == FLDPLUS) {
 		    fieldsz = sizeof field;
-		    state = m_getfld (&gstate, name, field, &fieldsz, in);
+		    state = m_getfld2(&gstate, name, field, &fieldsz);
 		    lp = add (field, lp);
 		}
 		for (p = hdrs; p->p_name; p++) {
@@ -786,7 +808,7 @@ parse (int fd)
 		break;
 
 	    default: 
-		inform("internal error in m_getfld");
+		inform("internal error in m_getfld2");
 		fclose (in);
 		return -1;
 	}
@@ -801,12 +823,12 @@ parse (int fd)
 	p->p_value = getcpy (q ? q->p_value : "");
 	p->p_flags &= ~P_CHK;
 	if (debug)
-	    debug_printf ("vars[%d]: name=\"%s\" value=\"%s\"\n",
-		    p - vars, p->p_name, trim(p->p_value));
+            debug_printf ("vars[%td]: name=\"%s\" value=\"%s\"\n",
+                p - vars, p->p_name, trim(p->p_value));
     }
     if (debug) {
 	for (p = hdrs; p->p_name; p++)
-	    debug_printf ("hdrs[%d]: name=\"%s\" value=\"%s\"\n",
+	    debug_printf ("hdrs[%td]: name=\"%s\" value=\"%s\"\n",
 		p - hdrs, p->p_name, p->p_value ? trim(p->p_value) : "");
     }
 
@@ -885,9 +907,18 @@ glob (int fd)
 	p->p_value = getcpy (info);
 
     if (debug) {
-	for (p = vars; p->p_name; p++)
-	    debug_printf ("vars[%d]: name=\"%s\" value=\"%s\"\n",
-		    p - vars, p->p_name, trim(p->p_value));
+        for (p = vars; p->p_name; p++) {
+            char *value, *quote;
+            if (p->p_value) {
+                value = trim(p->p_value);
+                quote = "\"";
+            } else {
+                value = "null";
+                quote = "";
+            }
+            debug_printf("vars[%td]: name=\"%s\" value=%s%s%s\n",
+                p - vars, p->p_name, quote, value, quote);
+        }
     }
 }
 
@@ -925,18 +956,19 @@ logged_in (void)
     setutxent();
 
     while ((utp = getutxent()) != NULL) {
+        assert (user != NULL);
         if ( utp->ut_type == USER_PROCESS && utp->ut_user[0] != 0
                 && strncmp (user, utp->ut_user, sizeof(utp->ut_user)) == 0) {
             if (debug)
                 continue;
             endutxent();
-            return (utmped = DONE);
+            return utmped = DONE;
         }
     }
 
     endutxent();
 #endif /* HAVE_GETUTXENT */
-    return (utmped = NOTOK);
+    return utmped = NOTOK;
 }
 
 #define	check(t,a,b)		if (t < a || t > b) return -1
@@ -1019,7 +1051,7 @@ usr_folder (int fd, char *string)
 
     /* get folder name ready */
     if (*string == '+')
-	strncpy(folder, string, sizeof(folder));
+	strncpy(folder, string, sizeof(folder) - 1);
     else
 	snprintf(folder, sizeof(folder), "+%s", string);
 
@@ -1045,7 +1077,7 @@ usr_pipe (int fd_arg, char *cmd, char *pgm, char **vec, int suppress)
 {
     volatile int fd = fd_arg;
     pid_t child_id;
-    int i, bytes, seconds, status;
+    int bytes, seconds, status;
     struct stat st;
 
     if (verbose && !suppress)
@@ -1053,9 +1085,7 @@ usr_pipe (int fd_arg, char *cmd, char *pgm, char **vec, int suppress)
 
     lseek(fd, 0, SEEK_SET);
 
-    for (i = 0; (child_id = fork()) == -1 && i < 5; i++)
-	sleep (5);
-
+    child_id = fork();
     switch (child_id) {
 	case -1: 
 	    /* fork error */
@@ -1075,7 +1105,6 @@ usr_pipe (int fd_arg, char *cmd, char *pgm, char **vec, int suppress)
 	    }
 	    if (fd != 3)
 		dup2 (fd, 3);
-	    closefds (4);
 
 #ifdef TIOCNOTTY
 	    if ((fd = open ("/dev/tty", O_RDWR)) != -1) {
@@ -1084,7 +1113,7 @@ usr_pipe (int fd_arg, char *cmd, char *pgm, char **vec, int suppress)
 	    }
 #endif /* TIOCNOTTY */
 
-	    setpgid ((pid_t) 0, getpid ());	/* put in own process group */
+            setpgid(0, getpid());	/* put in own process group */
 
 	    *environ = NULL;
 	    setenv("USER", pw->pw_name, 1);
@@ -1092,7 +1121,7 @@ usr_pipe (int fd_arg, char *cmd, char *pgm, char **vec, int suppress)
 	    setenv("SHELL", pw->pw_shell, 1);
 
 	    execvp (pgm, vec);
-	    _exit (-1);
+	    _exit(1);
 
 	default: 
 	    /* parent process */
@@ -1117,13 +1146,12 @@ usr_pipe (int fd_arg, char *cmd, char *pgm, char **vec, int suppress)
 		if (verbose) {
 		    if (status == 0)
 			verbose_printf (", success.\n");
-		    else
-			if ((status & 0xff00) == 0xff00)
-			    verbose_printf (", system error\n");
-			else
-			    pidstatus (status, stdout, ", failed");
+		    else if ((status & 0xff00) == 0xff00)
+                        verbose_printf (", system error\n");
+                    else
+                        pidstatus (status, stdout, ", failed");
 		}
-		return (status == 0 ? 0 : -1);
+		return status == 0 ? 0 : -1;
 	    }
             /*
              * Ruthlessly kill the child and anything
@@ -1164,7 +1192,7 @@ get_sender (char *envelope, char **sender)
     }
 
     i = LEN("From ");
-    strncpy (buffer, envelope + i, sizeof(buffer));
+    strncpy (buffer, envelope + i, sizeof(buffer) - 1);
     if ((cp = strchr(buffer, '\n'))) {
 	*cp = 0;
 	cp -= 24;
@@ -1295,14 +1323,14 @@ fputs_error:
 static char *
 trim (char *cp)
 {
-    char buffer[BUFSIZ*4];
+    static char buffer[BUFSIZ * 4];
     char *bp, *sp;
 
     if (cp == NULL)
 	return NULL;
 
     /* copy string into temp buffer */
-    strncpy (buffer, cp, sizeof(buffer));
+    strncpy (buffer, cp, sizeof(buffer) - 1);
     bp = buffer;
 
     /* skip over leading whitespace */
@@ -1322,8 +1350,7 @@ trim (char *cp)
 	if (isspace((unsigned char) *sp))
 	    *sp = ' ';
 
-    /* now return a copy */
-    return mh_xstrdup(bp);
+    return bp;
 }
 
 /*
@@ -1356,7 +1383,7 @@ adorn (char *what, char *fmt, ...)
     char *s;
 
     eindex = errno;	/* save the errno */
-    printf(", ");
+    fputs(", ", stdout);
 
     va_start(ap, fmt);
     vprintf(fmt, ap);
@@ -1405,7 +1432,7 @@ suppress_duplicates (int fd, char *file)
     datum key, value;
     DBM *db;
     FILE *in;
-    m_getfld_state_t gstate = 0;
+    m_getfld_state_t gstate;
 
     if ((fd1 = dup (fd)) == -1)
 	return -1;
@@ -1415,10 +1442,11 @@ suppress_duplicates (int fd, char *file)
     }
     rewind (in);
 
+    gstate = m_getfld_state_init(in);
     for (;;) {
         int failed_to_lock = 0;
 	int bufsz = sizeof buf;
-	state = m_getfld (&gstate, name, buf, &bufsz, in);
+	state = m_getfld2(&gstate, name, buf, &bufsz);
 	switch (state) {
 	    case FLD:
 	    case FLDPLUS:
@@ -1426,7 +1454,7 @@ suppress_duplicates (int fd, char *file)
 		if (strcasecmp (name, "Message-ID")) {
 		    while (state == FLDPLUS) {
 			bufsz = sizeof buf;
-			state = m_getfld (&gstate, name, buf, &bufsz, in);
+			state = m_getfld2(&gstate, name, buf, &bufsz);
 		    }
 		    continue;
 		}
@@ -1434,7 +1462,7 @@ suppress_duplicates (int fd, char *file)
 		cp = mh_xstrdup(buf);
 		while (state == FLDPLUS) {
 		    bufsz = sizeof buf;
-		    state = m_getfld (&gstate, name, buf, &bufsz, in);
+		    state = m_getfld2(&gstate, name, buf, &bufsz);
 		    cp = add (buf, cp);
 		}
 		key.dptr = trimcpy (cp);
@@ -1465,7 +1493,7 @@ suppress_duplicates (int fd, char *file)
 		if (value.dptr) {
 		    if (verbose)
 		        verbose_printf ("Message-ID: %s\n            already received on %s",
-				 cp, value.dptr);
+				 cp, (const char *) value.dptr);
 		    result = DONE;
 		} else {
 		    value.dptr  = ddate + sizeof("Delivery-Date:");
