@@ -5,15 +5,28 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "sendsbr.h"
+#include "sbr/m_gmprot.h"
+#include "sbr/getarguments.h"
+#include "sbr/concat.h"
+#include "sbr/smatch.h"
+#include "sbr/context_find.h"
+#include "sbr/ambigsw.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/utils.h"
+#include "sbr/arglist.h"
+#include "sbr/error.h"
 #include <fcntl.h>
-#include <h/md5.h>
-#include <h/mts.h>
-#include <h/tws.h>
-#include <h/mime.h>
-#include <h/mhparse.h>
-#include <h/utils.h>
-#include "../sbr/m_mktemp.h"
+#include "sbr/mts.h"
+#include "h/tws.h"
+#include "h/mime.h"
+#include "h/mhparse.h"
+#include "sbr/done.h"
+#include "sbr/m_mktemp.h"
+#include "sbr/base64.h"
+#include "sbr/globals.h"
 
 #define VIAMAIL_SWITCHES \
     X("to mailpath", 0, TOSW) \
@@ -22,7 +35,6 @@
     X("parameters arguments", 0, PARAMSW) \
     X("description text", 0, DESCRIPTSW) \
     X("comment text", 0, COMMENTSW) \
-    X("delay seconds", 0, DELAYSW) \
     X("verbose", 0, VERBSW) \
     X("noverbose", 0, NVERBSW) \
     X("version", 0, VERSIONSW) \
@@ -37,26 +49,23 @@ DEFINE_SWITCH_ENUM(VIAMAIL);
 DEFINE_SWITCH_ARRAY(VIAMAIL, switches);
 #undef X
 
-extern int debugsw;
-extern int splitsw;
-extern int verbsw;
+extern bool debugsw;
 
 /*
  * static prototypes
  */
-static int via_mail (char *, char *, char *, char *, char *, int, char *);
+static int via_mail (char *, char *, char *, char *, char *, char *);
 
 
 int
 main (int argc, char **argv)
 {
-    int delay = 0;
     char *f1 = NULL, *f2 = NULL, *f3 = NULL;
-    char *f4 = NULL, *f5 = NULL, *f7 = NULL;
+    char *f4 = NULL, *f5 = NULL, *f6 = NULL;
     char *cp, buf[BUFSIZ];
     char **argp, **arguments;
 
-    if (nmh_init(argv[0], 2)) { return 1; }
+    if (nmh_init(argv[0], true, false)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 0);
     argp = arguments;
@@ -68,7 +77,7 @@ main (int argc, char **argv)
 		ambigsw (cp, switches);
 		done (1);
 	    case UNKWNSW: 
-		adios (NULL, "-%s unknown", cp);
+		die("-%s unknown", cp);
 
 	    case HELPSW: 
 		snprintf (buf, sizeof(buf), "%s [switches]", invo_name);
@@ -80,58 +89,47 @@ main (int argc, char **argv)
     
 	    case TOSW:
 		if (!(f1 = *argp++))
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 	    case SUBJECTSW:
 		if (!(f2 = *argp++))
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 	    case PARAMSW:
 		if (!(f3 = *argp++))
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 	    case DESCRIPTSW:
 		if (!(f4 = *argp++))
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 	    case COMMENTSW:
 		if (!(f5 = *argp++))
-		    adios (NULL, "missing argument to %s", argp[-2]);
-		continue;
-	    case DELAYSW:
-		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
-
-		/*
-		 * If there is an error, just reset the delay parameter
-		 * to -1.  We will set a default delay later.
-		 */
-		if (sscanf (cp, "%d", &delay) != 1)
-		    delay = -1;
+		    die("missing argument to %s", argp[-2]);
 		continue;
 	    case FROMSW:
-		if (!(f7 = *argp++))
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		if (!(f6 = *argp++))
+		    die("missing argument to %s", argp[-2]);
 		continue;
 
 	    case VERBSW: 
-		verbsw = 1;
+		verbsw = true;
 		continue;
 	    case NVERBSW: 
-		verbsw = 0;
+		verbsw = false;
 		continue;
 
 	    case DEBUGSW:
-		debugsw = 1;
+		debugsw = true;
 		continue;
 	    }
 	}
     }
 
     if (!f1)
-	adios (NULL, "missing -viamail \"mailpath\" switch");
+	die("missing -viamail \"mailpath\" switch");
 
-    via_mail (f1, f2, f3, f4, f5, delay, f7);
+    via_mail (f1, f2, f3, f4, f5, f6);
     return 0;  /* dead code to satisfy the compiler */
 }
 
@@ -142,9 +140,9 @@ main (int argc, char **argv)
 
 static int
 via_mail (char *mailsw, char *subjsw, char *parmsw, char *descsw,
-          char *cmntsw, int delay, char *fromsw)
+          char *cmntsw, char *fromsw)
 {
-    int	status, vecp;
+    int	vecp;
     char tmpfil[BUFSIZ], *program;
     char **vec;
     struct stat st;
@@ -155,9 +153,9 @@ via_mail (char *mailsw, char *subjsw, char *parmsw, char *descsw,
     umask (~m_gmprot ());
 
     if ((tfile = m_mktemp2(NULL, invo_name, NULL, &fp)) == NULL) {
-	adios(NULL, "unable to create temporary file in %s", get_temp_dir());
+	die("unable to create temporary file in %s", get_temp_dir());
     }
-    strncpy (tmpfil, tfile, sizeof(tmpfil));
+    strncpy (tmpfil, tfile, sizeof(tmpfil) - 1);
 
     if (!strchr(mailsw, '@'))
 	mailsw = concat (mailsw, "@", LocalName (0), NULL);
@@ -193,16 +191,9 @@ via_mail (char *mailsw, char *subjsw, char *parmsw, char *descsw,
     if (fflush (fp))
 	adios (tmpfil, "error writing to");
 
-    if (fstat (fileno (fp), &st) == NOTOK)
+    if (fstat (fileno (fp), &st) == -1)
 	adios ("failed", "fstat of %s", tmpfil);
 
-    if (delay < 0)
-	splitsw = 10;
-    else
-	splitsw = delay;
-
-    status = 0;
-
     vec = argsplit(postproc, &program, &vecp);
     if (verbsw)
 	vec[vecp++] = "-verbose";
@@ -213,14 +204,7 @@ via_mail (char *mailsw, char *subjsw, char *parmsw, char *descsw,
 	vec[vecp++] = cp;
     }
 
-    switch (sendsbr (vec, vecp, program, tmpfil, &st, 0, NULL)) {
-	case DONE:
-	case NOTOK:
-	    status++;
-	    break;
-	case OK:
-	    break;
-    }
+    int	status = sendsbr(vec, vecp, program, tmpfil, &st, 0, NULL) == OK ? 0 : 1;
 
     fclose (fp);
     if (m_unlink (tmpfil) == -1)
