@@ -5,14 +5,15 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
+#include "h/mh.h"
+#include "whatnowsbr.h"
+#include "sbr/utils.h"
 
 
 int
 main (int argc, char **argv)
 {
-    if (nmh_init(argv[0], 2)) { return 1; }
+    if (nmh_init(argv[0], true, false)) { return 1; }
 
     return WhatNow (argc, argv);
 }
