@@ -37,16 +37,49 @@
  *				by attachment number.
  */
 
-#include <h/mh.h>
+#include "h/mh.h"
+#include "sbr/pidwait.h"
+#include "whatnowsbr.h"
+#include "sendsbr.h"
+#include "annosbr.h"
+#include "sbr/m_getfld.h"
+#include "sbr/getarguments.h"
+#ifdef READLINE_SUPPORT
+#include "sbr/read_switch_multiword_via_readline.h"
+#else /* ! READLINE_SUPPORT */
+#include "sbr/read_switch_multiword.h"
+#endif /* READLINE_SUPPORT */
+#include "sbr/concat.h"
+#include "sbr/showfile.h"
+#include "sbr/smatch.h"
+#include "sbr/r1bindex.h"
+#include "sbr/refile.h"
+#include "sbr/copyip.h"
+#include "sbr/getcpy.h"
+#include "sbr/m_draft.h"
+#include "sbr/m_backup.h"
+#include "sbr/geteditor.h"
+#include "sbr/context_save.h"
+#include "sbr/context_find.h"
+#include "sbr/brkstring.h"
+#include "sbr/ambigsw.h"
+#include "sbr/push.h"
+#include "sbr/path.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/utils.h"
+#include "sbr/arglist.h"
+#include "sbr/error.h"
 #include <fcntl.h>
-#include <h/mime.h>
-#include <h/utils.h>
+#include "h/mime.h"
 #ifdef OAUTH_SUPPORT
-# include <h/oauth.h>
+#include "sbr/oauth.h"
 #endif
-#include "../sbr/m_maildir.h"
-#include "../sbr/m_mktemp.h"
-#include "../sbr/mime_type.h"
+#include "sbr/done.h"
+#include "sbr/m_maildir.h"
+#include "sbr/m_mktemp.h"
+#include "sbr/mime_type.h"
+#include "sbr/globals.h"
 
 #define WHATNOW_SWITCHES \
     X("draftfolder +folder", 0, DFOLDSW) \
@@ -103,12 +136,12 @@ static char *myprompt = "\nWhat now? ";
  */
 static int editfile (char **, char **, char *, int, struct msgs *,
 	char *, char *, int, int);
-static int sendfile (char **, char *, int);
-static void sendit (char *, char **, char *, int);
+static int sendfile(char **, char *, bool);
+static void sendit(char *, char **, char *, bool);
 static int buildfile (char **, char *);
 static int whomfile (char **, char *);
 static int removefile (char *);
-static int checkmimeheader (char *);
+static bool checkmimeheader (char *);
 static void writelscmd(char *, int, char *, char **);
 static void writesomecmd(char *buf, int bufsz, char *cmd, char *trailcmd, char **argp);
 static FILE* popen_in_dir(const char *dir, const char *cmd, const char *type);
@@ -119,14 +152,15 @@ static int copyf (char *, char *);
 int
 WhatNow (int argc, char **argv)
 {
-    int isdf = 0, nedit = 0, use = 0, atfile = 1;
+    int isdf = 0;
+    bool nedit = false;
+    int use = 0, atfile = 1;
     char *cp, *dfolder = NULL, *dmsg = NULL;
     char *ed = NULL, *drft = NULL, *msgnam = NULL;
-    char buf[BUFSIZ], prompt[BUFSIZ];
+    char buf[BUFSIZ];
     char **argp, **arguments;
     struct stat st;
     char	cwd[PATH_MAX + 1];	/* current working directory */
-    char	file[PATH_MAX + 1];	/* file name buffer */
     char	shell[PATH_MAX + 1];	/* shell response buffer */
     FILE	*f;			/* read pointer for bgnd proc */
     char	*l;			/* set on -l to alist  command */
@@ -153,7 +187,7 @@ WhatNow (int argc, char **argv)
 		ambigsw (cp, whatnowswitches);
 		done (1);
 	    case UNKWNSW:
-		adios (NULL, "-%s unknown", cp);
+		die("-%s unknown", cp);
 
 	    case HELPSW:
 		snprintf (buf, sizeof(buf), "%s [switches] [file]", invo_name);
@@ -165,17 +199,17 @@ WhatNow (int argc, char **argv)
 
 	    case DFOLDSW:
 		if (dfolder)
-		    adios (NULL, "only one draft folder at a time!");
+		    die("only one draft folder at a time!");
 		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		dfolder = path (*cp == '+' || *cp == '@' ? cp + 1 : cp,
 				*cp != '@' ? TFOLDER : TSUBCWF);
 		continue;
 	    case DMSGSW:
 		if (dmsg)
-		    adios (NULL, "only one draft message at a time!");
+		    die("only one draft message at a time!");
 		if (!(dmsg = *argp++) || *dmsg == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 	    case NDFLDSW:
 		dfolder = NULL;
@@ -184,52 +218,50 @@ WhatNow (int argc, char **argv)
 
 	    case EDITRSW:
 		if (!(ed = *argp++) || *ed == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
-		nedit = 0;
+		    die("missing argument to %s", argp[-2]);
+		nedit = false;
 		continue;
 	    case NEDITSW:
-		nedit++;
+		nedit = true;
 		continue;
 
 	    case PRMPTSW:
 		if (!(myprompt = *argp++) || *myprompt == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
+		    die("missing argument to %s", argp[-2]);
 		continue;
 	    }
 	}
 	if (drft)
-	    adios (NULL, "only one draft at a time!");
-	else
-	    drft = cp;
+	    die("only one draft at a time!");
+        drft = cp;
     }
 
     if ((drft == NULL && (drft = getenv ("mhdraft")) == NULL) || *drft == 0)
-	drft = getcpy (m_draft (dfolder, dmsg, 1, &isdf));
+	drft = mh_xstrdup(m_draft(dfolder, dmsg, 1, &isdf));
 
     msgnam = (cp = getenv ("mhaltmsg")) && *cp ? mh_xstrdup(cp) : NULL;
 
     if ((cp = getenv ("mhatfile")) && *cp)
-    	atfile = atoi(cp);
+	atfile = atoi(cp);
 
     if ((cp = getenv ("mhuse")) && *cp)
 	use = atoi (cp);
 
     if (ed == NULL && ((ed = getenv ("mheditor")) == NULL || *ed == 0)) {
 	ed = NULL;
-	nedit++;
+	nedit = true;
     }
 
     /* start editing the draft, unless -noedit was given */
     if (!nedit && editfile (&ed, NULL, drft, use, NULL, msgnam,
-    			    NULL, 1, atfile) < 0)
+			    NULL, 1, atfile) < 0)
 	done (1);
 
-    snprintf (prompt, sizeof(prompt), myprompt, invo_name);
     for (;;) {
 #ifdef READLINE_SUPPORT
-	if (!(argp = read_switch_multiword_via_readline (prompt, aleqs))) {
+	if (!(argp = read_switch_multiword_via_readline(myprompt, aleqs))) {
 #else /* ! READLINE_SUPPORT */
-	if (!(argp = read_switch_multiword (prompt, aleqs))) {
+	if (!(argp = read_switch_multiword(myprompt, aleqs))) {
 #endif /* READLINE_SUPPORT */
 	    (void) m_unlink (LINK);
 	    done (1);
@@ -253,7 +285,7 @@ WhatNow (int argc, char **argv)
 	    if (*++argp)
 		ed = *argp++;
 	    if (editfile (&ed, argp, drft, NOUSE, NULL, msgnam,
-	    		  NULL, 1, atfile) == NOTOK)
+			  NULL, 1, atfile) == NOTOK)
 		done (1);
 	    break;
 
@@ -273,7 +305,7 @@ WhatNow (int argc, char **argv)
 		((*argp)[0] == '-' && (*argp)[1] == 'd'))) {
 		removefile (drft);
 	    } else {
-		if (stat (drft, &st) != NOTOK)
+		if (stat (drft, &st) != -1)
 		    inform("draft left on %s", drft);
 	    }
 	    done (1);
@@ -285,13 +317,13 @@ WhatNow (int argc, char **argv)
 
 	case PUSHSW:
 	    /* Send draft in background */
-	    if (sendfile (++argp, drft, 1))
+	    if (sendfile(++argp, drft, true))
 		done (1);
 	    break;
 
 	case SENDSW:
 	    /* Send draft */
-	    sendfile (++argp, drft, 0);
+	    sendfile(++argp, drft, false);
 	    break;
 
 	case REFILEOPT:
@@ -310,8 +342,7 @@ WhatNow (int argc, char **argv)
 
 	    if (*(argp+1) == NULL) {
 		strcpy(buf, "$SHELL -c \"cd&&pwd\"");
-	    }
-	    else {
+	    } else {
 		writesomecmd(buf, BUFSIZ, "cd", "pwd", argp);
 	    }
 	    if ((f = popen_in_dir(cwd, buf, "r")) != NULL) {
@@ -320,8 +351,7 @@ WhatNow (int argc, char **argv)
 		}
                 trim_suffix_c(cwd, '\n');
 		pclose(f);
-	    }
-	    else {
+	    } else {
 		advise("popen", "could not get directory");
 	    }
 
@@ -387,7 +417,7 @@ WhatNow (int argc, char **argv)
 	     *	Attach files to current draft.
 	     */
 
-            int verbose = 0;
+            bool verbose = false;
             char **ap;
 
 	    if (checkmimeheader(drft))
@@ -396,7 +426,7 @@ WhatNow (int argc, char **argv)
 	    for (ap = argp+1; *ap; ++ap) {
 		if (strcmp(*ap, "-v") == 0) {
 		    ++argp;
-		    verbose = 1;
+		    verbose = true;
 		} else if (*ap[0] != '-') {
 		    break;
 		}
@@ -422,13 +452,15 @@ WhatNow (int argc, char **argv)
 	     */
 
 	    if ((f = popen_in_dir(cwd, buf, "r")) != NULL) {
+                char file[2 * PATH_MAX + 2]; /* file name buffer */
+
 		while (fgets(shell, sizeof (shell), f) != NULL) {
 		    char *ctype;
 
                     trim_suffix_c(shell, '\n');
 
 		    if (*shell == '/') {
-		    	strncpy(file, shell, sizeof(file));
+			strncpy(file, shell, sizeof(file));
 			file[sizeof(file) - 1] = '\0';
 		    } else {
 			snprintf(file, sizeof(file), "%s/%s", cwd, shell);
@@ -443,8 +475,7 @@ WhatNow (int argc, char **argv)
 		}
 
 		pclose(f);
-	    }
-	    else {
+	    } else {
 		advise("popen", "could not get file from shell");
 	    }
 
@@ -552,7 +583,7 @@ writesomecmd(char *buf, int bufsz, char *cmd, char *trailcmd, char **argp)
      */
     int trailln = strlen(trailcmd) + 4;
     if (ln < 0 || ln + trailln > bufsz)
-	adios(NULL, "arguments too long");
+	die("arguments too long");
 
     cp = buf + ln;
 
@@ -560,7 +591,7 @@ writesomecmd(char *buf, int bufsz, char *cmd, char *trailcmd, char **argp)
 	ln = strlen(*argp);
 	/* +1 for leading space */
 	if (ln + trailln + 1 > bufsz - (cp-buf))
-	    adios(NULL, "arguments too long");
+	    die("arguments too long");
 	*cp++ = ' ';
 	memcpy(cp, *argp, ln+1);
 	cp += ln;
@@ -635,7 +666,7 @@ popen_in_dir(const char *dir, const char *cmd, const char *type)
  * EDIT
  */
 
-static int  reedit = 0;		/* have we been here before?     */
+static bool reedit;		/* have we been here before?     */
 static char *edsave = NULL;	/* the editor we used previously */
 
 
@@ -648,7 +679,7 @@ editfile (char **ed, char **arg, char *file, int use, struct msgs *mp,
     char *cp, *prog, **vec;
     struct stat st;
 
-    int	slinked = 0;
+    bool slinked = false;
 
     /* Was there a previous edit session? */
     if (reedit && (*ed || edsave)) {
@@ -670,7 +701,7 @@ editfile (char **ed, char **arg, char *file, int use, struct msgs *mp,
 
     if (altmsg) {
 	if (mp == NULL || *altmsg == '/' || cwd == NULL)
-	    strncpy (altpath, altmsg, sizeof(altpath));
+	    strncpy (altpath, altmsg, sizeof(altpath) - 1);
 	else
 	    snprintf (altpath, sizeof(altpath), "%s/%s", mp->foldpath, altmsg);
 	if (cwd == NULL)
@@ -680,13 +711,13 @@ editfile (char **ed, char **arg, char *file, int use, struct msgs *mp,
 
 	if (atfile) {
 	    (void) m_unlink (linkpath);
-	    if (link (altpath, linkpath) == NOTOK) {
+	    if (link (altpath, linkpath) == -1) {
 		if (symlink (altpath, linkpath) < 0) {
 		    adios (linkpath, "symlink");
 		}
-		slinked = 1;
+		slinked = true;
 	    } else {
-		slinked = 0;
+		slinked = false;
 	    }
 	}
     }
@@ -723,14 +754,14 @@ editfile (char **ed, char **arg, char *file, int use, struct msgs *mp,
 	    execvp (prog, vec);
 	    fprintf (stderr, "unable to exec ");
 	    perror (*ed);
-	    _exit (-1);
+	    _exit(1);
 
 	default:
 	    if ((status = pidwait (pid, NOTOK))) {
 		if (((status & 0xff00) != 0xff00)
 		    && (!reedit || (status & 0x00ff))) {
 		    if (!use && (status & 0xff00) &&
-			    (rename (file, cp = m_backup (file)) != NOTOK)) {
+			    (rename (file, cp = m_backup (file)) != -1)) {
 			inform("problems with edit--draft left in %s", cp);
 		    } else {
 			inform("problems with edit--%s preserved", file);
@@ -740,18 +771,18 @@ editfile (char **ed, char **arg, char *file, int use, struct msgs *mp,
 		break;
 	    }
 
-	    reedit++;
+	    reedit = true;
 	    if (altmsg
 		    && mp
 		    && !is_readonly(mp)
 		    && (slinked
-		           ? lstat (linkpath, &st) != NOTOK
+		           ? lstat (linkpath, &st) != -1
 				&& S_ISREG(st.st_mode)
 				&& copyf (linkpath, altpath) == NOTOK
-		           : stat (linkpath, &st) != NOTOK
+		           : stat (linkpath, &st) != -1
 				&& st.st_nlink == 1
 				&& (m_unlink (altpath) == NOTOK
-					|| link (linkpath, altpath) == NOTOK)))
+					|| link (linkpath, altpath) == -1)))
 		advise (linkpath, "unable to update %s from", altmsg);
     }
 
@@ -773,9 +804,9 @@ copyf (char *ifile, char *ofile)
     int i, in, out;
     char buffer[BUFSIZ];
 
-    if ((in = open (ifile, O_RDONLY)) == NOTOK)
+    if ((in = open (ifile, O_RDONLY)) == -1)
 	return NOTOK;
-    if ((out = open (ofile, O_WRONLY | O_TRUNC)) == NOTOK) {
+    if ((out = open (ofile, O_WRONLY | O_TRUNC)) == -1) {
 	admonish (ofile, "unable to open and truncate");
 	close (in);
 	return NOTOK;
@@ -799,10 +830,10 @@ copyf (char *ifile, char *ofile)
  */
 
 static int
-sendfile (char **arg, char *file, int pushsw)
+sendfile(char **arg, char *file, bool pushsw)
 {
     pid_t child_id;
-    int i, vecp;
+    int vecp;
     char *cp, *sp, **vec, *program;
 
     /*
@@ -811,7 +842,7 @@ sendfile (char **arg, char *file, int pushsw)
      */
     if (strcmp (sp = r1bindex (sendproc, '/'), "send") == 0) {
 	cp = invo_name;
-	sendit (invo_name = sp, arg, file, pushsw);
+	sendit(invo_name = sp, arg, file, pushsw);
 	invo_name = cp;
 	return 1;
     }
@@ -819,12 +850,11 @@ sendfile (char **arg, char *file, int pushsw)
     context_save ();	/* save the context file */
     fflush (stdout);
 
-    for (i = 0; (child_id = fork()) == NOTOK && i < 5; i++)
-	sleep (5);
+    child_id = fork();
     switch (child_id) {
 	case NOTOK:
-	    inform("unable to fork, so sending directly...");
-	    /* FALLTHRU */
+            adios("fork", "failed:");
+
 	case OK:
 	    vec = argsplit(sendproc, &program, &vecp);
 	    if (pushsw)
@@ -838,7 +868,7 @@ sendfile (char **arg, char *file, int pushsw)
 	    execvp (program, vec);
 	    fprintf (stderr, "unable to exec ");
 	    perror (sendproc);
-	    _exit (-1);
+	    _exit(1);
 
 	default:
 	    if (pidwait(child_id, OK) == 0)
@@ -866,7 +896,7 @@ buildfile (char **argp, char *file)
 	while (argp[i])
 	    i++;
     }
-    args = (char **) mh_xmalloc((i + 2) * sizeof(char *));
+    args = mh_xmalloc((i + 2) * sizeof(char *));
 
     /*
      * For backward compatibility, we need to add -build
@@ -884,7 +914,7 @@ buildfile (char **argp, char *file)
     i = editfile (&ed, args, file, NOUSE, NULL, NULL, NULL, 0, 0);
     free (args);
 
-    return (i ? NOTOK : OK);
+    return i ? NOTOK : OK;
 }
 
 
@@ -957,34 +987,19 @@ DEFINE_SWITCH_ARRAY(SEND, sendswitches);
 #undef X
 
 
-extern int debugsw;		/* from sendsbr.c */
-extern int forwsw;
-extern int inplace;
-extern int pushsw;
-extern int splitsw;
-extern int unique;
-extern int verbsw;
-
-extern char *altmsg;		/*  .. */
-extern char *annotext;
-extern char *distfile;
+extern bool debugsw;		/* from sendsbr.c */
 
 
 static void
-sendit (char *sp, char **arg, char *file, int pushed)
+sendit(char *sp, char **arg, char *file, bool pushed)
 {
     int	vecp, n = 1;
     char *cp, buf[BUFSIZ], **argp, *program;
     char **arguments, *savearg[MAXARGS], **vec;
     const char *user = NULL, *saslmech = NULL;
     char *auth_svc = NULL;
-    int snoop = 0;
     struct stat st;
 
-#ifndef	lint
-    int	distsw = 0;
-#endif
-
     /*
      * Make sure these are defined.  In particular, we need
      * savearg[1] to be NULL, in case "arg" is NULL below.  It
@@ -1018,10 +1033,10 @@ sendit (char *sp, char **arg, char *file, int pushed)
     arguments = getarguments (sp, n, savearg, 1);
     argp = arguments;
 
-    debugsw = 0;
-    forwsw = 1;
+    debugsw = false;
+    forwsw = true;
     inplace = 1;
-    unique = 0;
+    unique = false;
 
     altmsg = NULL;
     annotext = NULL;
@@ -1034,7 +1049,7 @@ sendit (char *sp, char **arg, char *file, int pushed)
     vec = argsplit(postproc, &program, &vecp);
 
     vec[vecp++] = "-library";
-    vec[vecp++] = getcpy (m_maildir (""));
+    vec[vecp++] = mh_xstrdup(m_maildir(""));
 
     if ((cp = context_find ("fileproc"))) {
 	vec[vecp++] = "-fileproc";
@@ -1071,43 +1086,36 @@ sendit (char *sp, char **arg, char *file, int pushed)
 		    return;
 
 		case SPSHSW:
-		    pushed++;
+		    pushed = true;
 		    continue;
 		case NSPSHSW:
-		    pushed = 0;
-		    continue;
-
-		case SPLITSW:
-		    if (!(cp = *argp++) || sscanf (cp, "%d", &splitsw) != 1) {
-			inform("missing argument to %s", argp[-2]);
-			return;
-		    }
+		    pushed = false;
 		    continue;
 
 		case UNIQSW:
-		    unique++;
+		    unique = true;
 		    continue;
 		case NUNIQSW:
-		    unique = 0;
+		    unique = false;
 		    continue;
 		case FORWSW:
-		    forwsw++;
+		    forwsw = true;
 		    continue;
 		case NFORWSW:
-		    forwsw = 0;
+		    forwsw = false;
 		    continue;
 
 		case VERBSW:
-		    verbsw++;
+		    verbsw = true;
 		    vec[vecp++] = --cp;
 		    continue;
 		case NVERBSW:
-		    verbsw = 0;
+		    verbsw = false;
 		    vec[vecp++] = --cp;
 		    continue;
 
 		case DEBUGSW:
-		    debugsw++;
+		    debugsw = true;
 		    /* FALLTHRU */
 		case NFILTSW:
 		case FRMTSW:
@@ -1131,18 +1139,17 @@ sendit (char *sp, char **arg, char *file, int pushed)
 		    continue;
 
 		case SNOOPSW:
-                    snoop++;
 		    vec[vecp++] = --cp;
 		    continue;
 
 		case AUTHSERVICESW:
 #ifdef OAUTH_SUPPORT
 		    if (!(auth_svc = *argp++) || *auth_svc == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 #else
                     NMH_UNUSED (user);
                     NMH_UNUSED (auth_svc);
-		    adios (NULL, "not built with OAuth support");
+		    die("not built with OAuth support");
 #endif
 		    continue;
 
@@ -1207,18 +1214,15 @@ sendit (char *sp, char **arg, char *file, int pushed)
 
     if ((cp = getenv ("mhdist"))
 	    && *cp
-#ifndef lint
-	    && (distsw = atoi (cp))
-#endif /* not lint */
+	    && atoi(cp)
 	    && altmsg) {
 	vec[vecp++] = "-dist";
 	if ((cp = m_mktemp2(altmsg, invo_name, NULL, NULL)) == NULL) {
-	    adios(NULL, "unable to create temporary file in %s",
-		  get_temp_dir());
+	    die("unable to create temporary file");
 	}
 	distfile = mh_xstrdup(cp);
 	(void) m_unlink(distfile);
-	if (link (altmsg, distfile) == NOTOK)
+	if (link (altmsg, distfile) == -1)
 	    adios (distfile, "unable to link %s to", altmsg);
     } else {
 	distfile = NULL;
@@ -1227,18 +1231,18 @@ sendit (char *sp, char **arg, char *file, int pushed)
 #ifdef OAUTH_SUPPORT
     if (auth_svc == NULL) {
         if (saslmech  &&  ! strcasecmp(saslmech, "xoauth2")) {
-            adios (NULL, "must specify -authservice with -saslmech xoauth2");
+            die("must specify -authservice with -saslmech xoauth2");
         }
     } else {
         if (user == NULL) {
-            adios (NULL, "must specify -user with -saslmech xoauth2");
+            die("must specify -user with -saslmech xoauth2");
         }
     }
 #else
     NMH_UNUSED(saslmech);
 #endif /* OAUTH_SUPPORT */
 
-    if (altmsg == NULL || stat (altmsg, &st) == NOTOK) {
+    if (altmsg == NULL || stat (altmsg, &st) == -1) {
 	st.st_mtime = 0;
 	st.st_dev = 0;
 	st.st_ino = 0;
@@ -1246,8 +1250,6 @@ sendit (char *sp, char **arg, char *file, int pushed)
     if ((pushsw = pushed))
 	push ();
 
-    closefds (3);
-
     if (sendsbr (vec, vecp, program, file, &st, 1, auth_svc) == OK)
 	done (0);
 }
@@ -1267,7 +1269,7 @@ whomfile (char **arg, char *file)
     fflush (stdout);
 
     switch (pid = fork()) {
-	case NOTOK:
+	case -1:
 	    advise ("fork", "unable to");
 	    return 1;
 
@@ -1282,10 +1284,10 @@ whomfile (char **arg, char *file)
 	    execvp (program, vec);
 	    fprintf (stderr, "unable to exec ");
 	    perror (whomproc);
-	    _exit (-1);		/* NOTREACHED */
+	    _exit(1);		/* NOTREACHED */
 
 	default:
-	    return (pidwait (pid, NOTOK) & 0377 ? 1 : 0);
+	    return pidwait(pid, NOTOK) & 0377 ? 1 : 0;
     }
 }
 
@@ -1308,28 +1310,29 @@ removefile (char *drft)
  * Return 1 if we already have a MIME-Version header, 0 otherwise.
  */
 
-static int
+static bool
 checkmimeheader (char *drft)
 {
     FILE *f;
-    m_getfld_state_t gstate = 0;
+    m_getfld_state_t gstate;
     char buf[NMH_BUFSIZ], name[NAMESZ];
-    int state, retval = 0;
+    bool retval = false;
 
     if ((f = fopen(drft, "r")) == NULL) {
 	admonish(drft, "unable to read draft");
-	return (0);
+	return false;
     }
 
+    gstate = m_getfld_state_init(f);
     for (;;) {
 	int bufsz = sizeof(buf);
-	switch (state = m_getfld(&gstate, name, buf, &bufsz, f)) {
+	switch (m_getfld2(&gstate, name, buf, &bufsz)) {
 	case FLD:
 	case FLDPLUS:
 	    if (strcasecmp(name, VRSN_FIELD) == 0) {
 		inform("Cannot use attach commands with already-"
 		       "formatted MIME message \"%s\"", drft);
-		retval = 1;
+		retval = true;
 		break;
 	    }
 	    continue;
