@@ -5,11 +5,26 @@
  * complete copyright information.
  */
 
-#include <h/mh.h>
-#include <h/utils.h>
-#include <h/signals.h>
-#include "../sbr/m_maildir.h"
-#include "../sbr/m_mktemp.h"
+#include "h/mh.h"
+#include "sbr/pidwait.h"
+#include "distsbr.h"
+#include "sbr/getarguments.h"
+#include "sbr/smatch.h"
+#include "sbr/m_draft.h"
+#include "sbr/context_find.h"
+#include "sbr/brkstring.h"
+#include "sbr/ambigsw.h"
+#include "sbr/path.h"
+#include "sbr/print_version.h"
+#include "sbr/print_help.h"
+#include "sbr/error.h"
+#include "sbr/done.h"
+#include "sbr/utils.h"
+#include <signal.h>
+#include "sbr/signals.h"
+#include "sbr/m_maildir.h"
+#include "sbr/m_mktemp.h"
+#include "sbr/globals.h"
 
 #ifndef CYRUS_SASL
 # define SASLminc(a) (a)
@@ -58,13 +73,14 @@ int
 main (int argc, char **argv)
 {
     pid_t child_id = OK;
-    int i, status, isdf = 0;
-    int distsw = 0, vecp = 0;
+    int status, isdf = 0;
+    int vecp = 0;
+    bool distsw;
     char *cp, *dfolder = NULL, *dmsg = NULL;
     char *msg = NULL, **ap, **argp, backup[BUFSIZ];
     char buf[BUFSIZ], **arguments, *vec[MAXARGS];
 
-    if (nmh_init(argv[0], 2)) { return 1; }
+    if (nmh_init(argv[0], true, false)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -72,7 +88,7 @@ main (int argc, char **argv)
     vec[vecp++] = invo_name;
     vec[vecp++] = "-whom";
     vec[vecp++] = "-library";
-    vec[vecp++] = getcpy (m_maildir (""));
+    vec[vecp++] = mh_xstrdup(m_maildir(""));
 
     if ((cp = context_find ("credentials"))) {
 	/* post doesn't read context so need to pass credentials. */
@@ -90,7 +106,7 @@ main (int argc, char **argv)
 		    ambigsw (cp, switches);
 		    done (1);
 		case UNKWNSW: 
-		    adios (NULL, "-%s unknown", cp);
+		    die("-%s unknown", cp);
 
 		case HELPSW: 
 		    snprintf (buf, sizeof(buf), "%s [switches] [file]", invo_name);
@@ -116,17 +132,17 @@ main (int argc, char **argv)
 
 		case DFOLDSW: 
 		    if (dfolder)
-			adios (NULL, "only one draft folder at a time!");
+			die("only one draft folder at a time!");
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    dfolder = path (*cp == '+' || *cp == '@' ? cp + 1 : cp,
 			    *cp != '@' ? TFOLDER : TSUBCWF);
 		    continue;
 		case DMSGSW: 
 		    if (dmsg)
-			adios (NULL, "only one draft message at a time!");
+			die("only one draft message at a time!");
 		    if (!(dmsg = *argp++) || *dmsg == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    continue;
 		case NDFLDSW: 
 		    dfolder = NULL;
@@ -142,73 +158,60 @@ main (int argc, char **argv)
 		case MTSSW:
 		    vec[vecp++] = --cp;
 		    if (!(cp = *argp++) || *cp == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+			die("missing argument to %s", argp[-2]);
 		    vec[vecp++] = cp;
 		    continue;
 	    }
 	}
 	if (msg)
-	    adios (NULL, "only one draft at a time!");
-	else
-	    vec[vecp++] = msg = cp;
+	    die("only one draft at a time!");
+        vec[vecp++] = msg = cp;
     }
 
     /* allow Aliasfile: profile entry */
     if ((cp = context_find ("Aliasfile"))) {
-	char *dp = NULL;
-
-	for (ap = brkstring(dp = mh_xstrdup(cp), " ", "\n"); ap && *ap; ap++) {
+	for (ap = brkstring(mh_xstrdup(cp), " ", "\n"); ap && *ap; ap++) {
 	    vec[vecp++] = "-alias";
 	    vec[vecp++] = *ap;
 	}
     }
 
     if (msg == NULL) {
-	    cp  = getcpy (m_draft (dfolder, dmsg, 1, &isdf));
+	    cp  = mh_xstrdup(m_draft(dfolder, dmsg, 1, &isdf));
 	msg = vec[vecp++] = cp;
     }
-    if ((cp = getenv ("mhdist"))
-	    && *cp
-	    && (distsw = atoi (cp))
-	    && (cp = getenv ("mhaltmsg"))
-	    && *cp) {
+
+    distsw = (cp = getenv("mhdist")) && *cp && atoi(cp) &&
+        (cp = getenv("mhaltmsg")) && *cp;
+    if (distsw) {
 	if (distout (msg, cp, backup) == NOTOK)
 	    done (1);
 	vec[vecp++] = "-dist";
-	distsw++;
     }
     vec[vecp] = NULL;
 
-    closefds (3);
-
     if (distsw) {
-	for (i = 0; (child_id = fork()) == NOTOK && i < 5; i++)
-	    sleep (5);
+        if ((child_id = fork()) == -1)
+            adios("fork", "failed:");
+
+        if (child_id) {
+            SIGNAL (SIGHUP, SIG_IGN);
+            SIGNAL (SIGINT, SIG_IGN);
+            SIGNAL (SIGQUIT, SIG_IGN);
+            SIGNAL (SIGTERM, SIG_IGN);
+
+            status = pidwait(child_id, OK);
+
+            (void) m_unlink (msg);
+            if (rename (backup, msg) == -1)
+                adios (msg, "unable to rename %s to", backup);
+            done (status);
+        }
     }
 
-    switch (distsw ? child_id : OK) {
-	case NOTOK:
-	    inform("unable to fork, so checking directly...");
-	    /* FALLTHRU */
-	case OK:
-	    execvp (postproc, vec);
-	    fprintf (stderr, "unable to exec ");
-	    perror (postproc);
-	    _exit (-1);
-
-	default:
-	    SIGNAL (SIGHUP, SIG_IGN);
-	    SIGNAL (SIGINT, SIG_IGN);
-	    SIGNAL (SIGQUIT, SIG_IGN);
-	    SIGNAL (SIGTERM, SIG_IGN);
-
-	    status = pidwait(child_id, OK);
-
-	    (void) m_unlink (msg);
-	    if (rename (backup, msg) == NOTOK)
-		adios (msg, "unable to rename %s to", backup);
-	    done (status);
-    }
-
-    return 0;  /* dead code to satisfy the compiler */
+    /* Either the child, or no fork occurred. */
+    execvp (postproc, vec);
+    fprintf (stderr, "unable to exec ");
+    perror (postproc);
+    _exit(1);
 }
